#!/bin/sh
# File: drtm2d-vert.sh
# 1st argument will be the value of macro ${m}, which is the name of velocity model vel${m}.mig.H
# Script file to parallelly compute RTM image using FK decomposition techniques for vertical components (computing every ${jsx} shots along x-axis at the same time) 
# Input file: "rtm${m}in.H" (interpolated shot gathers)
# axes: 1=rec_z, 2=rec_x, 3=time, 4=sou_x, 5=sou_z
# Output file: "rtm${m}out.H"
# axes: 1=z, 2=x, 3=Undefined

###############################
### How to use this script in makefile???:
# drtmout$(m).H drtmout$(m).SdownRdown.H drtmout$(m).SupRup.H drtmout$(m).SdownRup.H drtmout$(m).SupRdown.H: inputdata$(m).H velfile.H soufile.H Par/drtm0.vel.P ./Bin/crtm2d_program.x ./Bin/Fkdecomp2D_program.x ./Bin/Padntape_program.x
#       sh drtm2d-vert.sh '$(m)' 'inputdata$(m).H' 'drtmout$(m).H' 'drtmout$(m).SdownRdown.H' 'drtmout$(m).SupRup.H' 'drtmout$(m).SdownRup.H' 'drtmout$(m).SupRdown.H' 'conventional_rtm2d_program.x' 'Fkdecomp2d_program.x' 'Padntape_beforeFFT.x' 'vel.mig.H' 'soufile' 'parfile' 'jsx=4' 'tmpdir=.' 'FKSdownOUT.H' 'FKSupOUT.H' 'FKRdownOUT.H' 'FKRupOUT.H' 'getonlyrealpart' 'decompicon'

###############################

###############################
## Note that parfile for DRTM definitely contains all parameters for calling the conventional RTM program which is able to produce all wavefields at every time steps.
## In addition, this parfile must contain parameters for Fk-decomposition and Padding as follows:
#
# For Padntape program we need:
#              beg1, beg2, extend(optional), taper(optional)
#              [ I might suggest to use "beg1=50 beg2=50 extend=2 taper=0.01" ]
#              n1out,n2out will already be computed in this script.
#
# For Fkdecomp2d program, we always use dim=2 axis1=-1 axis2=1 we need:
#              filter(optional), phi(optional,available if using filter=2), get_filter, get_fkdataout
#              [ I might suggest to use "filter=2 phi=10" ]
#        NOTE that if get_*=y, only those auxiliary data from the 1st shot will be produced
#
# For selecting types of decomposed RTM output:
#              getonlyrealpart = 1, default    # the output is real-valued (only real part of complex-valued output)
#              getonlyrealpart = 0             # the output is complex-valued
#
# For selecting imaging condition for decomposed RTM (must be declared):
#              decompicon = 1          # Real part is the correlation of real parts, where Imag part is the correlation of imag parts
#              decompicon = 2          # taking complex conjugate for dsouwvfld in correlation, so the Imaginary part involves the image formation
###############################

## use double quatation marks "$NAME" to view the variable's contents
## single quatation marks '$NAME' does not show the contents of NAME 

echo "calling $0 script"
echo "reading parameters..."

m=$1         #suffix names for showong identity of processing
echo m=$m

IN=$2       #input file /gz,gx,t,sx,sz/ with interpreted gx
if [ -z $IN ];then echo 'IN=$2 does not exist';exit;fi #test for a zero-length string
echo IN=$IN

OUT=$3       #conventional rtm image file /z,x/
if [ -z $OUT ];then echo 'OUT=$3 does not exist';exit;fi
echo OUT=$OUT

OUTSdownRdown=$4      # rtm image from SdownRdown
if [ -z $OUTSdownRdown ];then echo 'OUTSdownRdown=$4 does not exist';exit;fi
echo OUTSdownRdown=$OUTSdownRdown

OUTSupRup=$5
if [ -z $OUTSupRup ];then echo 'OUTSupRup=$5 does not exist';exit;fi
echo OUTSupRup=$OUTSupRup

OUTSdownRup=$6
if [ -z $OUTSdownRup ];then echo 'OUTSdownRup=$6 does not exist';exit;fi
echo OUTSdownRup=$OUTSdownRup

OUTSupRdown=$7
if [ -z $OUTSupRdown ];then echo 'OUTSupRdown=$7 does not exist';exit;fi
echo OUTSupRdown=$OUTSupRdown

RTMPROG=$8 # rtm program
if [ -z $RTMPROG ];then echo 'RTMPROG=$8 does not exist';exit;fi
echo RTMPROG=$RTMPROG

FKDECOMPROG=$9 # Fkdecomposition program
if [ -z $FKDECOMPROG ];then echo 'FKDECOMPROG=$9 does not exist';exit;fi
echo FKDECOMPROG=$FKDECOMPROG

PADNTAPEPROG=${10} # Pad and tape prgram before doing FFT
if [ -z $PADNTAPEPROG ];then echo 'PADNTAPEPROG=$8{10} does not exist';exit;fi
echo PADNTAPEPROG=$PADNTAPEPROG

VELFILE=${11}   #velocity file
if [ -z $VELFILE ];then echo 'VELFILE=${11} does not exist';exit;fi
echo VELFILE=$VELFILE

SOUFILE=${12}   #source wavelet file
if [ -z $SOUFILE ];then echo 'SOUFILE=${12} does not exist';exit;fi
echo SOUFILE=$SOUFILE

PARFILE=${13}   #parameter file for RTM program
if [ -z $PARFILE ];then echo 'PARFILE=${13} does not exist';exit;fi
echo PARFILE=$PARFILE

################################
## Read parameters from parfile to this shell script
echo "Reading parameters from the paramter file"
. ${PARFILE}
################################

jsx=${14}       #increment between shots along x-axis  (the number of shot record to be muted in the same time)
if [ -z $jsx ];then jsx=4;fi
echo jsx=$jsx

TEMPDIR=${15}   # temp file to store temporary data
if [ -z $TEMPDIR ];then TEMPDIR=.;fi
echo TEMPDIR=$TEMPDIR

##################
## Optional snapshot files and filtering results in FK domain
##################
# Sdown in FK
FKSdownOUT=${16}  # Sdown file in FK domain
if [ -z $FKSdownOUT ];then FKSdownOUT=FKSdown${m}.H;fi
echo FKSdownOUT=$FKSdownOUT
# Sup in FK
FKSupOUT=${17}  # Sup file in FK domain
if [ -z $FKSupOUT ];then FKSupOUT=FKSdown${m}.H;fi
echo FKSupOUT=$FKSupOUT
# Rdown in FK
FKRdownOUT=${18}  # Rdown file in FK domain
if [ -z $FKRdownOUT ];then FKRdownOUT=FKRdown${m}.H;fi
echo FKRdownOUT=$FKRdownOUT
# Rup in FK
FKRupOUT=${19}  # Rup file in FK domain
if [ -z $FKRupOUT ];then FKRupOUT=FKRup${m}.H;fi
echo FKRupOUT=$FKRupOUT

##################
## Optional parameters for selecting output types of DRTM and imaging condition
## if these parameters exist on the command line, the script will ignore the same parameters already defined in parfile
##################
if [ ! -z ${20} ]; then # if ${20} is a nonzero string (not blank or just free-space string), new getonlyrealpart will be assigned
    getonlyrealpart=${20}   # [0] the output is complex-valued, [1] the output is real-valued (only real part of complex-valued output)
fi

if [ ! -z ${21} ]; then # if ${21} is a nonzero string (not blank or just free-space string), new decompicon will be assigned
    decompicon=${21}   # [1] Real part is the correlation of real parts, where Imag part is the correlation of imag parts
                       # [2] taking complex conjugate for dsouwvfld in correlation, so the Imaginary part involves the image formation
fi


echo "reading parameters complete..."

##########################################
## IGNORE HERE
##########################################
#m=$1 # name of velocity model is the 1st argument when calling this script
###############################
#jsx=4  # increment between shots along x-axis  (the intershots will be RTMed at the same time)
#tdelay=8 # delayed time in seconds
###############################
#IN=rtm${m}in.H
## output history files
#OUT=rtm${m}out.H
#OUTSdownRdown=rtm${m}out.SdownRdown.H
#OUTSupRup=rtm${m}out.SupRup.H
#OUTSdownRup=rtm${m}out.SdownRup.H
#OUTSupRdown=rtm${m}out.SupRdown.H
#SOUSNAPOUT=sousnap${m}.H
#RECSNAPOUT=recsnap${m}.H
#SNAPOUT=reflsnap${m}.H
#PARFILE=./Par/rtm0.vel${m}.P
#RTMPROG=./Bin/crtm2d_program.x
#FKDECOMPROG=./Bin/Fkdecomp2D_program.x
#TEMPDIR=./temp
#VELFILE=vel${m}.mig.H
#SOUFILE=sou.HH
##########################################

echo "Start program to compute RTM image in vel1 model..."
echo "Reading shot info..."
nsx=`Get < $IN parform=n n4`
dsx=`Get < $IN parform=n d4`
osx=`Get < $IN parform=n o4`
echo "nsx,dsx,osx,jsx" $nsx $dsx $osx $jsx
nt=`Get < $IN parform=n n3`
dt=`Get < $IN parform=n d3`
ot=`Get < $IN parform=n o3`
echo "nt,dt,ot" $nt $dt $ot
ngx=`Get < $IN parform=n n1`
dgx=`Get < $IN parform=n d1`
ogx=`Get < $IN parform=n o1`
echo "ngx,dgx,ogx" $ngx $dgx $ogx
nz=`Get < $VELFILE parform=n n1`
dz=`Get < $VELFILE parform=n d1`
oz=`Get < $VELFILE parform=n o1`
echo "nz,dz,oz" $nz $dz $oz
nx=`Get < $VELFILE parform=n n2`
dx=`Get < $VELFILE parform=n d2`
ox=`Get < $VELFILE parform=n o2`
echo "nx,dx,ox" $nx $dx $ox

######################
## reorganizing some parameters
###### wavefield time step should equal to jt_image step
jtsnaprtm=${jt_image}
echo "jt_image=${jt_image}, and jtsnaprtm=${jtsnaprtm}"

echo "Input parameters for Padntape program:"
echo ".... beg1=${beg1}, beg2=${beg2}, extend=${extend} taper=${taper}"
PADTAPE="${PADNTAPEPROG} n1out=$(((${nt}-1)/${jtsnaprtm}+1+${beg1}+${beg1})) n2out=$((${nz}+${beg2}+${beg2})) beg1=${beg1} beg2=${beg2} extend=${extend} taper=${taper} esize=4"
UNPADTAPE="Window n1=$(((${nt}-1)/${jtsnaprtm}+1)) f1=${beg1} n2=${nz} f2=${beg2}"
UNPADTAPEecho="o1=${ot} o2=${oz}"  # synchronize origin of unpadded vertical wvfld

echo "Input parameters for Fkdecomp2d program:"
echo ".... dim=2 axis1=-1 axis2=1 filter=${filter} phi=${phi} get_filter=${get_filter} get_fkdataout=${get_fkdataout}"
filter2d="filter=${filter} phi=${phi} get_filter=${get_filter} get_fkdataout=${get_fkdataout} verb=${verb}"

###### Feature of decomposed RTM output is only REAL part (DEFAULT) after applying imaging condition or complex-valued file.
echo "getonlyrealpart=${getonlyrealpart}"
if [ ${getonlyrealpart} -eq 0 ]
then
    getonlyrealpart=" "
else
    getonlyrealpart="| Real"
fi

###### Imaging condition for decomposed RTM #####
echo "Declare decomposed RTM imaging condition where file1=dsouwvfld and file2=drecwvfld:"
if [ ${decompicon} -eq 1 ]
then 
    echo "Real part is the correlation of real parts, where Imag part is the correlation of imag parts"
    decompicon="exp_real='file1.re*file2.re' exp_imag='file1.im*file2.im'"
elif [ ${decompicon} -eq 2 ]
then
    #taking complex conjugate for dsouwvfld in correlation, so the Im part involves the image formation
    echo "taking complex conjugate for dsouwvfld in correlation, so the Im part involves the image formation"
    decompicon="exp_real='file1.re*file2.re+file1.im*file2.im' exp_imag='file1.re*file2.im-file1.im*file2.re'"
else
    echo "decompicon=${decompicon} is invalid"
    exit
fi
echo "decompicon=$decompicon"

#######
####################################
## Parameters for selecting decomposed RTM components [0 or 1] read from parfile
####################################
echo "get_DRTM_SdownRdown=${get_DRTM_SdownRdown}"
echo "get_DRTM_SupRup=${get_DRTM_SupRup}"
echo "get_DRTM_SdownRup=${get_DRTM_SdownRup}"
echo "get_DRTM_SupRdown=${get_DRTM_SupRdown}"


###############
## Old definition
##PADTAPE="${PADNTAPEPROG} n1out=$(((${nt}-1)/${jtsnaprtm}+1+100)) n2out=$((${nz}+100)) beg1=50 beg2=50 extend=2 taper=0.01"
##UNPADTAPE="Window n1=$(((${nt}-1)/${jtsnaprtm}+1)) f1=50 n2=${nz} f2=50"
##filter2d="filter=2 phi=10"
##	<$< ${PADTAPE} | Rtoc | $B/Fkdecomp2d_program.x dim=2 axis1=-1 axis2=1 flag=-1 ${filter2d} | Real | ${UNPADTAPE} |Grey ${GreyFFT}|Tube&
##
##  Decomposed RTM imaging condition where file1=decomposed.souwvfld and file2=decomposed.recwvfld .
##  exp_real='file1.re*file2.re+file1.im*file2.im' exp_imag='file1.re*file2.im-file1.im*file2.re'
######

# parameters for conventional RTM 
TEXT0=tmp0rtm${m}.txt
TEXT1=tmp1rtm${m}.txt
TEXT2=tmp2rtm${m}.txt
TEXT3=tmp3rtm${m}.txt
TEXT4=tmp4rtm${m}.txt
TEXT5=tmp5rtm${m}.txt
TEXT6=tmp6rtm${m}.txt
OUT1=rtm${m}out1.H
OUT2=rtm${m}out2.H
OUT3=rtm${m}out3.H
OUT4=rtm${m}out4.H
OUT5=rtm${m}out5.H
OUT6=rtm${m}out6.H

# parameters for RTM from SdownRdown
TEXTSdownRdown0=tmp0rtm${m}SdownRdown.txt
TEXTSdownRdown1=tmp1rtm${m}SdownRdown.txt
TEXTSdownRdown2=tmp2rtm${m}SdownRdown.txt
TEXTSdownRdown3=tmp3rtm${m}SdownRdown.txt
TEXTSdownRdown4=tmp4rtm${m}SdownRdown.txt
TEXTSdownRdown5=tmp5rtm${m}SdownRdown.txt
TEXTSdownRdown6=tmp6rtm${m}SdownRdown.txt
OUTSdownRdown1=rtm${m}out1SdownRdown.H
OUTSdownRdown2=rtm${m}out2SdownRdown.H
OUTSdownRdown3=rtm${m}out3SdownRdown.H
OUTSdownRdown4=rtm${m}out4SdownRdown.H
OUTSdownRdown5=rtm${m}out5SdownRdown.H
OUTSdownRdown6=rtm${m}out6SdownRdown.H

# parameters for RTM from SupRup
TEXTSupRup0=tmp0rtm${m}SupRup.txt
TEXTSupRup1=tmp1rtm${m}SupRup.txt
TEXTSupRup2=tmp2rtm${m}SupRup.txt
TEXTSupRup3=tmp3rtm${m}SupRup.txt
TEXTSupRup4=tmp4rtm${m}SupRup.txt
TEXTSupRup5=tmp5rtm${m}SupRup.txt
TEXTSupRup6=tmp6rtm${m}SupRup.txt
OUTSupRup1=rtm${m}out1SupRup.H
OUTSupRup2=rtm${m}out2SupRup.H
OUTSupRup3=rtm${m}out3SupRup.H
OUTSupRup4=rtm${m}out4SupRup.H
OUTSupRup5=rtm${m}out5SupRup.H
OUTSupRup6=rtm${m}out6SupRup.H

# parameters for RTM from SdownRup
TEXTSdownRup0=tmp0rtm${m}SdownRup.txt
TEXTSdownRup1=tmp1rtm${m}SdownRup.txt
TEXTSdownRup2=tmp2rtm${m}SdownRup.txt
TEXTSdownRup3=tmp3rtm${m}SdownRup.txt
TEXTSdownRup4=tmp4rtm${m}SdownRup.txt
TEXTSdownRup5=tmp5rtm${m}SdownRup.txt
TEXTSdownRup6=tmp6rtm${m}SdownRup.txt
OUTSdownRup1=rtm${m}out1SdownRup.H
OUTSdownRup2=rtm${m}out2SdownRup.H
OUTSdownRup3=rtm${m}out3SdownRup.H
OUTSdownRup4=rtm${m}out4SdownRup.H
OUTSdownRup5=rtm${m}out5SdownRup.H
OUTSdownRup6=rtm${m}out6SdownRup.H

# parameters for RTM from SupRdown
TEXTSupRdown0=tmp0rtm${m}SupRdown.txt
TEXTSupRdown1=tmp1rtm${m}SupRdown.txt
TEXTSupRdown2=tmp2rtm${m}SupRdown.txt
TEXTSupRdown3=tmp3rtm${m}SupRdown.txt
TEXTSupRdown4=tmp4rtm${m}SupRdown.txt
TEXTSupRdown5=tmp5rtm${m}SupRdown.txt
TEXTSupRdown6=tmp6rtm${m}SupRdown.txt
OUTSupRdown1=rtm${m}out1SupRdown.H
OUTSupRdown2=rtm${m}out2SupRdown.H
OUTSupRdown3=rtm${m}out3SupRdown.H
OUTSupRdown4=rtm${m}out4SupRdown.H
OUTSupRdown5=rtm${m}out5SupRdown.H
OUTSupRdown6=rtm${m}out6SupRdown.H

# remove files or text files before launching programs
Rm ${TEMPDIR}/rtm${m}imageshot*.H
Rm ${TEMPDIR}/rtm${m}shotrecord*.H
Rm ${OUT} ${OUTSdownRdown} ${OUTSupRup} ${OUTSdownRup} ${OUTSupRdown}
Rm ${OUT1} ${OUTSdownRdown1} ${OUTSupRup1} ${OUTSdownRup1} ${OUTSupRdown1}
Rm ${OUT2} ${OUTSdownRdown2} ${OUTSupRup2} ${OUTSdownRup2} ${OUTSupRdown2}
Rm ${OUT3} ${OUTSdownRdown3} ${OUTSupRup3} ${OUTSdownRup3} ${OUTSupRdown3}
Rm ${OUT4} ${OUTSdownRdown4} ${OUTSupRup4} ${OUTSdownRup4} ${OUTSupRdown4}
Rm ${OUT5} ${OUTSdownRdown5} ${OUTSupRup5} ${OUTSdownRup5} ${OUTSupRdown5}
Rm ${OUT6} ${OUTSdownRdown6} ${OUTSupRup6} ${OUTSdownRup6} ${OUTSupRdown6}
rm -f ${TEXT0} ${TEXTSdownRdown0} ${TEXTSupRup0} ${TEXTSdownRup0} ${TEXTSupRdown0}
rm -f ${TEXT1} ${TEXTSdownRdown1} ${TEXTSupRup1} ${TEXTSdownRup1} ${TEXTSupRdown1}
rm -f ${TEXT2} ${TEXTSdownRdown2} ${TEXTSupRup2} ${TEXTSdownRup2} ${TEXTSupRdown2}
rm -f ${TEXT3} ${TEXTSdownRdown3} ${TEXTSupRup3} ${TEXTSdownRup3} ${TEXTSupRdown3}
rm -f ${TEXT4} ${TEXTSdownRdown4} ${TEXTSupRup4} ${TEXTSdownRup4} ${TEXTSupRdown4}
rm -f ${TEXT5} ${TEXTSdownRdown5} ${TEXTSupRup5} ${TEXTSdownRup5} ${TEXTSupRdown5}
rm -f ${TEXT6} ${TEXTSdownRdown6} ${TEXTSupRup6} ${TEXTSdownRup6} ${TEXTSupRdown6}

#<${IN} Transp plane=13 | Transp plane=34 > ${OUT}  # for windowing purpose
#In ${OUT}


###########

echo "5555555555555555555555555555555555555555555555555555555555555555555555"
echo "The script is computing decomposed RTM images of vel${m} from ${jsx} shots at the same time."
echo "5555555555555555555555555555555555555555555555555555555555555555555555"
for((isx=1; $isx<=$nsx; isx=$isx+$jsx))
do # each iteration will do rtm for ${jsx} shots at the same time

    ##############
    ### do Windowing and Transposing input data to enter a conventional RTM program
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]
	then
	    echo "************* Windowing and Transposing SHOT$((${isx}+$k)) *****************"
	    echo "isx,osx,sx:" $(($isx+$k)) $osx `echo $osx+$(($isx+$k-1))*$dsx | bc`
	    # This set of command will consume 2 cpu's at the same time; one might guess tha the simutaneous loops should not be greater than half of #cpu
	    # BUT computing more than half of #cpu simutaneouly is significantly more effective than limiting ourselves to just 4 simutaneous loops.
	    # SO, what I did here is the best way. No need to window and transpose the whole shots first with 4-simutaneous process before entering RTM with ${jsx}-simutaneous process
	    # Note that calling only-Window commands simutaneously will share the whole processes in a single CPU. BUT calling only-Transp commands simutaneously will separate every single process to a different CPU. SO, if we make Window-Transp dependent to each other, it will behave like only-Transp. That's why I use .... Window | Transp .....
	    <${IN} Window n4=1 f4=$((${isx}+${k}-1)) | Transp plane=12 reshape=2,3,5 > ${TEMPDIR}/drtm${m}shotrecord$((${isx}+${k})).H &
	fi
    done
    # wait untill all background processes to finish
    wait
 
    #############
    #### do conventional RTM imaging and get source and receiver wavefields
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do # run rtm program within each combination of ${jsx} shots
	if [ $((${isx}+${k})) -le ${nsx} ]
	then
	    echo "************* Conventional RTMing SHOT$((${isx}+$k))VEL${m} *****************"
	    echo "**** and getting source & receiver wavefields at all sufficient time steps***"
	    # In the middle of each iteration we need to run the rtm program as background.
            # Every shot gives us snapshot file
	    <${VELFILE} ${RTMPROG} par=${PARFILE} jsnap=${jtsnaprtm} get_snap=0 get_sousnap=1 get_recsnap=1 source=${SOUFILE} receiver=${TEMPDIR}/drtm${m}shotrecord$((${isx}+${k})).H >${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H sousnap=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).H recsnap=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).H &

	    if [ $((${isx}+${k})) -le 20 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT1}
	    elif [ $((${isx}+${k})) -le 40 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT2}
	    elif [ $((${isx}+${k})) -le 60 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT3}
	    elif [ $((${isx}+${k})) -le 80 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT4}
	    elif [ $((${isx}+${k})) -le 100 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT5}
	    else
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT6}
	    fi
	fi
    done
    # wait untill all background processes to finish
    wait

    #############
    #### remove shot records that have been used.
    echo "removing shot records that have been used..."
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/drtm${m}shotrecord$((${isx}+$k)).H
	fi
    done

#############################################################
### after this line it is all about decomposition and DRTM
#############################################################
if [ ${get_DRTM_SdownRdown} -eq 1 -o ${get_DRTM_SupRup} -eq 1 -o ${get_DRTM_SdownRup} -eq 1 -o ${get_DRTM_SupRdown} -eq 1 ]; then

    ##################################################
    #####***************************************######
    #### decompose source wavefield
    ##################################################
    echo "decomposing source wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
    #### Sorting the source wavefield ready for decomposition
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).H Transp plane=12 reshape=2,3 >${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.H &
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed source wavefields before reordered. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).H
	fi
    done

    #####################################
    ## Preparing input for decomposing source wavefield Sup&Sdown
    #####################################
    # convert from real to complex input for Sup and Sdown
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.H ${PADTAPE} | Rtoc >${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.H &
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed source wavefields that have been used to separate wavefields. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.H
	fi
    done

    #####################################
    #### select upgoing source wavefield Sup if needed
    #####################################
    if [ ${get_DRTM_SupRup} -eq 1 -o ${get_DRTM_SupRdown} -eq 1 ]; then
	echo "Getting upgoing source wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp2d program for Sup
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=-1 ${filter2d} >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H fkdataout=${FKSupOUT} &
		else
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=-1 ${filter2d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H ${UNPADTAPE} > ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H &
	    fi
	done
        # wait untill all background processes to finish
	wait
	
        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpSup
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H
	    fi
	done
	
    fi
    #####################################

    #####################################
    #### select downgoing source wavefield Sdown if needed
    #####################################
    if [ ${get_DRTM_SdownRup} -eq 1 -o ${get_DRTM_SdownRdown} -eq 1 ]; then
	echo "Getting downgoing source wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp2d program for Sdown
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=1 ${filter2d} >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H fkdataout=${FKSdownOUT} &
		else
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=1 ${filter2d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H ${UNPADTAPE} > ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpSdown
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H
	    fi
	done

    fi

    ##### remove source wvfld that had been prepared for decomposition
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do #tmpSource
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.H
	fi
    done

    #####################################


    ##################################################
    #####***************************************######
    #### decompose receiver wavefield
    ##################################################
    echo "decomposing receiver wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
    #### Sorting the receiver wavefield ready for decomposition
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).H Transp plane=12 reshape=2,3 >${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.H &
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed receiver wavefields before reordered. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).H
	fi
    done

    #####################################
    ## Preparing input for decomposing receiver wavefield Rup&Rdown
    #####################################
    # convert from real to complex input for Rup and Rdown
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.H ${PADTAPE} | Rtoc >${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.H &
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed receiver wavefields that have been used to separate wavefields. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.H
	fi
    done

    #####################################
    #### select upgoing receiver wavefield Rup if needed
    #####################################
    if [ ${get_DRTM_SdownRup} -eq 1 -o ${get_DRTM_SupRup} -eq 1 ]; then
	echo "Getting upgoing receiver wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp2d program for Rup
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=-1 ${filter2d} >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H fkdataout=${FKRupOUT} &
		else
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=-1 ${filter2d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H ${UNPADTAPE} > ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpRup
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H
	    fi
	done

    fi
    #####################################

    #####################################
    #### select downgoing receiver wavefield Rdown
    #####################################
    if [ ${get_DRTM_SdownRdown} -eq 1 -o ${get_DRTM_SupRdown} -eq 1 ]; then
	echo "Getting downgoing receiver wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp2d program for Rdown
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
        do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=1 ${filter2d} >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H fkdataout=${FKRdownOUT} &
		else
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=1 ${filter2d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H ${UNPADTAPE} > ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpRdown
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H
	    fi
	done

    fi

    ##### remove receiver wvfld that had been prepared for decomposition

    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do #tmpReceiver
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.H
	fi
    done

    #####################################


######################
######################
    
     #########################################################################################
    #### apply RTM imaging condition to all combinations of source and receiver wavefields ####
     #########################################################################################
    
         # Math file1=souwvfld$*.vert.fkffto.down.H file2=recwvfld$*.vert.fkffto.down.H exp_real='file1.re*file2.re+file1.im*file2.im' exp_imag='file1.re*file2.im-file1.im*file2.re' >$@
         # <$< Transp plane=12 | Stack axes=2 | Real >$@ # sum over time
    ##########################################
    #### Getting RTM image from SdownRdown components
    ##########################################
    if [ ${get_DRTM_SdownRdown} -eq 1 ]; then
	echo "RTM from SdownRdown SHOT${isx} to SHOT$((${isx}+${jsx}-1)).."
        # RTM from Sdown*Rdown
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).vert.SdownRdown.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).vert.SdownRdown.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SdownRdown.H &
	        # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SdownRdown.H" >> ${TEXTSdownRdown1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SdownRdown.H" >> ${TEXTSdownRdown2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SdownRdown.H" >> ${TEXTSdownRdown3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SdownRdown.H" >> ${TEXTSdownRdown4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SdownRdown.H" >> ${TEXTSdownRdown5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SdownRdown.H" >> ${TEXTSdownRdown6}
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SdownRdown
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).vert.SdownRdown.H
	    fi
	done

    fi
    ##########################################

    ##########################################
    ## RTM from Sup*Rup
    ##########################################
    if [ ${get_DRTM_SupRup} -eq 1 ]; then
	echo "RTM from SupRup SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).vert.SupRup.H &
	    fi
	done
    # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).vert.SupRup.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SupRup.H &
	    # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SupRup.H" >> ${TEXTSupRup1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SupRup.H" >> ${TEXTSupRup2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SupRup.H" >> ${TEXTSupRup3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SupRup.H" >> ${TEXTSupRup4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SupRup.H" >> ${TEXTSupRup5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SupRup.H" >> ${TEXTSupRup6}
		fi
	    fi
	done
    # wait untill all background processes to finish
	wait
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SupRup
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).vert.SupRup.H
	fi
	done

    fi
    ##########################################

    ##########################################
    ### RTM from Sdown*Rup
    ##########################################
    if [ ${get_DRTM_SdownRup} -eq 1 ]; then
	echo "RTM from SdownRup SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).vert.SdownRup.H &
	    fi
	done
    # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).vert.SdownRup.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SdownRup.H &
	    # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SdownRup.H" >> ${TEXTSdownRup1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SdownRup.H" >> ${TEXTSdownRup2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SdownRup.H" >> ${TEXTSdownRup3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SdownRup.H" >> ${TEXTSdownRup4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SdownRup.H" >> ${TEXTSdownRup5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SdownRup.H" >> ${TEXTSdownRup6}
		fi
	    fi
	done
    # wait untill all background processes to finish
	wait
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SupRup
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).vert.SdownRup.H
	    fi
	done

    fi
    ##########################################

    ##########################################
    #### RTM from Sup*Rdown
    ##########################################
    if [ ${get_DRTM_SupRdown} -eq 1 ]; then
	echo "RTM from SupRdown SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).vert.SupRdown.H &
	    fi
	done
    # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).vert.SupRdown.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SupRdown.H &
	    # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SupRdown.H" >> ${TEXTSupRdown1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SupRdown.H" >> ${TEXTSupRdown2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SupRdown.H" >> ${TEXTSupRdown3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SupRdown.H" >> ${TEXTSupRdown4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SupRdown.H" >> ${TEXTSupRdown5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).vert.SupRdown.H" >> ${TEXTSupRdown6}
		fi
	    fi
	done
    # wait untill all background processes to finish
	wait
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SupRup
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).vert.SupRdown.H
	    fi
	done

    fi
    ##########################################

    #############
    #### remove source wavefield and receiver wavefield that have been used to compute RTM images. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H &
	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H &
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.up.H &
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).vert.down.H
	fi
    done

fi
#############################################################
### above this line it is all about decomposition and DRTM
#############################################################

    #############
    #### remove all source and receiver wavefields records that have been used. 
    #for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    #do
    #	if [ $((${isx}+${k})) -le ${nsx} ]; then
    #	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).vert*.H
    #	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).vert*.H
    #	fi
    #done

done

###########################################
### Combine the conventional RTM images
###########################################
echo "================================="
echo "combining conventional RTM images"
echo "================================="
if [ -f ${TEXT1} ]; then
    rtmlist1=`cat ${TEXT1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUT1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT1}
    echo ${OUT1} >> ${TEXT0}
    runloop=1
fi
if [ -f ${TEXT2} ]; then
    rtmlist2=`cat ${TEXT2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUT2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT2}
    echo ${OUT2} >> ${TEXT0}
    runloop=2
fi
if [ -f ${TEXT3} ]; then
    rtmlist3=`cat ${TEXT3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUT3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT3}
    echo ${OUT3} >> ${TEXT0}
    runloop=3
fi
if [ -f ${TEXT4} ]; then
    rtmlist4=`cat ${TEXT4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUT4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT4}
    echo ${OUT4} >> ${TEXT0}
    runloop=4
fi
if [ -f ${TEXT5} ]; then
    rtmlist5=`cat ${TEXT5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUT5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT5}
    echo ${OUT5} >> ${TEXT0}
    runloop=5
fi
if [ -f ${TEXT6} ]; then
    rtmlist6=`cat ${TEXT6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUT6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT6}
    echo ${OUT6} >> ${TEXT0}
    runloop=6
fi
#######################
## generate conventional RTM images
#######################
if [ -f ${TEXT0} ]; then
    rtmlist0=`cat ${TEXT0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23|Stack axes=2 > ${OUT}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUT1} ${OUT2} ${OUT3} ${OUT4} ${OUT5} ${OUT6}
    rm -f ${TEXT0} ${TEXT1} ${TEXT2} ${TEXT3} ${TEXT4} ${TEXT5} ${TEXT6}
fi

###########################################
### Combine the RTM images from SdownRdown
###########################################
echo "===================================="
echo "combining RTM images from SdownRdown"
echo "===================================="
if [ -f ${TEXTSdownRdown1} ]; then
    rtmlist1=`cat ${TEXTSdownRdown1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSdownRdown1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownRdown1}
    echo ${OUTSdownRdown1} >> ${TEXTSdownRdown0}
    runloop=1
fi
if [ -f ${TEXTSdownRdown2} ]; then
    rtmlist2=`cat ${TEXTSdownRdown2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSdownRdown2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownRdown2}
    echo ${OUTSdownRdown2} >> ${TEXTSdownRdown0}
    runloop=2
fi
if [ -f ${TEXTSdownRdown3} ]; then
    rtmlist3=`cat ${TEXTSdownRdown3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSdownRdown3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownRdown3}
    echo ${OUTSdownRdown3} >> ${TEXTSdownRdown0}
    runloop=3
fi
if [ -f ${TEXTSdownRdown4} ]; then
    rtmlist4=`cat ${TEXTSdownRdown4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSdownRdown4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownRdown4}
    echo ${OUTSdownRdown4} >> ${TEXTSdownRdown0}
    runloop=4
fi
if [ -f ${TEXTSdownRdown5} ]; then
    rtmlist5=`cat ${TEXTSdownRdown5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSdownRdown5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownRdown5}
    echo ${OUTSdownRdown5} >> ${TEXTSdownRdown0}
    runloop=5
fi
if [ -f ${TEXTSdownRdown6} ]; then
    rtmlist6=`cat ${TEXTSdownRdown6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSdownRdown6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownRdown6}
    echo ${OUTSdownRdown6} >> ${TEXTSdownRdown0}
    runloop=6
fi

#######################
## generate DRTM images from SdownRdown
#######################
if [ -f ${TEXTSdownRdown0} ]; then
    rtmlist0=`cat ${TEXTSdownRdown0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2 > ${OUTSdownRdown}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownRdown}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSdownRdown1} ${OUTSdownRdown2} ${OUTSdownRdown3} ${OUTSdownRdown4} ${OUTSdownRdown5} ${OUTSdownRdown6}
    rm -f ${TEXTSdownRdown0} ${TEXTSdownRdown1} ${TEXTSdownRdown2} ${TEXTSdownRdown3} ${TEXTSdownRdown4} ${TEXTSdownRdown5} ${TEXTSdownRdown6}
fi


###########################################
### Combine the RTM images from SupRup
###########################################
echo "===================================="
echo "combining RTM images from SupRup"
echo "===================================="
if [ -f ${TEXTSupRup1} ]; then
    rtmlist1=`cat ${TEXTSupRup1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSupRup1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupRup1}
    echo ${OUTSupRup1} >> ${TEXTSupRup0}
    runloop=1
fi
if [ -f ${TEXTSupRup2} ]; then
    rtmlist2=`cat ${TEXTSupRup2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSupRup2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupRup2}
    echo ${OUTSupRup2} >> ${TEXTSupRup0}
    runloop=2
fi
if [ -f ${TEXTSupRup3} ]; then
    rtmlist3=`cat ${TEXTSupRup3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSupRup3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupRup3}
    echo ${OUTSupRup3} >> ${TEXTSupRup0}
    runloop=3
fi
if [ -f ${TEXTSupRup4} ]; then
    rtmlist4=`cat ${TEXTSupRup4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSupRup4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupRup4}
    echo ${OUTSupRup4} >> ${TEXTSupRup0}
    runloop=4
fi
if [ -f ${TEXTSupRup5} ]; then
    rtmlist5=`cat ${TEXTSupRup5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSupRup5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupRup5}
    echo ${OUTSupRup5} >> ${TEXTSupRup0}
    runloop=5
fi
if [ -f ${TEXTSupRup6} ]; then
    rtmlist6=`cat ${TEXTSupRup6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSupRup6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupRup6}
    echo ${OUTSupRup6} >> ${TEXTSupRup0}
    runloop=6
fi

#######################
## generate DRTM images from SupRup
#######################
if [ -f ${TEXTSupRup0} ]; then
    rtmlist0=`cat ${TEXTSupRup0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2 > ${OUTSupRup}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupRup}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSupRup1} ${OUTSupRup2} ${OUTSupRup3} ${OUTSupRup4} ${OUTSupRup5} ${OUTSupRup6}
    rm -f ${TEXTSupRup0} ${TEXTSupRup1} ${TEXTSupRup2} ${TEXTSupRup3} ${TEXTSupRup4} ${TEXTSupRup5} ${TEXTSupRup6}
fi

###########################################
### Combine the RTM images from SdownRup
###########################################
echo "===================================="
echo "combining RTM images from SdownRup"
echo "===================================="
if [ -f ${TEXTSdownRup1} ]; then
    rtmlist1=`cat ${TEXTSdownRup1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSdownRup1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownRup1}
    echo ${OUTSdownRup1} >> ${TEXTSdownRup0}
    runloop=1
fi
if [ -f ${TEXTSdownRup2} ]; then
    rtmlist2=`cat ${TEXTSdownRup2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSdownRup2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownRup2}
    echo ${OUTSdownRup2} >> ${TEXTSdownRup0}
    runloop=2
fi
if [ -f ${TEXTSdownRup3} ]; then
    rtmlist3=`cat ${TEXTSdownRup3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSdownRup3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownRup3}
    echo ${OUTSdownRup3} >> ${TEXTSdownRup0}
    runloop=3
fi
if [ -f ${TEXTSdownRup4} ]; then
    rtmlist4=`cat ${TEXTSdownRup4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSdownRup4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownRup4}
    echo ${OUTSdownRup4} >> ${TEXTSdownRup0}
    runloop=4
fi
if [ -f ${TEXTSdownRup5} ]; then
    rtmlist5=`cat ${TEXTSdownRup5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSdownRup5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownRup5}
    echo ${OUTSdownRup5} >> ${TEXTSdownRup0}
    runloop=5
fi
if [ -f ${TEXTSdownRup6} ]; then
    rtmlist6=`cat ${TEXTSdownRup6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSdownRup6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownRup6}
    echo ${OUTSdownRup6} >> ${TEXTSdownRup0}
    runloop=6
fi
#######################
## generate DRTM images from SdownRup
#######################
if [ -f ${TEXTSdownRup0} ]; then
    rtmlist0=`cat ${TEXTSdownRup0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2 > ${OUTSdownRup}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownRup}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSdownRup1} ${OUTSdownRup2} ${OUTSdownRup3} ${OUTSdownRup4} ${OUTSdownRup5} ${OUTSdownRup6}
    rm -f ${TEXTSdownRup0} ${TEXTSdownRup1} ${TEXTSdownRup2} ${TEXTSdownRup3} ${TEXTSdownRup4} ${TEXTSdownRup5} ${TEXTSdownRup6}
fi

###########################################
### Combine the RTM images from SupRdown
###########################################
echo "===================================="
echo "combining RTM images from SupRdown"
echo "===================================="
if [ -f ${TEXTSupRdown1} ]; then
    rtmlist1=`cat ${TEXTSupRdown1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSupRdown1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupRdown1}
    echo ${OUTSupRdown1} >> ${TEXTSupRdown0}
    runloop=1
fi
if [ -f ${TEXTSupRdown2} ]; then
    rtmlist2=`cat ${TEXTSupRdown2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSupRdown2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupRdown2}
    echo ${OUTSupRdown2} >> ${TEXTSupRdown0}
    runloop=2
fi
if [ -f ${TEXTSupRdown3} ]; then
    rtmlist3=`cat ${TEXTSupRdown3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSupRdown3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupRdown3}
    echo ${OUTSupRdown3} >> ${TEXTSupRdown0}
    runloop=3
fi
if [ -f ${TEXTSupRdown4} ]; then
    rtmlist4=`cat ${TEXTSupRdown4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSupRdown4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupRdown4}
    echo ${OUTSupRdown4} >> ${TEXTSupRdown0}
    runloop=4
fi
if [ -f ${TEXTSupRdown5} ]; then
    rtmlist5=`cat ${TEXTSupRdown5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSupRdown5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupRdown5}
    echo ${OUTSupRdown5} >> ${TEXTSupRdown0}
    runloop=5
fi
if [ -f ${TEXTSupRdown6} ]; then
    rtmlist6=`cat ${TEXTSupRdown6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSupRdown6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupRdown6}
    echo ${OUTSupRdown6} >> ${TEXTSupRdown0}
    runloop=6
fi

#######################
## generate DRTM images from SupRdown
#######################
if [ -f ${TEXTSupRdown0} ]; then
    rtmlist0=`cat ${TEXTSupRdown0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2 > ${OUTSupRdown}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupRdown}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSupRdown1} ${OUTSupRdown2} ${OUTSupRdown3} ${OUTSupRdown4} ${OUTSupRdown5} ${OUTSupRdown6}
    rm -f ${TEXTSupRdown0} ${TEXTSupRdown1} ${TEXTSupRdown2} ${TEXTSupRdown3} ${TEXTSupRdown4} ${TEXTSupRdown5} ${TEXTSupRdown6}
fi

## remove all RTM images from all shots
# Rm ${TEMPDIR}/drtm${m}imageshot*.H

exit
