#!/bin/sh
# File: drtm2d-horiz.sh
# 1st argument will be the value of macro ${m}, which is the name of velocity model vel${m}.mig.H
# Script file to parallelly compute RTM image using FK decomposition techniques for horizontal components (computing every ${jsx} shots along x-axis at the same time) 
# Input file: "rtm${m}in.H" (interpolated shot gathers)
# axes: 1=rec_z, 2=rec_x, 3=time, 4=sou_x, 5=sou_z
# Output file: "rtm${m}out.H"
# axes: 1=z, 2=x, 3=Undefined

###############################
### How to use this script in makefile???:
# drtmout$(m).H drtmout$(m).SrightRright.H drtmout$(m).SleftRleft.H drtmout$(m).SrightRleft.H drtmout$(m).SleftRright.H: inputdata$(m).H velfile.H soufile.H Par/drtm0.vel.P ./Bin/crtm2d_program.x ./Bin/Fkdecomp2D_program.x ./Bin/Padntape_program.x
#       sh drtm2d-horiz.sh '$(m)' 'inputdata$(m).H' 'drtmout$(m).H' 'drtmout$(m).SrightRright.H' 'drtmout$(m).SleftRleft.H' 'drtmout$(m).SrightRleft.H' 'drtmout$(m).SleftRright.H' 'conventional_rtm2d_program.x' 'Fkdecomp2d_program.x' 'Padntape_beforeFFT.x' 'vel.mig.H' 'soufile' 'parfile' 'jsx=4' 'tmpdir=.' 'FKSrightOUT.H' 'FKSleftOUT.H' 'FKRrightOUT.H' 'FKRleftOUT.H' 'SrightOUT.H' 'SleftOUT.H' 'RrightOUT.H' 'RleftOUT.H' 'getonlyrealpart' 'decompicon'

###############################

###############################
## Note that parfile for DRTM definitely contains all parameters for calling the conventional RTM program which is able to produce all wavefields at every time steps.
## In addition, this parfile must contain parameters for Fk-decomposition and Padding as follows:
#
# For Padntape program we need:
#              beg1, beg2, extend(optional), taper(optional)
#              [ I might suggest to use "beg1=50 beg2=50 extend=2 taper=0.01" ]
#              n1out,n2out will already be computed in this script.
#
# For Fkdecomp2d program, we always use dim=2 axis1=-1 axis2=1 we need:
#              filter(optional), phi(optional,available if using filter=2), get_filter, get_fkdataout
#              [ I might suggest to use "filter=2 phi=10" ]
#        NOTE that if get_*=y, only those auxiliary data from the 1st shot will be produced
#
# For selecting types of decomposed RTM output:
#              getonlyrealpart = 1, default    # the output is real-valued (only real part of complex-valued output)
#              getonlyrealpart = 0             # the output is complex-valued
#
# For selecting imaging condition for decomposed RTM (must be declared):
#              decompicon = 1          # Real part is the correlation of real parts, where Imag part is the correlation of imag parts
#              decompicon = 2          # taking complex conjugate for dsouwvfld in correlation, so the Imaginary part involves the image formation
#
####################################
## Parameters for selecting decomposed RTM components [0 or 1]: 1 means yes
####################################
#get_DRTM_SrightRright=
#get_DRTM_SleftRleft=
#get_DRTM_SrightRleft=
#get_DRTM_SleftRright=
###############################

## use double quatation marks "$NAME" to view the variable's contents
## single quatation marks '$NAME' does not show the contents of NAME 

echo "calling $0 script"
echo "reading parameters..."

m=$1         #suffix names for showong identity of processing
echo m=$m

IN=$2       #input file /gz,gx,t,sx,sz/ with interpreted gx
if [ -z $IN ];then echo 'IN=$2 does not exist';exit;fi #test for a zero-length string
echo IN=$IN

OUT=$3       #conventional rtm image file /z,x/
if [ -z $OUT ];then echo 'OUT=$3 does not exist';exit;fi
echo OUT=$OUT

OUTSrightRright=$4      # rtm image from SrightRright
if [ -z $OUTSrightRright ];then echo 'OUTSrightRright=$4 does not exist';exit;fi
echo OUTSrightRright=$OUTSrightRright

OUTSleftRleft=$5
if [ -z $OUTSleftRleft ];then echo 'OUTSleftRleft=$5 does not exist';exit;fi
echo OUTSleftRleft=$OUTSleftRleft

OUTSrightRleft=$6
if [ -z $OUTSrightRleft ];then echo 'OUTSrightRleft=$6 does not exist';exit;fi
echo OUTSrightRleft=$OUTSrightRleft

OUTSleftRright=$7
if [ -z $OUTSleftRright ];then echo 'OUTSleftRright=$7 does not exist';exit;fi
echo OUTSleftRright=$OUTSleftRright

RTMPROG=$8 # rtm program
if [ -z $RTMPROG ];then echo 'RTMPROG=$8 does not exist';exit;fi
echo RTMPROG=$RTMPROG

FKDECOMPROG=$9 # Fkdecomposition program
if [ -z $FKDECOMPROG ];then echo 'FKDECOMPROG=$9 does not exist';exit;fi
echo FKDECOMPROG=$FKDECOMPROG

PADNTAPEPROG=${10} # Pad and tape prgram before doing FFT
if [ -z $PADNTAPEPROG ];then echo 'PADNTAPEPROG=${10} does not exist';exit;fi
echo PADNTAPEPROG=$PADNTAPEPROG

VELFILE=${11}   #velocity file
if [ -z $VELFILE ];then echo 'VELFILE=${11} does not exist';exit;fi
echo VELFILE=$VELFILE

SOUFILE=${12}   #source wavelet file
if [ -z $SOUFILE ];then echo 'SOUFILE=${12} does not exist';exit;fi
echo SOUFILE=$SOUFILE

PARFILE=${13}   #parameter file for RTM program
if [ -z $PARFILE ];then echo 'PARFILE=${13} does not exist';exit;fi
echo PARFILE=$PARFILE

################################
## Read parameters from parfile to this shell script
echo "Reading parameters from the paramter file"
. ${PARFILE}
################################

jsx=${14}       #increment between shots along x-axis  (the number of shot record to be muted in the same time)
if [ -z $jsx ];then jsx=4;fi
echo jsx=$jsx

TEMPDIR=${15}   # temp file to store temporary data
if [ -z $TEMPDIR ];then TEMPDIR=.;fi
echo TEMPDIR=$TEMPDIR

##################
## Optional snapshot files and filtering results in FK domain
##################
# Sright in FK
FKSrightOUT=${16}  # Sright file in FK domain
if [ -z $FKSrightOUT ];then FKSrightOUT=FKSright${m}.H;fi
echo FKSrightOUT=$FKSrightOUT
# Sleft in FK
FKSleftOUT=${17}  # Sleft file in FK domain
if [ -z $FKSleftOUT ];then FKSleftOUT=FKSright${m}.H;fi
echo FKSleftOUT=$FKSleftOUT
# Rright in FK
FKRrightOUT=${18}  # Rright file in FK domain
if [ -z $FKRrightOUT ];then FKRrightOUT=FKRright${m}.H;fi
echo FKRrightOUT=$FKRrightOUT
# Rleft in FK
FKRleftOUT=${19}  # Rleft file in FK domain
if [ -z $FKRleftOUT ];then FKRleftOUT=FKRleft${m}.H;fi
echo FKRleftOUT=$FKRleftOUT

##################
## Optional snapshot files of decomposed wavefields
## these file will keep only real parts of the wavefields after filtering
## output is /z,x,t/
##################
# Sright wavefield snapshots
SrightOUT=${20}  # Sright file
if [ -z $SrightOUT ];then SrightOUT=Sright${m}.H;fi
echo SrightOUT=$SrightOUT
# Sleft wavefield snapshots
SleftOUT=${21}  # Sleft file
if [ -z $SleftOUT ];then SleftOUT=Sright${m}.H;fi
echo SleftOUT=$SleftOUT
# Rright wavefield snapshots
RrightOUT=${22}  # Rright file
if [ -z $RrightOUT ];then RrightOUT=Rright${m}.H;fi
echo RrightOUT=$RrightOUT
# Rleft wavefield snapshots
RleftOUT=${23}  # Rleft file
if [ -z $RleftOUT ];then RleftOUT=Rleft${m}.H;fi
echo RleftOUT=$RleftOUT

##################
## Optional parameters for selecting output types of DRTM and imaging condition
## if these parameters exist on the command line, the script will ignore the same parameters already defined in parfile
##################
if [ ! -z ${24} ]; then # if ${24} is a nonzero string (not blank or just free-space string), new getonlyrealpart will be assigned
    getonlyrealpart=${24}   # [0] the output is complex-valued, [1] the output is real-valued (only real part of complex-valued output)
fi

if [ ! -z ${25} ]; then # if ${25} is a nonzero string (not blank or just free-space string), new decompicon will be assigned
    decompicon=${25}   # [1] Real part is the correlation of real parts, where Imag part is the correlation of imag parts
                       # [2] taking complex conjugate for dsouwvfld in correlation, so the Imaginary part involves the image formation
fi

echo "reading parameters complete..."

##########################################
## IGNORE HERE
##########################################
#m=$1 # name of velocity model is the 1st argument when calling this script
###############################
#jsx=4  # increment between shots along x-axis  (the intershots will be RTMed at the same time)
#tdelay=8 # delayed time in seconds
###############################
#IN=rtm${m}in.H
## output history files
#OUT=rtm${m}out.H
#OUTSrightRright=rtm${m}out.SrightRright.H
#OUTSleftRleft=rtm${m}out.SleftRleft.H
#OUTSrightRleft=rtm${m}out.SrightRleft.H
#OUTSleftRright=rtm${m}out.SleftRright.H
#SOUSNAPOUT=sousnap${m}.H
#RECSNAPOUT=recsnap${m}.H
#SNAPOUT=reflsnap${m}.H
#PARFILE=./Par/rtm0.vel${m}.P
#RTMPROG=./Bin/crtm2d_program.x
#FKDECOMPROG=./Bin/Fkdecomp2D_program.x
#TEMPDIR=./temp
#VELFILE=vel${m}.mig.H
#SOUFILE=sou.HH
##########################################

echo "Start program to compute RTM image in vel1 model..."
echo "Reading shot info..."
nsx=`Get < $IN parform=n n4`
dsx=`Get < $IN parform=n d4`
osx=`Get < $IN parform=n o4`
echo "nsx,dsx,osx,jsx" $nsx $dsx $osx $jsx
nt=`Get < $IN parform=n n3`
dt=`Get < $IN parform=n d3`
ot=`Get < $IN parform=n o3`
echo "nt,dt,ot" $nt $dt $ot
ngx=`Get < $IN parform=n n1`
dgx=`Get < $IN parform=n d1`
ogx=`Get < $IN parform=n o1`
echo "ngx,dgx,ogx" $ngx $dgx $ogx
nz=`Get < $VELFILE parform=n n1`
dz=`Get < $VELFILE parform=n d1`
oz=`Get < $VELFILE parform=n o1`
echo "nz,dz,oz" $nz $dz $oz
nx=`Get < $VELFILE parform=n n2`
dx=`Get < $VELFILE parform=n d2`
ox=`Get < $VELFILE parform=n o2`
echo "nx,dx,ox" $nx $dx $ox

######################
## reorganizing some parameters
###### wavefield time step should equal to jt_image step
jtsnaprtm=${jt_image}
echo "jt_image=${jt_image}, and jtsnaprtm=${jtsnaprtm}"

echo "Input parameters for Padntape program:"
echo ".... beg1=${beg1}, beg2=${beg2}, extend=${extend} taper=${taper}"
## for horizontal decomposition padding axes are /t,x,z/
PADTAPE="${PADNTAPEPROG} n1out=$(((${nt}-1)/${jtsnaprtm}+1+${beg1}+${beg1})) n2out=$((${nx}+${beg2}+${beg2})) beg1=${beg1} beg2=${beg2} extend=${extend} taper=${taper} esize=4"
UNPADTAPE="Window n1=$(((${nt}-1)/${jtsnaprtm}+1)) f1=${beg1} n2=${nx} f2=${beg2}"
UNPADTAPEecho="o1=${ot} o2=${ox}"  # synchronize origin of unpadded vertical wvfld

echo "Input parameters for Fkdecomp2d program:"
echo ".... dim=2 axis1=-1 axis2=1 filter=${filter} phi=${phi} get_filter=${get_filter} get_fkdataout=${get_fkdataout}"
filter2d="filter=${filter} phi=${phi} get_filter=${get_filter} get_fkdataout=${get_fkdataout} verb=${verb}"

###### Feature of decomposed RTM output is only REAL part (DEFAULT) after applying imaging condition or complex-valued file.
echo "getonlyrealpart=${getonlyrealpart}"
if [ ${getonlyrealpart} -eq 0 ]
then
    getonlyrealpart=" "
else
    getonlyrealpart="| Real"
fi

###### Imaging condition for decomposed RTM #####
echo "Declare decomposed RTM imaging condition where file1=dsouwvfld and file2=drecwvfld:"
if [ ${decompicon} -eq 1 ]
then 
    echo "Real part is the correlation of real parts, where Imag part is the correlation of imag parts"
    decompicon="exp_real='file1.re*file2.re' exp_imag='file1.im*file2.im'"
elif [ ${decompicon} -eq 2 ]
then
    #taking complex conjugate for dsouwvfld in correlation, so the Im part involves the image formation
    echo "taking complex conjugate for dsouwvfld in correlation, so the Im part involves the image formation"
    decompicon="exp_real='file1.re*file2.re+file1.im*file2.im' exp_imag='file1.re*file2.im-file1.im*file2.re'"
else
    echo "decompicon=${decompicon} is invalid"
    exit
fi
echo "decompicon=$decompicon"

#######
####################################
## Parameters for selecting decomposed RTM components [0 or 1] read from parfile
####################################
echo "get_DRTM_SrightRright=${get_DRTM_SrightRright}"
echo "get_DRTM_SleftRleft=${get_DRTM_SleftRleft}"
echo "get_DRTM_SrightRleft=${get_DRTM_SrightRleft}"
echo "get_DRTM_SleftRright=${get_DRTM_SleftRright}"


###############
## Old definition
##PADTAPE="${PADNTAPEPROG} n1out=$(((${nt}-1)/${jtsnaprtm}+1+100)) n2out=$((${nz}+100)) beg1=50 beg2=50 extend=2 taper=0.01"
##UNPADTAPE="Window n1=$(((${nt}-1)/${jtsnaprtm}+1)) f1=50 n2=${nz} f2=50"
##filter2d="filter=2 phi=10"
##	<$< ${PADTAPE} | Rtoc | $B/Fkdecomp2d_program.x dim=2 axis1=-1 axis2=1 flag=-1 ${filter2d} | Real | ${UNPADTAPE} |Grey ${GreyFFT}|Tube&
##
##  Decomposed RTM imaging condition where file1=decomposed.souwvfld and file2=decomposed.recwvfld .
##  exp_real='file1.re*file2.re+file1.im*file2.im' exp_imag='file1.re*file2.im-file1.im*file2.re'
######

# parameters for conventional RTM 
TEXT0=tmp0rtm${m}.txt
TEXT1=tmp1rtm${m}.txt
TEXT2=tmp2rtm${m}.txt
TEXT3=tmp3rtm${m}.txt
TEXT4=tmp4rtm${m}.txt
TEXT5=tmp5rtm${m}.txt
TEXT6=tmp6rtm${m}.txt
OUT1=rtm${m}out1.H
OUT2=rtm${m}out2.H
OUT3=rtm${m}out3.H
OUT4=rtm${m}out4.H
OUT5=rtm${m}out5.H
OUT6=rtm${m}out6.H

# parameters for RTM from SrightRright
TEXTSrightRright0=tmp0rtm${m}SrightRright.txt
TEXTSrightRright1=tmp1rtm${m}SrightRright.txt
TEXTSrightRright2=tmp2rtm${m}SrightRright.txt
TEXTSrightRright3=tmp3rtm${m}SrightRright.txt
TEXTSrightRright4=tmp4rtm${m}SrightRright.txt
TEXTSrightRright5=tmp5rtm${m}SrightRright.txt
TEXTSrightRright6=tmp6rtm${m}SrightRright.txt
OUTSrightRright1=rtm${m}out1SrightRright.H
OUTSrightRright2=rtm${m}out2SrightRright.H
OUTSrightRright3=rtm${m}out3SrightRright.H
OUTSrightRright4=rtm${m}out4SrightRright.H
OUTSrightRright5=rtm${m}out5SrightRright.H
OUTSrightRright6=rtm${m}out6SrightRright.H

# parameters for RTM from SleftRleft
TEXTSleftRleft0=tmp0rtm${m}SleftRleft.txt
TEXTSleftRleft1=tmp1rtm${m}SleftRleft.txt
TEXTSleftRleft2=tmp2rtm${m}SleftRleft.txt
TEXTSleftRleft3=tmp3rtm${m}SleftRleft.txt
TEXTSleftRleft4=tmp4rtm${m}SleftRleft.txt
TEXTSleftRleft5=tmp5rtm${m}SleftRleft.txt
TEXTSleftRleft6=tmp6rtm${m}SleftRleft.txt
OUTSleftRleft1=rtm${m}out1SleftRleft.H
OUTSleftRleft2=rtm${m}out2SleftRleft.H
OUTSleftRleft3=rtm${m}out3SleftRleft.H
OUTSleftRleft4=rtm${m}out4SleftRleft.H
OUTSleftRleft5=rtm${m}out5SleftRleft.H
OUTSleftRleft6=rtm${m}out6SleftRleft.H

# parameters for RTM from SrightRleft
TEXTSrightRleft0=tmp0rtm${m}SrightRleft.txt
TEXTSrightRleft1=tmp1rtm${m}SrightRleft.txt
TEXTSrightRleft2=tmp2rtm${m}SrightRleft.txt
TEXTSrightRleft3=tmp3rtm${m}SrightRleft.txt
TEXTSrightRleft4=tmp4rtm${m}SrightRleft.txt
TEXTSrightRleft5=tmp5rtm${m}SrightRleft.txt
TEXTSrightRleft6=tmp6rtm${m}SrightRleft.txt
OUTSrightRleft1=rtm${m}out1SrightRleft.H
OUTSrightRleft2=rtm${m}out2SrightRleft.H
OUTSrightRleft3=rtm${m}out3SrightRleft.H
OUTSrightRleft4=rtm${m}out4SrightRleft.H
OUTSrightRleft5=rtm${m}out5SrightRleft.H
OUTSrightRleft6=rtm${m}out6SrightRleft.H

# parameters for RTM from SleftRright
TEXTSleftRright0=tmp0rtm${m}SleftRright.txt
TEXTSleftRright1=tmp1rtm${m}SleftRright.txt
TEXTSleftRright2=tmp2rtm${m}SleftRright.txt
TEXTSleftRright3=tmp3rtm${m}SleftRright.txt
TEXTSleftRright4=tmp4rtm${m}SleftRright.txt
TEXTSleftRright5=tmp5rtm${m}SleftRright.txt
TEXTSleftRright6=tmp6rtm${m}SleftRright.txt
OUTSleftRright1=rtm${m}out1SleftRright.H
OUTSleftRright2=rtm${m}out2SleftRright.H
OUTSleftRright3=rtm${m}out3SleftRright.H
OUTSleftRright4=rtm${m}out4SleftRright.H
OUTSleftRright5=rtm${m}out5SleftRright.H
OUTSleftRright6=rtm${m}out6SleftRright.H

# remove files or text files before launching programs
Rm ${TEMPDIR}/rtm${m}imageshot*.H
Rm ${TEMPDIR}/rtm${m}shotrecord*.H
Rm ${OUT} ${OUTSrightRright} ${OUTSleftRleft} ${OUTSrightRleft} ${OUTSleftRright}
Rm ${OUT1} ${OUTSrightRright1} ${OUTSleftRleft1} ${OUTSrightRleft1} ${OUTSleftRright1}
Rm ${OUT2} ${OUTSrightRright2} ${OUTSleftRleft2} ${OUTSrightRleft2} ${OUTSleftRright2}
Rm ${OUT3} ${OUTSrightRright3} ${OUTSleftRleft3} ${OUTSrightRleft3} ${OUTSleftRright3}
Rm ${OUT4} ${OUTSrightRright4} ${OUTSleftRleft4} ${OUTSrightRleft4} ${OUTSleftRright4}
Rm ${OUT5} ${OUTSrightRright5} ${OUTSleftRleft5} ${OUTSrightRleft5} ${OUTSleftRright5}
Rm ${OUT6} ${OUTSrightRright6} ${OUTSleftRleft6} ${OUTSrightRleft6} ${OUTSleftRright6}
rm -f ${TEXT0} ${TEXTSrightRright0} ${TEXTSleftRleft0} ${TEXTSrightRleft0} ${TEXTSleftRright0}
rm -f ${TEXT1} ${TEXTSrightRright1} ${TEXTSleftRleft1} ${TEXTSrightRleft1} ${TEXTSleftRright1}
rm -f ${TEXT2} ${TEXTSrightRright2} ${TEXTSleftRleft2} ${TEXTSrightRleft2} ${TEXTSleftRright2}
rm -f ${TEXT3} ${TEXTSrightRright3} ${TEXTSleftRleft3} ${TEXTSrightRleft3} ${TEXTSleftRright3}
rm -f ${TEXT4} ${TEXTSrightRright4} ${TEXTSleftRleft4} ${TEXTSrightRleft4} ${TEXTSleftRright4}
rm -f ${TEXT5} ${TEXTSrightRright5} ${TEXTSleftRleft5} ${TEXTSrightRleft5} ${TEXTSleftRright5}
rm -f ${TEXT6} ${TEXTSrightRright6} ${TEXTSleftRleft6} ${TEXTSrightRleft6} ${TEXTSleftRright6}

#<${IN} Transp plane=13 | Transp plane=34 > ${OUT}  # for windowing purpose
#In ${OUT}


###########

echo "5555555555555555555555555555555555555555555555555555555555555555555555"
echo "The script is computing decomposed RTM images of vel${m} from ${jsx} shots at the same time."
echo "5555555555555555555555555555555555555555555555555555555555555555555555"
for((isx=1; $isx<=$nsx; isx=$isx+$jsx))
do # each iteration will do rtm for ${jsx} shots at the same time

    ##############
    ### do Windowing and Transposing input data to enter a conventional RTM program
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]
	then
	    echo "************* Windowing and Transposing SHOT$((${isx}+$k)) *****************"
	    echo "isx,osx,sx:" $(($isx+$k)) $osx `echo $osx+$(($isx+$k-1))*$dsx | bc`
	    # This set of command will consume 2 cpu's at the same time; one might guess tha the simutaneous loops should not be greater than half of #cpu
	    # BUT computing more than half of #cpu simutaneouly is significantly more effective than limiting ourselves to just 4 simutaneous loops.
	    # SO, what I did here is the best way. No need to window and transpose the whole shots first with 4-simutaneous process before entering RTM with ${jsx}-simutaneous process
	    # Note that calling only-Window commands simutaneously will share the whole processes in a single CPU. BUT calling only-Transp commands simutaneously will separate every single process to a different CPU. SO, if we make Window-Transp dependent to each other, it will behave like only-Transp. That's why I use .... Window | Transp .....
	    <${IN} Window n4=1 f4=$((${isx}+${k}-1)) | Transp plane=12 reshape=2,3,5 > ${TEMPDIR}/drtm${m}shotrecord$((${isx}+${k})).H &
	fi
    done
    # wait untill all background processes to finish
    wait
 
    #############
    #### do conventional RTM imaging and get source and receiver wavefields
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do # run rtm program within each combination of ${jsx} shots
	if [ $((${isx}+${k})) -le ${nsx} ]
	then
	    echo "************* Conventional RTMing SHOT$((${isx}+$k))VEL${m} *****************"
	    echo "**** and getting source & receiver wavefields at all sufficient time steps***"
	    # In the middle of each iteration we need to run the rtm program as background.
            # Every shot gives us snapshot file
	    <${VELFILE} ${RTMPROG} par=${PARFILE} jsnap=${jtsnaprtm} get_snap=0 get_sousnap=1 get_recsnap=1 source=${SOUFILE} receiver=${TEMPDIR}/drtm${m}shotrecord$((${isx}+${k})).H >${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H sousnap=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).H recsnap=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).H &

	    if [ $((${isx}+${k})) -le 20 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT1}
	    elif [ $((${isx}+${k})) -le 40 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT2}
	    elif [ $((${isx}+${k})) -le 60 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT3}
	    elif [ $((${isx}+${k})) -le 80 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT4}
	    elif [ $((${isx}+${k})) -le 100 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT5}
	    else
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT6}
	    fi
	fi
    done
    # wait untill all background processes to finish
    wait

    #############
    #### remove shot records that have been used.
    echo "removing shot records that have been used..."
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/drtm${m}shotrecord$((${isx}+$k)).H
	fi
    done

#############################################################
### after this line it is all about decomposition and DRTM
#############################################################
if [ ${get_DRTM_SrightRright} -eq 1 -o ${get_DRTM_SleftRleft} -eq 1 -o ${get_DRTM_SrightRleft} -eq 1 -o ${get_DRTM_SleftRright} -eq 1 ]; then

    ##################################################
    #####***************************************######
    #### decompose source wavefield
    ##################################################
    echo "decomposing source wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
    #### Sorting the source wavefield ready for decomposition
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).H Transp plane=13 >${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H & # transposing from /z,x,t/ to /t,x,z/
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed source wavefields before reordered. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).H
	fi
    done

    #####################################
    ## Preparing input for decomposing source wavefield Sleft&Sright
    #####################################
    # convert from real to complex input for Sleft and Sright
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H ${PADTAPE} | Rtoc >${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H &
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed source wavefields that have been used to separate wavefields. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H
	fi
    done

    #####################################
    #### select leftgoing source wavefield Sleft if needed
    #####################################
    if [ ${get_DRTM_SleftRleft} -eq 1 -o ${get_DRTM_SleftRright} -eq 1 ]; then
	echo "Getting leftgoing source wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp2d program for Sleft
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=-1 ${filter2d} >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H fkdataout=${FKSleftOUT} &
		else
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=-1 ${filter2d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H ${UNPADTAPE} > ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H &
	    fi
	done
        # wait untill all background processes to finish
	wait
	
        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpSleft
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H
	    fi
	done
	
    fi
    #####################################

    #####################################
    #### select rightgoing source wavefield Sright if needed
    #####################################
    if [ ${get_DRTM_SrightRleft} -eq 1 -o ${get_DRTM_SrightRright} -eq 1 ]; then
	echo "Getting rightgoing source wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp2d program for Sright
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=1 ${filter2d} >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H fkdataout=${FKSrightOUT} &
		else
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=1 ${filter2d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H ${UNPADTAPE} > ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpSright
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H
	    fi
	done

    fi

    ##### remove source wvfld that had been prepared for decomposition
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do #tmpSource
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H
	fi
    done

    #####################################


    ##################################################
    #####***************************************######
    #### decompose receiver wavefield
    ##################################################
    echo "decomposing receiver wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
    #### Sorting the receiver wavefield ready for decomposition
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).H Transp plane=13 >${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H & # transposing from /z,x,t/ to /t,x,z/
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed receiver wavefields before reordered. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).H
	fi
    done

    #####################################
    ## Preparing input for decomposing receiver wavefield Rleft&Rright
    #####################################
    # convert from real to complex input for Rleft and Rright
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H ${PADTAPE} | Rtoc >${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H &
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed receiver wavefields that have been used to separate wavefields. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H
	fi
    done

    #####################################
    #### select leftgoing receiver wavefield Rleft if needed
    #####################################
    if [ ${get_DRTM_SrightRleft} -eq 1 -o ${get_DRTM_SleftRleft} -eq 1 ]; then
	echo "Getting leftgoing receiver wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp2d program for Rleft
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=-1 ${filter2d} >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H fkdataout=${FKRleftOUT} &
		else
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=-1 ${filter2d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H ${UNPADTAPE} > ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpRleft
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H
	    fi
	done

    fi
    #####################################

    #####################################
    #### select rightgoing receiver wavefield Rright
    #####################################
    if [ ${get_DRTM_SrightRright} -eq 1 -o ${get_DRTM_SleftRright} -eq 1 ]; then
	echo "Getting rightgoing receiver wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp2d program for Rright
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
        do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=1 ${filter2d} >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H fkdataout=${FKRrightOUT} &
		else
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H ${FKDECOMPROG} dim=2 axis1=-1 axis2=1 flag=1 ${filter2d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H ${UNPADTAPE} > ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpRright
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H
	    fi
	done

    fi

    ##### remove receiver wvfld that had been prepared for decomposition

    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do #tmpReceiver
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.H
	fi
    done

    #####################################


######################
######################
    
     #########################################################################################
    #### apply RTM imaging condition to all combinations of source and receiver wavefields ####
     #########################################################################################
    
         # Math file1=souwvfld$*.horiz.fkffto.right.H file2=recwvfld$*.horiz.fkffto.right.H exp_real='file1.re*file2.re+file1.im*file2.im' exp_imag='file1.re*file2.im-file1.im*file2.re' >$@
         # <$< Transp plane=12 | Stack axes=2 | Real >$@ # sum over time
    ##########################################
    #### Getting RTM image from SrightRright components
    ##########################################
    if [ ${get_DRTM_SrightRright} -eq 1 ]; then
	echo "RTM from SrightRright SHOT${isx} to SHOT$((${isx}+${jsx}-1)).."
        # RTM from Sright*Rright
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).horiz.SrightRright.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).horiz.SrightRright.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SrightRright.H &
	        # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SrightRright.H" >> ${TEXTSrightRright1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SrightRright.H" >> ${TEXTSrightRright2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SrightRright.H" >> ${TEXTSrightRright3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SrightRright.H" >> ${TEXTSrightRright4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SrightRright.H" >> ${TEXTSrightRright5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SrightRright.H" >> ${TEXTSrightRright6}
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SrightRright
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).horiz.SrightRright.H
	    fi
	done

    fi
    ##########################################

    ##########################################
    ## RTM from Sleft*Rleft
    ##########################################
    if [ ${get_DRTM_SleftRleft} -eq 1 ]; then
	echo "RTM from SleftRleft SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).horiz.SleftRleft.H &
	    fi
	done
    # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).horiz.SleftRleft.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SleftRleft.H &
	    # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SleftRleft.H" >> ${TEXTSleftRleft1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SleftRleft.H" >> ${TEXTSleftRleft2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SleftRleft.H" >> ${TEXTSleftRleft3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SleftRleft.H" >> ${TEXTSleftRleft4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SleftRleft.H" >> ${TEXTSleftRleft5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SleftRleft.H" >> ${TEXTSleftRleft6}
		fi
	    fi
	done
    # wait untill all background processes to finish
	wait
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SleftRleft
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).horiz.SleftRleft.H
	fi
	done

    fi
    ##########################################

    ##########################################
    ### RTM from Sright*Rleft
    ##########################################
    if [ ${get_DRTM_SrightRleft} -eq 1 ]; then
	echo "RTM from SrightRleft SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).horiz.SrightRleft.H &
	    fi
	done
    # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).horiz.SrightRleft.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SrightRleft.H &
	    # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SrightRleft.H" >> ${TEXTSrightRleft1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SrightRleft.H" >> ${TEXTSrightRleft2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SrightRleft.H" >> ${TEXTSrightRleft3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SrightRleft.H" >> ${TEXTSrightRleft4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SrightRleft.H" >> ${TEXTSrightRleft5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SrightRleft.H" >> ${TEXTSrightRleft6}
		fi
	    fi
	done
    # wait untill all background processes to finish
	wait
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SleftRleft
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).horiz.SrightRleft.H
	    fi
	done

    fi
    ##########################################

    ##########################################
    #### RTM from Sleft*Rright
    ##########################################
    if [ ${get_DRTM_SleftRright} -eq 1 ]; then
	echo "RTM from SleftRright SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).horiz.SleftRright.H &
	    fi
	done
    # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).horiz.SleftRright.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SleftRright.H &
	    # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SleftRright.H" >> ${TEXTSleftRright1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SleftRright.H" >> ${TEXTSleftRright2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SleftRright.H" >> ${TEXTSleftRright3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SleftRright.H" >> ${TEXTSleftRright4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SleftRright.H" >> ${TEXTSleftRright5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).horiz.SleftRright.H" >> ${TEXTSleftRright6}
		fi
	    fi
	done
    # wait untill all background processes to finish
	wait
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SleftRleft
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).horiz.SleftRright.H
	    fi
	done

    fi
    ##########################################

    #############
    #### remove source wavefield and receiver wavefield that have been used to compute RTM images. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then

	    if [ $((${isx}+${k})) -eq 1 ]; then
	        ## output the decomposed source- and receiver wavefield snapshots of the first shot
		<${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H Transp plane=13 | Real > ${SleftOUT} &
		<${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H Transp plane=13 | Real > ${SrightOUT} &
		<${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H Transp plane=13 | Real > ${RleftOUT} &
		<${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H Transp plane=13 | Real > ${RrightOUT} &
		wait
	    fi

	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H &
	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H &
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.left.H &
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz.right.H
	fi
    done

fi
#############################################################
### above this line it is all about decomposition and DRTM
#############################################################

    #############
    #### remove all source and receiver wavefields records that have been used. 
    #for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    #do
    #	if [ $((${isx}+${k})) -le ${nsx} ]; then
    #	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).horiz*.H
    #	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).horiz*.H
    #	fi
    #done

done

###########################################
### Combine the conventional RTM images
###########################################
echo "================================="
echo "combining conventional RTM images"
echo "================================="
if [ -f ${TEXT1} ]; then
    rtmlist1=`cat ${TEXT1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUT1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT1}
    echo ${OUT1} >> ${TEXT0}
    runloop=1
fi
if [ -f ${TEXT2} ]; then
    rtmlist2=`cat ${TEXT2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUT2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT2}
    echo ${OUT2} >> ${TEXT0}
    runloop=2
fi
if [ -f ${TEXT3} ]; then
    rtmlist3=`cat ${TEXT3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUT3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT3}
    echo ${OUT3} >> ${TEXT0}
    runloop=3
fi
if [ -f ${TEXT4} ]; then
    rtmlist4=`cat ${TEXT4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUT4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT4}
    echo ${OUT4} >> ${TEXT0}
    runloop=4
fi
if [ -f ${TEXT5} ]; then
    rtmlist5=`cat ${TEXT5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUT5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT5}
    echo ${OUT5} >> ${TEXT0}
    runloop=5
fi
if [ -f ${TEXT6} ]; then
    rtmlist6=`cat ${TEXT6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUT6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT6}
    echo ${OUT6} >> ${TEXT0}
    runloop=6
fi
#######################
## generate conventional RTM images
#######################
if [ -f ${TEXT0} ]; then
    rtmlist0=`cat ${TEXT0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23|Stack axes=2 > ${OUT}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUT1} ${OUT2} ${OUT3} ${OUT4} ${OUT5} ${OUT6}
    rm -f ${TEXT0} ${TEXT1} ${TEXT2} ${TEXT3} ${TEXT4} ${TEXT5} ${TEXT6}
fi

###########################################
### Combine the RTM images from SrightRright
###########################################
echo "===================================="
echo "combining RTM images from SrightRright"
echo "===================================="
if [ -f ${TEXTSrightRright1} ]; then
    rtmlist1=`cat ${TEXTSrightRright1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSrightRright1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSrightRright1}
    echo ${OUTSrightRright1} >> ${TEXTSrightRright0}
    runloop=1
fi
if [ -f ${TEXTSrightRright2} ]; then
    rtmlist2=`cat ${TEXTSrightRright2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSrightRright2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSrightRright2}
    echo ${OUTSrightRright2} >> ${TEXTSrightRright0}
    runloop=2
fi
if [ -f ${TEXTSrightRright3} ]; then
    rtmlist3=`cat ${TEXTSrightRright3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSrightRright3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSrightRright3}
    echo ${OUTSrightRright3} >> ${TEXTSrightRright0}
    runloop=3
fi
if [ -f ${TEXTSrightRright4} ]; then
    rtmlist4=`cat ${TEXTSrightRright4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSrightRright4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSrightRright4}
    echo ${OUTSrightRright4} >> ${TEXTSrightRright0}
    runloop=4
fi
if [ -f ${TEXTSrightRright5} ]; then
    rtmlist5=`cat ${TEXTSrightRright5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSrightRright5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSrightRright5}
    echo ${OUTSrightRright5} >> ${TEXTSrightRright0}
    runloop=5
fi
if [ -f ${TEXTSrightRright6} ]; then
    rtmlist6=`cat ${TEXTSrightRright6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSrightRright6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSrightRright6}
    echo ${OUTSrightRright6} >> ${TEXTSrightRright0}
    runloop=6
fi

#######################
## generate DRTM images from SrightRright
#######################
if [ -f ${TEXTSrightRright0} ]; then
    rtmlist0=`cat ${TEXTSrightRright0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2|Transp plane=12 > ${OUTSrightRright}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSrightRright}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSrightRright1} ${OUTSrightRright2} ${OUTSrightRright3} ${OUTSrightRright4} ${OUTSrightRright5} ${OUTSrightRright6}
    rm -f ${TEXTSrightRright0} ${TEXTSrightRright1} ${TEXTSrightRright2} ${TEXTSrightRright3} ${TEXTSrightRright4} ${TEXTSrightRright5} ${TEXTSrightRright6}
fi


###########################################
### Combine the RTM images from SleftRleft
###########################################
echo "===================================="
echo "combining RTM images from SleftRleft"
echo "===================================="
if [ -f ${TEXTSleftRleft1} ]; then
    rtmlist1=`cat ${TEXTSleftRleft1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSleftRleft1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSleftRleft1}
    echo ${OUTSleftRleft1} >> ${TEXTSleftRleft0}
    runloop=1
fi
if [ -f ${TEXTSleftRleft2} ]; then
    rtmlist2=`cat ${TEXTSleftRleft2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSleftRleft2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSleftRleft2}
    echo ${OUTSleftRleft2} >> ${TEXTSleftRleft0}
    runloop=2
fi
if [ -f ${TEXTSleftRleft3} ]; then
    rtmlist3=`cat ${TEXTSleftRleft3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSleftRleft3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSleftRleft3}
    echo ${OUTSleftRleft3} >> ${TEXTSleftRleft0}
    runloop=3
fi
if [ -f ${TEXTSleftRleft4} ]; then
    rtmlist4=`cat ${TEXTSleftRleft4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSleftRleft4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSleftRleft4}
    echo ${OUTSleftRleft4} >> ${TEXTSleftRleft0}
    runloop=4
fi
if [ -f ${TEXTSleftRleft5} ]; then
    rtmlist5=`cat ${TEXTSleftRleft5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSleftRleft5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSleftRleft5}
    echo ${OUTSleftRleft5} >> ${TEXTSleftRleft0}
    runloop=5
fi
if [ -f ${TEXTSleftRleft6} ]; then
    rtmlist6=`cat ${TEXTSleftRleft6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSleftRleft6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSleftRleft6}
    echo ${OUTSleftRleft6} >> ${TEXTSleftRleft0}
    runloop=6
fi

#######################
## generate DRTM images from SleftRleft
#######################
if [ -f ${TEXTSleftRleft0} ]; then
    rtmlist0=`cat ${TEXTSleftRleft0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2|Transp plane=12 > ${OUTSleftRleft}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSleftRleft}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSleftRleft1} ${OUTSleftRleft2} ${OUTSleftRleft3} ${OUTSleftRleft4} ${OUTSleftRleft5} ${OUTSleftRleft6}
    rm -f ${TEXTSleftRleft0} ${TEXTSleftRleft1} ${TEXTSleftRleft2} ${TEXTSleftRleft3} ${TEXTSleftRleft4} ${TEXTSleftRleft5} ${TEXTSleftRleft6}
fi

###########################################
### Combine the RTM images from SrightRleft
###########################################
echo "===================================="
echo "combining RTM images from SrightRleft"
echo "===================================="
if [ -f ${TEXTSrightRleft1} ]; then
    rtmlist1=`cat ${TEXTSrightRleft1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSrightRleft1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSrightRleft1}
    echo ${OUTSrightRleft1} >> ${TEXTSrightRleft0}
    runloop=1
fi
if [ -f ${TEXTSrightRleft2} ]; then
    rtmlist2=`cat ${TEXTSrightRleft2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSrightRleft2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSrightRleft2}
    echo ${OUTSrightRleft2} >> ${TEXTSrightRleft0}
    runloop=2
fi
if [ -f ${TEXTSrightRleft3} ]; then
    rtmlist3=`cat ${TEXTSrightRleft3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSrightRleft3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSrightRleft3}
    echo ${OUTSrightRleft3} >> ${TEXTSrightRleft0}
    runloop=3
fi
if [ -f ${TEXTSrightRleft4} ]; then
    rtmlist4=`cat ${TEXTSrightRleft4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSrightRleft4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSrightRleft4}
    echo ${OUTSrightRleft4} >> ${TEXTSrightRleft0}
    runloop=4
fi
if [ -f ${TEXTSrightRleft5} ]; then
    rtmlist5=`cat ${TEXTSrightRleft5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSrightRleft5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSrightRleft5}
    echo ${OUTSrightRleft5} >> ${TEXTSrightRleft0}
    runloop=5
fi
if [ -f ${TEXTSrightRleft6} ]; then
    rtmlist6=`cat ${TEXTSrightRleft6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSrightRleft6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSrightRleft6}
    echo ${OUTSrightRleft6} >> ${TEXTSrightRleft0}
    runloop=6
fi
#######################
## generate DRTM images from SrightRleft
#######################
if [ -f ${TEXTSrightRleft0} ]; then
    rtmlist0=`cat ${TEXTSrightRleft0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2|Transp plane=12 > ${OUTSrightRleft}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSrightRleft}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSrightRleft1} ${OUTSrightRleft2} ${OUTSrightRleft3} ${OUTSrightRleft4} ${OUTSrightRleft5} ${OUTSrightRleft6}
    rm -f ${TEXTSrightRleft0} ${TEXTSrightRleft1} ${TEXTSrightRleft2} ${TEXTSrightRleft3} ${TEXTSrightRleft4} ${TEXTSrightRleft5} ${TEXTSrightRleft6}
fi

###########################################
### Combine the RTM images from SleftRright
###########################################
echo "===================================="
echo "combining RTM images from SleftRright"
echo "===================================="
if [ -f ${TEXTSleftRright1} ]; then
    rtmlist1=`cat ${TEXTSleftRright1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSleftRright1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSleftRright1}
    echo ${OUTSleftRright1} >> ${TEXTSleftRright0}
    runloop=1
fi
if [ -f ${TEXTSleftRright2} ]; then
    rtmlist2=`cat ${TEXTSleftRright2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSleftRright2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSleftRright2}
    echo ${OUTSleftRright2} >> ${TEXTSleftRright0}
    runloop=2
fi
if [ -f ${TEXTSleftRright3} ]; then
    rtmlist3=`cat ${TEXTSleftRright3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSleftRright3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSleftRright3}
    echo ${OUTSleftRright3} >> ${TEXTSleftRright0}
    runloop=3
fi
if [ -f ${TEXTSleftRright4} ]; then
    rtmlist4=`cat ${TEXTSleftRright4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSleftRright4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSleftRright4}
    echo ${OUTSleftRright4} >> ${TEXTSleftRright0}
    runloop=4
fi
if [ -f ${TEXTSleftRright5} ]; then
    rtmlist5=`cat ${TEXTSleftRright5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSleftRright5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSleftRright5}
    echo ${OUTSleftRright5} >> ${TEXTSleftRright0}
    runloop=5
fi
if [ -f ${TEXTSleftRright6} ]; then
    rtmlist6=`cat ${TEXTSleftRright6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSleftRright6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSleftRright6}
    echo ${OUTSleftRright6} >> ${TEXTSleftRright0}
    runloop=6
fi

#######################
## generate DRTM images from SleftRright
#######################
if [ -f ${TEXTSleftRright0} ]; then
    rtmlist0=`cat ${TEXTSleftRright0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2|Transp plane=12 > ${OUTSleftRright}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSleftRright}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSleftRright1} ${OUTSleftRright2} ${OUTSleftRright3} ${OUTSleftRright4} ${OUTSleftRright5} ${OUTSleftRright6}
    rm -f ${TEXTSleftRright0} ${TEXTSleftRright1} ${TEXTSleftRright2} ${TEXTSleftRright3} ${TEXTSleftRright4} ${TEXTSleftRright5} ${TEXTSleftRright6}
fi

## remove all RTM images from all shots
# Rm ${TEMPDIR}/drtm${m}imageshot*.H

exit
