#!/bin/sh
# File: drtm2d-tiltdown.sh
# Script file to parallelly compute RTM image using FK decomposition techniques for downright- and upleft-going (tiltdown) components (computing every ${jsx} shots along x-axis at the same time) 
# Input file: "rtm${m}in.H" (interpolated shot gathers)
# axes: 1=rec_z, 2=rec_x, 3=time, 4=sou_x, 5=sou_z
# Output file: "rtm${m}out.H"
# axes: 1=z, 2=x, 3=Undefined

###############################
### How to use this script in makefile???:
# drtmout$(m).H drtmout$(m).SdownrightRdownright.H drtmout$(m).SupleftRupleft.H drtmout$(m).SdownrightRupleft.H drtmout$(m).SupleftRdownright.H: inputdata$(m).H velfile.H soufile.H Par/drtm0.vel.P ./Bin/crtm2d_program.x ./Bin/Fkdecomp3d_program.x ./Bin/Padntape_program.x
#       sh drtm2d-tiltdown.sh '$(m)' 'inputdata$(m).H' 'drtmout$(m).H' 'drtmout$(m).SdownrightRdownright.H' 'drtmout$(m).SupleftRupleft.H' 'drtmout$(m).SdownrightRupleft.H' 'drtmout$(m).SupleftRdownright.H' 'conventional_rtm2d_program.x' 'Fkdecomp3d_program.x' 'Padntape_beforeFFT.x' 'vel.mig.H' 'soufile' 'parfile' 'jsx=4' 'tmpdir=.' 'FKSdownrightOUT.H' 'FKSupleftOUT.H' 'FKRdownrightOUT.H' 'FKRupleftOUT.H' 'SdownrightOUT.H' 'SupleftOUT.H' 'RdownrightOUT.H' 'RupleftOUT.H' 'getonlyrealpart' 'decompicon'

###############################

###############################
## Note that parfile for DRTM definitely contains all parameters for calling the conventional RTM program which is able to produce all wavefields at every time steps.
## In addition, this parfile must contain parameters for Fk-decomposition and Padding as follows:
#
# For Padntape program we need:
#              beg1, beg2, extend(optional), taper(optional)
#              [ I might suggest to use "beg1=50 beg2=50 extend=2 taper=0.01" ]
#              n1out,n2out will already be computed in this script.
#
# For Fkdecomp3d program, we always use dim=2 axis1=-1 axis2=1 we need:
#              filter(optional), alpha(optional,available if using filter=2), phi(optional,available if using filter=2), get_filter, get_fkdataout
#              [ I might suggest to use "filter=2 alpha=0.2 phi=10" ]
#        NOTE that if get_*=y, only those auxiliary data from the 1st shot will be produced
#
# For selecting types of decomposed RTM output:
#              getonlyrealpart = 1, default    # the output is real-valued (only real part of complex-valued output)
#              getonlyrealpart = 0             # the output is complex-valued
#
# For selecting imaging condition for decomposed RTM (must be declared):
#              decompicon = 1          # Real part is the correlation of real parts, where Imag part is the correlation of imag parts
#              decompicon = 2          # taking complex conjugate for dsouwvfld in correlation, so the Imaginary part involves the image formation
###############################

## use double quatation marks "$NAME" to view the variable's contents
## single quatation marks '$NAME' does not show the contents of NAME 

echo "calling $0 script"
echo "reading parameters..."

m=$1         #suffix names for showong identity of processing
echo m=$m

IN=$2       #input file /gz,gx,t,sx,sz/ with interpreted gx
if [ -z $IN ];then echo 'IN=$2 does not exist';exit;fi #test for a zero-length string
echo IN=$IN

OUT=$3       #conventional rtm image file /z,x/
if [ -z $OUT ];then echo 'OUT=$3 does not exist';exit;fi
echo OUT=$OUT

OUTSdownrightRdownright=$4      # rtm image from SdownrightRdownright
if [ -z $OUTSdownrightRdownright ];then echo 'OUTSdownrightRdownright=$4 does not exist';exit;fi
echo OUTSdownrightRdownright=$OUTSdownrightRdownright

OUTSupleftRupleft=$5
if [ -z $OUTSupleftRupleft ];then echo 'OUTSupleftRupleft=$5 does not exist';exit;fi
echo OUTSupleftRupleft=$OUTSupleftRupleft

OUTSdownrightRupleft=$6
if [ -z $OUTSdownrightRupleft ];then echo 'OUTSdownrightRupleft=$6 does not exist';exit;fi
echo OUTSdownrightRupleft=$OUTSdownrightRupleft

OUTSupleftRdownright=$7
if [ -z $OUTSupleftRdownright ];then echo 'OUTSupleftRdownright=$7 does not exist';exit;fi
echo OUTSupleftRdownright=$OUTSupleftRdownright

RTMPROG=$8 # rtm program
if [ -z $RTMPROG ];then echo 'RTMPROG=$8 does not exist';exit;fi
echo RTMPROG=$RTMPROG

FKDECOMPROG=$9 # Fkdecomposition program
if [ -z $FKDECOMPROG ];then echo 'FKDECOMPROG=$9 does not exist';exit;fi
echo FKDECOMPROG=$FKDECOMPROG

PADNTAPEPROG=${10} # Pad and tape prgram before doing FFT
if [ -z $PADNTAPEPROG ];then echo 'PADNTAPEPROG=$8{10} does not exist';exit;fi
echo PADNTAPEPROG=$PADNTAPEPROG

VELFILE=${11}   #velocity file
if [ -z $VELFILE ];then echo 'VELFILE=${11} does not exist';exit;fi
echo VELFILE=$VELFILE

SOUFILE=${12}   #source wavelet file
if [ -z $SOUFILE ];then echo 'SOUFILE=${12} does not exist';exit;fi
echo SOUFILE=$SOUFILE

PARFILE=${13}   #parameter file for RTM program
if [ -z $PARFILE ];then echo 'PARFILE=${13} does not exist';exit;fi
echo PARFILE=$PARFILE

################################
## Read parameters from parfile to this shell script
echo "Reading parameters from the paramter file"
. ${PARFILE}
################################

jsx=${14}       #increment between shots along x-axis  (the number of shot record to be muted in the same time)
if [ -z $jsx ];then jsx=4;fi
echo jsx=$jsx

TEMPDIR=${15}   # temp file to store temporary data
if [ -z $TEMPDIR ];then TEMPDIR=.;fi
echo TEMPDIR=$TEMPDIR

##################
## Optional filtering results in FK domain
##################
# Sdownright in FK
FKSdownrightOUT=${16}  # Sdownright file in FK domain
if [ -z $FKSdownrightOUT ];then FKSdownrightOUT=FKSdownright${m}.H;fi
echo FKSdownrightOUT=$FKSdownrightOUT
# Supleft in FK
FKSupleftOUT=${17}  # Supleft file in FK domain
if [ -z $FKSupleftOUT ];then FKSupleftOUT=FKSdownright${m}.H;fi
echo FKSupleftOUT=$FKSupleftOUT
# Rdownright in FK
FKRdownrightOUT=${18}  # Rdownright file in FK domain
if [ -z $FKRdownrightOUT ];then FKRdownrightOUT=FKRdownright${m}.H;fi
echo FKRdownrightOUT=$FKRdownrightOUT
# Rupleft in FK
FKRupleftOUT=${19}  # Rupleft file in FK domain
if [ -z $FKRupleftOUT ];then FKRupleftOUT=FKRupleft${m}.H;fi
echo FKRupleftOUT=$FKRupleftOUT

##################
## Optional snapshot files of decomposed wavefields
## these file will keep only real parts of the wavefields after filtering
## output is /z,x,t/
##################
# Sdownright wavefield snapshots
SdownrightOUT=${20}  # Sdownright file
if [ -z $SdownrightOUT ];then SdownrightOUT=Sdownright${m}.H;fi
echo SdownrightOUT=$SdownrightOUT
# Supleft wavefield snapshots
SupleftOUT=${21}  # Supleft file
if [ -z $SupleftOUT ];then SupleftOUT=Sdownright${m}.H;fi
echo SupleftOUT=$SupleftOUT
# Rdownright wavefield snapshots
RdownrightOUT=${22}  # Rdownright file
if [ -z $RdownrightOUT ];then RdownrightOUT=Rdownright${m}.H;fi
echo RdownrightOUT=$RdownrightOUT
# Rupleft wavefield snapshots
RupleftOUT=${23}  # Rupleft file
if [ -z $RupleftOUT ];then RupleftOUT=Rupleft${m}.H;fi
echo RupleftOUT=$RupleftOUT

##################
## Optional parameters for selecting output types of DRTM and imaging condition
## if these parameters exist on the command line, the script will ignore the same parameters already defined in parfile
##################
if [ ! -z ${24} ]; then # if ${24} is a nonzero string (not blank or free-space string), new getonlyrealpart will be assigned
    getonlyrealpart=${24}   # [0] the output is complex-valued, [1] the output is real-valued (only real part of complex-valued output)
fi

if [ ! -z ${25} ]; then # if ${21} is a nonzero string (not blank or free-space string), new decompicon will be assigned
    decompicon=${25}   # [1] Real part is the correlation of real parts, where Imag part is the correlation of imag parts
                       # [2] taking complex conjugate for dsouwvfld in correlation, so the Imaginary part involves the image formation
fi


echo "reading parameters complete..."

##########################################
## IGNORE HERE
##########################################
#m=$1 # name of velocity model is the 1st argument when calling this script
###############################
#jsx=4  # increment between shots along x-axis  (the intershots will be RTMed at the same time)
#tdelay=8 # delayed time in seconds
###############################
#IN=rtm${m}in.H
## output history files
#OUT=rtm${m}out.H
#OUTSdownrightRdownright=rtm${m}out.SdownrightRdownright.H
#OUTSupleftRupleft=rtm${m}out.SupleftRupleft.H
#OUTSdownrightRupleft=rtm${m}out.SdownrightRupleft.H
#OUTSupleftRdownright=rtm${m}out.SupleftRdownright.H
#SOUSNAPOUT=sousnap${m}.H
#RECSNAPOUT=recsnap${m}.H
#SNAPOUT=reflsnap${m}.H
#PARFILE=./Par/rtm0.vel${m}.P
#RTMPROG=./Bin/crtm2d_program.x
#FKDECOMPROG=./Bin/Fkdecomp3d_program.x
#TEMPDIR=./temp
#VELFILE=vel${m}.mig.H
#SOUFILE=sou.HH
##########################################

echo "Start program to compute RTM image in vel1 model..."
echo "Reading shot info..."
nsx=`Get < $IN parform=n n4`
dsx=`Get < $IN parform=n d4`
osx=`Get < $IN parform=n o4`
echo "nsx,dsx,osx,jsx" $nsx $dsx $osx $jsx
nt=`Get < $IN parform=n n3`
dt=`Get < $IN parform=n d3`
ot=`Get < $IN parform=n o3`
echo "nt,dt,ot" $nt $dt $ot
ngx=`Get < $IN parform=n n1`
dgx=`Get < $IN parform=n d1`
ogx=`Get < $IN parform=n o1`
echo "ngx,dgx,ogx" $ngx $dgx $ogx
nz=`Get < $VELFILE parform=n n1`
dz=`Get < $VELFILE parform=n d1`
oz=`Get < $VELFILE parform=n o1`
echo "nz,dz,oz" $nz $dz $oz
nx=`Get < $VELFILE parform=n n2`
dx=`Get < $VELFILE parform=n d2`
ox=`Get < $VELFILE parform=n o2`
echo "nx,dx,ox" $nx $dx $ox

######################
## reorganizing some parameters
###### wavefield time step should equal to jt_image step
jtsnaprtm=${jt_image}
echo "jt_image=${jt_image}, and jtsnaprtm=${jtsnaprtm}"

echo "Input parameters for Padntape program:"
echo ".... beg1=${beg1}, beg2=${beg2}, extend=${extend} taper=${taper}"
PADTAPE="${PADNTAPEPROG} n1out=$(((${nt}-1)/${jtsnaprtm}+1+${beg1}+${beg1})) n2out=$((${nz}+${beg2}+${beg2})) n3out=$((${nx}+${beg3}+${beg3})) beg1=${beg1} beg2=${beg2} beg3=${beg3} extend=${extend} taper=${taper} esize=4"
UNPADTAPE="Window n1=$(((${nt}-1)/${jtsnaprtm}+1)) f1=${beg1} n2=${nz} f2=${beg2} n3=${nx} f3=${beg3}"
UNPADTAPEecho="o1=${ot} o2=${oz} o3={ox}"  # synchronize origin of unpadded tiltdownright wvfld

echo "Input parameters for Fkdecomp3d program:"
echo ".... dim=3 axis1=-1 axis2=1 axis3=1 filter=${filter} alpha=${alpha} phi=${phi} get_filter=${get_filter} get_fkdataout=${get_fkdataout}"
filter3d="filter=${filter} alpha=${alpha} phi=${phi} get_filter=${get_filter} get_fkdataout=${get_fkdataout} verb=${verb}"

###### Feature of decomposed RTM output is only REAL part (DEFAULT) after applying imaging condition or complex-valued file.
echo "getonlyrealpart=${getonlyrealpart}"
if [ ${getonlyrealpart} -eq 0 ]
then
    getonlyrealpart=" "
else
    getonlyrealpart="| Real"
fi

###### Imaging condition for decomposed RTM #####
echo "Declare decomposed RTM imaging condition where file1=dsouwvfld and file2=drecwvfld:"
if [ ${decompicon} -eq 1 ]
then 
    echo "Real part is the correlation of real parts, where Imag part is the correlation of imag parts"
    decompicon="exp_real='file1.re*file2.re' exp_imag='file1.im*file2.im'"
elif [ ${decompicon} -eq 2 ]
then
    #taking complex conjugate for dsouwvfld in correlation, so the Im part involves the image formation
    echo "taking complex conjugate for dsouwvfld in correlation, so the Im part involves the image formation"
    decompicon="exp_real='file1.re*file2.re+file1.im*file2.im' exp_imag='file1.re*file2.im-file1.im*file2.re'"
else
    echo "decompicon=${decompicon} is invalid"
    exit
fi
echo "decompicon=$decompicon"

#######
####################################
## Parameters for selecting decomposed RTM components [0 or 1] read from parfile
####################################
echo "get_DRTM_SdownrightRdownright=${get_DRTM_SdownrightRdownright}"
echo "get_DRTM_SupleftRupleft=${get_DRTM_SupleftRupleft}"
echo "get_DRTM_SdownrightRupleft=${get_DRTM_SdownrightRupleft}"
echo "get_DRTM_SupleftRdownright=${get_DRTM_SupleftRdownright}"


###############
## Old definition
##PADTAPE="${PADNTAPEPROG} n1out=$(((${nt}-1)/${jtsnaprtm}+1+100)) n2out=$((${nz}+100)) beg1=50 beg2=50 extend=2 taper=0.01"
##UNPADTAPE="Window n1=$(((${nt}-1)/${jtsnaprtm}+1)) f1=50 n2=${nz} f2=50"
##filter2d="filter=2 phi=10"
##	<$< ${PADTAPE} | Rtoc | $B/Fkdecomp3d_program.x dim=2 axis1=-1 axis2=1 flag=-1 ${filter2d} | Real | ${UNPADTAPE} |Grey ${GreyFFT}|Tube&
##
##  Decomposed RTM imaging condition where file1=decomposed.souwvfld and file2=decomposed.recwvfld .
##  exp_real='file1.re*file2.re+file1.im*file2.im' exp_imag='file1.re*file2.im-file1.im*file2.re'
######

# parameters for conventional RTM 
TEXT0=tmp0rtm${m}.txt
TEXT1=tmp1rtm${m}.txt
TEXT2=tmp2rtm${m}.txt
TEXT3=tmp3rtm${m}.txt
TEXT4=tmp4rtm${m}.txt
TEXT5=tmp5rtm${m}.txt
TEXT6=tmp6rtm${m}.txt
OUT1=rtm${m}out1.H
OUT2=rtm${m}out2.H
OUT3=rtm${m}out3.H
OUT4=rtm${m}out4.H
OUT5=rtm${m}out5.H
OUT6=rtm${m}out6.H

# parameters for RTM from SdownrightRdownright
TEXTSdownrightRdownright0=tmp0rtm${m}SdownrightRdownright.txt
TEXTSdownrightRdownright1=tmp1rtm${m}SdownrightRdownright.txt
TEXTSdownrightRdownright2=tmp2rtm${m}SdownrightRdownright.txt
TEXTSdownrightRdownright3=tmp3rtm${m}SdownrightRdownright.txt
TEXTSdownrightRdownright4=tmp4rtm${m}SdownrightRdownright.txt
TEXTSdownrightRdownright5=tmp5rtm${m}SdownrightRdownright.txt
TEXTSdownrightRdownright6=tmp6rtm${m}SdownrightRdownright.txt
OUTSdownrightRdownright1=rtm${m}out1SdownrightRdownright.H
OUTSdownrightRdownright2=rtm${m}out2SdownrightRdownright.H
OUTSdownrightRdownright3=rtm${m}out3SdownrightRdownright.H
OUTSdownrightRdownright4=rtm${m}out4SdownrightRdownright.H
OUTSdownrightRdownright5=rtm${m}out5SdownrightRdownright.H
OUTSdownrightRdownright6=rtm${m}out6SdownrightRdownright.H

# parameters for RTM from SupleftRupleft
TEXTSupleftRupleft0=tmp0rtm${m}SupleftRupleft.txt
TEXTSupleftRupleft1=tmp1rtm${m}SupleftRupleft.txt
TEXTSupleftRupleft2=tmp2rtm${m}SupleftRupleft.txt
TEXTSupleftRupleft3=tmp3rtm${m}SupleftRupleft.txt
TEXTSupleftRupleft4=tmp4rtm${m}SupleftRupleft.txt
TEXTSupleftRupleft5=tmp5rtm${m}SupleftRupleft.txt
TEXTSupleftRupleft6=tmp6rtm${m}SupleftRupleft.txt
OUTSupleftRupleft1=rtm${m}out1SupleftRupleft.H
OUTSupleftRupleft2=rtm${m}out2SupleftRupleft.H
OUTSupleftRupleft3=rtm${m}out3SupleftRupleft.H
OUTSupleftRupleft4=rtm${m}out4SupleftRupleft.H
OUTSupleftRupleft5=rtm${m}out5SupleftRupleft.H
OUTSupleftRupleft6=rtm${m}out6SupleftRupleft.H

# parameters for RTM from SdownrightRupleft
TEXTSdownrightRupleft0=tmp0rtm${m}SdownrightRupleft.txt
TEXTSdownrightRupleft1=tmp1rtm${m}SdownrightRupleft.txt
TEXTSdownrightRupleft2=tmp2rtm${m}SdownrightRupleft.txt
TEXTSdownrightRupleft3=tmp3rtm${m}SdownrightRupleft.txt
TEXTSdownrightRupleft4=tmp4rtm${m}SdownrightRupleft.txt
TEXTSdownrightRupleft5=tmp5rtm${m}SdownrightRupleft.txt
TEXTSdownrightRupleft6=tmp6rtm${m}SdownrightRupleft.txt
OUTSdownrightRupleft1=rtm${m}out1SdownrightRupleft.H
OUTSdownrightRupleft2=rtm${m}out2SdownrightRupleft.H
OUTSdownrightRupleft3=rtm${m}out3SdownrightRupleft.H
OUTSdownrightRupleft4=rtm${m}out4SdownrightRupleft.H
OUTSdownrightRupleft5=rtm${m}out5SdownrightRupleft.H
OUTSdownrightRupleft6=rtm${m}out6SdownrightRupleft.H

# parameters for RTM from SupleftRdownright
TEXTSupleftRdownright0=tmp0rtm${m}SupleftRdownright.txt
TEXTSupleftRdownright1=tmp1rtm${m}SupleftRdownright.txt
TEXTSupleftRdownright2=tmp2rtm${m}SupleftRdownright.txt
TEXTSupleftRdownright3=tmp3rtm${m}SupleftRdownright.txt
TEXTSupleftRdownright4=tmp4rtm${m}SupleftRdownright.txt
TEXTSupleftRdownright5=tmp5rtm${m}SupleftRdownright.txt
TEXTSupleftRdownright6=tmp6rtm${m}SupleftRdownright.txt
OUTSupleftRdownright1=rtm${m}out1SupleftRdownright.H
OUTSupleftRdownright2=rtm${m}out2SupleftRdownright.H
OUTSupleftRdownright3=rtm${m}out3SupleftRdownright.H
OUTSupleftRdownright4=rtm${m}out4SupleftRdownright.H
OUTSupleftRdownright5=rtm${m}out5SupleftRdownright.H
OUTSupleftRdownright6=rtm${m}out6SupleftRdownright.H

# remove files or text files before launching programs
Rm ${TEMPDIR}/rtm${m}imageshot*.H
Rm ${TEMPDIR}/rtm${m}shotrecord*.H
Rm ${OUT} ${OUTSdownrightRdownright} ${OUTSupleftRupleft} ${OUTSdownrightRupleft} ${OUTSupleftRdownright}
Rm ${OUT1} ${OUTSdownrightRdownright1} ${OUTSupleftRupleft1} ${OUTSdownrightRupleft1} ${OUTSupleftRdownright1}
Rm ${OUT2} ${OUTSdownrightRdownright2} ${OUTSupleftRupleft2} ${OUTSdownrightRupleft2} ${OUTSupleftRdownright2}
Rm ${OUT3} ${OUTSdownrightRdownright3} ${OUTSupleftRupleft3} ${OUTSdownrightRupleft3} ${OUTSupleftRdownright3}
Rm ${OUT4} ${OUTSdownrightRdownright4} ${OUTSupleftRupleft4} ${OUTSdownrightRupleft4} ${OUTSupleftRdownright4}
Rm ${OUT5} ${OUTSdownrightRdownright5} ${OUTSupleftRupleft5} ${OUTSdownrightRupleft5} ${OUTSupleftRdownright5}
Rm ${OUT6} ${OUTSdownrightRdownright6} ${OUTSupleftRupleft6} ${OUTSdownrightRupleft6} ${OUTSupleftRdownright6}
rm -f ${TEXT0} ${TEXTSdownrightRdownright0} ${TEXTSupleftRupleft0} ${TEXTSdownrightRupleft0} ${TEXTSupleftRdownright0}
rm -f ${TEXT1} ${TEXTSdownrightRdownright1} ${TEXTSupleftRupleft1} ${TEXTSdownrightRupleft1} ${TEXTSupleftRdownright1}
rm -f ${TEXT2} ${TEXTSdownrightRdownright2} ${TEXTSupleftRupleft2} ${TEXTSdownrightRupleft2} ${TEXTSupleftRdownright2}
rm -f ${TEXT3} ${TEXTSdownrightRdownright3} ${TEXTSupleftRupleft3} ${TEXTSdownrightRupleft3} ${TEXTSupleftRdownright3}
rm -f ${TEXT4} ${TEXTSdownrightRdownright4} ${TEXTSupleftRupleft4} ${TEXTSdownrightRupleft4} ${TEXTSupleftRdownright4}
rm -f ${TEXT5} ${TEXTSdownrightRdownright5} ${TEXTSupleftRupleft5} ${TEXTSdownrightRupleft5} ${TEXTSupleftRdownright5}
rm -f ${TEXT6} ${TEXTSdownrightRdownright6} ${TEXTSupleftRupleft6} ${TEXTSdownrightRupleft6} ${TEXTSupleftRdownright6}

#<${IN} Transp plane=13 | Transp plane=34 > ${OUT}  # for windowing purpose
#In ${OUT}


###########

echo "5555555555555555555555555555555555555555555555555555555555555555555555"
echo "The script is computing decomposed RTM images of vel${m} from ${jsx} shots at the same time."
echo "5555555555555555555555555555555555555555555555555555555555555555555555"
for((isx=1; $isx<=$nsx; isx=$isx+$jsx))
do # each iteration will do rtm for ${jsx} shots at the same time

    ##############
    ### do Windowing and Transposing input data to enter a conventional RTM program
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]
	then
	    echo "************* Windowing and Transposing SHOT$((${isx}+$k)) *****************"
	    echo "isx,osx,sx:" $(($isx+$k)) $osx `echo $osx+$(($isx+$k-1))*$dsx | bc`
	    # This set of command will consume 2 cpu's at the same time; one might guess tha the simutaneous loops should not be greater than half of #cpu
	    # BUT computing more than half of #cpu simutaneouly is significantly more effective than limiting ourselves to just 4 simutaneous loops.
	    # SO, what I did here is the best way. No need to window and transpose the whole shots first with 4-simutaneous process before entering RTM with ${jsx}-simutaneous process
	    # Note that calling only-Window commands simutaneously will share the whole processes in a single CPU. BUT calling only-Transp commands simutaneously will separate every single process to a different CPU. SO, if we make Window-Transp dependent to each other, it will behave like only-Transp. That's why I use .... Window | Transp .....
	    <${IN} Window n4=1 f4=$((${isx}+${k}-1)) | Transp plane=12 reshape=2,3,5 > ${TEMPDIR}/drtm${m}shotrecord$((${isx}+${k})).H &
	fi
    done
    # wait untill all background processes to finish
    wait
 
    #############
    #### do conventional RTM imaging and get source and receiver wavefields
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do # run rtm program within each combination of ${jsx} shots
	if [ $((${isx}+${k})) -le ${nsx} ]
	then
	    echo "************* Conventional RTMing SHOT$((${isx}+$k))VEL${m} *****************"
	    echo "**** and getting source & receiver wavefields at all sufficient time steps***"
	    # In the middle of each iteration we need to run the rtm program as background.
            # Every shot gives us snapshot file
	    <${VELFILE} ${RTMPROG} par=${PARFILE} jsnap=${jtsnaprtm} get_snap=0 get_sousnap=1 get_recsnap=1 source=${SOUFILE} receiver=${TEMPDIR}/drtm${m}shotrecord$((${isx}+${k})).H >${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H sousnap=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).H recsnap=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).H &

	    if [ $((${isx}+${k})) -le 20 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT1}
	    elif [ $((${isx}+${k})) -le 40 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT2}
	    elif [ $((${isx}+${k})) -le 60 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT3}
	    elif [ $((${isx}+${k})) -le 80 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT4}
	    elif [ $((${isx}+${k})) -le 100 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT5}
	    else
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT6}
	    fi
	fi
    done
    # wait untill all background processes to finish
    wait

    #############
    #### remove shot records that have been used.
    echo "removing shot records that have been used..."
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/drtm${m}shotrecord$((${isx}+$k)).H
	fi
    done

#############################################################
### after this line it is all about decomposition and DRTM
#############################################################
if [ ${get_DRTM_SdownrightRdownright} -eq 1 -o ${get_DRTM_SupleftRupleft} -eq 1 -o ${get_DRTM_SdownrightRupleft} -eq 1 -o ${get_DRTM_SupleftRdownright} -eq 1 ]; then

    ##################################################
    #####***************************************######
    #### decompose source wavefield
    ##################################################
    echo "decomposing source wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
    #### Sorting the source wavefield ready for decomposition
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).H Transp plane=12 reshape=2,3 >${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H &
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed source wavefields before reordered. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).H
	fi
    done

    #####################################
    ## Preparing input for decomposing source wavefield Supleft&Sdownright
    #####################################
    # convert from real to complex input for Supleft and Sdownright
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H ${PADTAPE} | Rtoc >${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H &
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed source wavefields that have been used to separate wavefields. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H
	fi
    done

    #####################################
    #### select upleftgoing source wavefield Supleft if needed
    #####################################
    if [ ${get_DRTM_SupleftRupleft} -eq 1 -o ${get_DRTM_SupleftRdownright} -eq 1 ]; then
	echo "Getting upleftgoing source wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp3d program for Supleft
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=-1 ${filter3d} >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H fkdataout=${FKSupleftOUT} &
		else
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=-1 ${filter3d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H ${UNPADTAPE} > ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H &
	    fi
	done
        # wait untill all background processes to finish
	wait
	
        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpSupleft
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H
	    fi
	done
	
    fi
    #####################################

    #####################################
    #### select downrightgoing source wavefield Sdownright if needed
    #####################################
    if [ ${get_DRTM_SdownrightRupleft} -eq 1 -o ${get_DRTM_SdownrightRdownright} -eq 1 ]; then
	echo "Getting downrightgoing source wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp3d program for Sdownright
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=1 ${filter3d} >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H fkdataout=${FKSdownrightOUT} &
		else
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=1 ${filter3d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H ${UNPADTAPE} > ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpSdownright
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H
	    fi
	done

    fi

    ##### remove source wvfld that had been prepared for decomposition
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do #tmpSource
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H
	fi
    done

    #####################################


    ##################################################
    #####***************************************######
    #### decompose receiver wavefield
    ##################################################
    echo "decomposing receiver wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
    #### Sorting the receiver wavefield ready for decomposition
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).H Transp plane=12 reshape=2,3 >${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H &
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed receiver wavefields before reordered. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).H
	fi
    done

    #####################################
    ## Preparing input for decomposing receiver wavefield Rupleft&Rdownright
    #####################################
    # convert from real to complex input for Rupleft and Rdownright
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H ${PADTAPE} | Rtoc >${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H &
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed receiver wavefields that have been used to separate wavefields. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H
	fi
    done

    #####################################
    #### select upleftgoing receiver wavefield Rupleft if needed
    #####################################
    if [ ${get_DRTM_SdownrightRupleft} -eq 1 -o ${get_DRTM_SupleftRupleft} -eq 1 ]; then
	echo "Getting upleftgoing receiver wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp3d program for Rupleft
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=-1 ${filter3d} >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H fkdataout=${FKRupleftOUT} &
		else
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=-1 ${filter3d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H ${UNPADTAPE} > ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpRupleft
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H
	    fi
	done

    fi
    #####################################

    #####################################
    #### select downrightgoing receiver wavefield Rdownright
    #####################################
    if [ ${get_DRTM_SdownrightRdownright} -eq 1 -o ${get_DRTM_SupleftRdownright} -eq 1 ]; then
	echo "Getting downrightgoing receiver wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp3d program for Rdownright
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
        do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=1 ${filter3d} >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H fkdataout=${FKRdownrightOUT} &
		else
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=1 ${filter3d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H ${UNPADTAPE} > ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpRdownright
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H
	    fi
	done

    fi

    ##### remove receiver wvfld that had been prepared for decomposition

    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do #tmpReceiver
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.H
	fi
    done

    #####################################


######################
######################
    
     #########################################################################################
    #### apply RTM imaging condition to all combinations of source and receiver wavefields ####
     #########################################################################################
    
         # Math file1=souwvfld$*.tiltdown.fkffto.downright.H file2=recwvfld$*.tiltdown.fkffto.downright.H exp_real='file1.re*file2.re+file1.im*file2.im' exp_imag='file1.re*file2.im-file1.im*file2.re' >$@
         # <$< Transp plane=12 | Stack axes=2 | Real >$@ # sum over time
    ##########################################
    #### Getting RTM image from SdownrightRdownright components
    ##########################################
    if [ ${get_DRTM_SdownrightRdownright} -eq 1 ]; then
	echo "RTM from SdownrightRdownright SHOT${isx} to SHOT$((${isx}+${jsx}-1)).."
        # RTM from Sdownright*Rdownright
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRdownright.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRdownright.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRdownright.H &
	        # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRdownright.H" >> ${TEXTSdownrightRdownright1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRdownright.H" >> ${TEXTSdownrightRdownright2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRdownright.H" >> ${TEXTSdownrightRdownright3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRdownright.H" >> ${TEXTSdownrightRdownright4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRdownright.H" >> ${TEXTSdownrightRdownright5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRdownright.H" >> ${TEXTSdownrightRdownright6}
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SdownrightRdownright
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRdownright.H
	    fi
	done

    fi
    ##########################################

    ##########################################
    ## RTM from Supleft*Rupleft
    ##########################################
    if [ ${get_DRTM_SupleftRupleft} -eq 1 ]; then
	echo "RTM from SupleftRupleft SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRupleft.H &
	    fi
	done
    # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRupleft.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRupleft.H &
	    # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRupleft.H" >> ${TEXTSupleftRupleft1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRupleft.H" >> ${TEXTSupleftRupleft2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRupleft.H" >> ${TEXTSupleftRupleft3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRupleft.H" >> ${TEXTSupleftRupleft4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRupleft.H" >> ${TEXTSupleftRupleft5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRupleft.H" >> ${TEXTSupleftRupleft6}
		fi
	    fi
	done
    # wait untill all background processes to finish
	wait
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SupleftRupleft
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRupleft.H
	fi
	done

    fi
    ##########################################

    ##########################################
    ### RTM from Sdownright*Rupleft
    ##########################################
    if [ ${get_DRTM_SdownrightRupleft} -eq 1 ]; then
	echo "RTM from SdownrightRupleft SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRupleft.H &
	    fi
	done
    # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRupleft.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRupleft.H &
	    # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRupleft.H" >> ${TEXTSdownrightRupleft1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRupleft.H" >> ${TEXTSdownrightRupleft2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRupleft.H" >> ${TEXTSdownrightRupleft3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRupleft.H" >> ${TEXTSdownrightRupleft4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRupleft.H" >> ${TEXTSdownrightRupleft5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRupleft.H" >> ${TEXTSdownrightRupleft6}
		fi
	    fi
	done
    # wait untill all background processes to finish
	wait
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SupleftRupleft
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltdown.SdownrightRupleft.H
	    fi
	done

    fi
    ##########################################

    ##########################################
    #### RTM from Supleft*Rdownright
    ##########################################
    if [ ${get_DRTM_SupleftRdownright} -eq 1 ]; then
	echo "RTM from SupleftRdownright SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRdownright.H &
	    fi
	done
    # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRdownright.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRdownright.H &
	    # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRdownright.H" >> ${TEXTSupleftRdownright1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRdownright.H" >> ${TEXTSupleftRdownright2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRdownright.H" >> ${TEXTSupleftRdownright3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRdownright.H" >> ${TEXTSupleftRdownright4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRdownright.H" >> ${TEXTSupleftRdownright5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRdownright.H" >> ${TEXTSupleftRdownright6}
		fi
	    fi
	done
    # wait untill all background processes to finish
	wait
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SupleftRupleft
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltdown.SupleftRdownright.H
	    fi
	done

    fi
    ##########################################

    #############
    #### remove source wavefield and receiver wavefield that have been used to compute RTM images. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    if [ $((${isx}+${k})) -eq 1 ]; then
	        ## output the decomposed source- and receiver wavefield snapshots of the first shot
		<${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H Transp reshape=1,3 plane=12 | Real > ${SupleftOUT} &
		<${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H Transp reshape=1,3 plane=12 | Real > ${SdownrightOUT} &
		<${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H Transp reshape=1,3 plane=12 | Real > ${RupleftOUT} &
		<${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H Transp reshape=1,3 plane=12 | Real > ${RdownrightOUT} &
		wait
	    fi

	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H &
	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H &
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.upleft.H &
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown.downright.H
	fi
    done

fi
#############################################################
### above this line it is all about decomposition and DRTM
#############################################################

    #############
    #### remove all source and receiver wavefields records that have been used. 
    #for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    #do
    #	if [ $((${isx}+${k})) -le ${nsx} ]; then
    #	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown*.H
    #	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltdown*.H
    #	fi
    #done

done

###########################################
### Combine the conventional RTM images
###########################################
echo "================================="
echo "combining conventional RTM images"
echo "================================="
if [ -f ${TEXT1} ]; then
    rtmlist1=`cat ${TEXT1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUT1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT1}
    echo ${OUT1} >> ${TEXT0}
    runloop=1
fi
if [ -f ${TEXT2} ]; then
    rtmlist2=`cat ${TEXT2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUT2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT2}
    echo ${OUT2} >> ${TEXT0}
    runloop=2
fi
if [ -f ${TEXT3} ]; then
    rtmlist3=`cat ${TEXT3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUT3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT3}
    echo ${OUT3} >> ${TEXT0}
    runloop=3
fi
if [ -f ${TEXT4} ]; then
    rtmlist4=`cat ${TEXT4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUT4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT4}
    echo ${OUT4} >> ${TEXT0}
    runloop=4
fi
if [ -f ${TEXT5} ]; then
    rtmlist5=`cat ${TEXT5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUT5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT5}
    echo ${OUT5} >> ${TEXT0}
    runloop=5
fi
if [ -f ${TEXT6} ]; then
    rtmlist6=`cat ${TEXT6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUT6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT6}
    echo ${OUT6} >> ${TEXT0}
    runloop=6
fi
#######################
## generate conventional RTM images
#######################
if [ -f ${TEXT0} ]; then
    rtmlist0=`cat ${TEXT0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23|Stack axes=2 > ${OUT}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUT1} ${OUT2} ${OUT3} ${OUT4} ${OUT5} ${OUT6}
    rm -f ${TEXT0} ${TEXT1} ${TEXT2} ${TEXT3} ${TEXT4} ${TEXT5} ${TEXT6}
fi

###########################################
### Combine the RTM images from SdownrightRdownright
###########################################
echo "===================================="
echo "combining RTM images from SdownrightRdownright"
echo "===================================="
if [ -f ${TEXTSdownrightRdownright1} ]; then
    rtmlist1=`cat ${TEXTSdownrightRdownright1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSdownrightRdownright1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownrightRdownright1}
    echo ${OUTSdownrightRdownright1} >> ${TEXTSdownrightRdownright0}
    runloop=1
fi
if [ -f ${TEXTSdownrightRdownright2} ]; then
    rtmlist2=`cat ${TEXTSdownrightRdownright2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSdownrightRdownright2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownrightRdownright2}
    echo ${OUTSdownrightRdownright2} >> ${TEXTSdownrightRdownright0}
    runloop=2
fi
if [ -f ${TEXTSdownrightRdownright3} ]; then
    rtmlist3=`cat ${TEXTSdownrightRdownright3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSdownrightRdownright3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownrightRdownright3}
    echo ${OUTSdownrightRdownright3} >> ${TEXTSdownrightRdownright0}
    runloop=3
fi
if [ -f ${TEXTSdownrightRdownright4} ]; then
    rtmlist4=`cat ${TEXTSdownrightRdownright4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSdownrightRdownright4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownrightRdownright4}
    echo ${OUTSdownrightRdownright4} >> ${TEXTSdownrightRdownright0}
    runloop=4
fi
if [ -f ${TEXTSdownrightRdownright5} ]; then
    rtmlist5=`cat ${TEXTSdownrightRdownright5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSdownrightRdownright5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownrightRdownright5}
    echo ${OUTSdownrightRdownright5} >> ${TEXTSdownrightRdownright0}
    runloop=5
fi
if [ -f ${TEXTSdownrightRdownright6} ]; then
    rtmlist6=`cat ${TEXTSdownrightRdownright6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSdownrightRdownright6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownrightRdownright6}
    echo ${OUTSdownrightRdownright6} >> ${TEXTSdownrightRdownright0}
    runloop=6
fi

#######################
## generate DRTM images from SdownrightRdownright
#######################
if [ -f ${TEXTSdownrightRdownright0} ]; then
    rtmlist0=`cat ${TEXTSdownrightRdownright0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2 > ${OUTSdownrightRdownright}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownrightRdownright}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSdownrightRdownright1} ${OUTSdownrightRdownright2} ${OUTSdownrightRdownright3} ${OUTSdownrightRdownright4} ${OUTSdownrightRdownright5} ${OUTSdownrightRdownright6}
    rm -f ${TEXTSdownrightRdownright0} ${TEXTSdownrightRdownright1} ${TEXTSdownrightRdownright2} ${TEXTSdownrightRdownright3} ${TEXTSdownrightRdownright4} ${TEXTSdownrightRdownright5} ${TEXTSdownrightRdownright6}
fi


###########################################
### Combine the RTM images from SupleftRupleft
###########################################
echo "===================================="
echo "combining RTM images from SupleftRupleft"
echo "===================================="
if [ -f ${TEXTSupleftRupleft1} ]; then
    rtmlist1=`cat ${TEXTSupleftRupleft1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSupleftRupleft1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupleftRupleft1}
    echo ${OUTSupleftRupleft1} >> ${TEXTSupleftRupleft0}
    runloop=1
fi
if [ -f ${TEXTSupleftRupleft2} ]; then
    rtmlist2=`cat ${TEXTSupleftRupleft2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSupleftRupleft2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupleftRupleft2}
    echo ${OUTSupleftRupleft2} >> ${TEXTSupleftRupleft0}
    runloop=2
fi
if [ -f ${TEXTSupleftRupleft3} ]; then
    rtmlist3=`cat ${TEXTSupleftRupleft3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSupleftRupleft3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupleftRupleft3}
    echo ${OUTSupleftRupleft3} >> ${TEXTSupleftRupleft0}
    runloop=3
fi
if [ -f ${TEXTSupleftRupleft4} ]; then
    rtmlist4=`cat ${TEXTSupleftRupleft4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSupleftRupleft4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupleftRupleft4}
    echo ${OUTSupleftRupleft4} >> ${TEXTSupleftRupleft0}
    runloop=4
fi
if [ -f ${TEXTSupleftRupleft5} ]; then
    rtmlist5=`cat ${TEXTSupleftRupleft5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSupleftRupleft5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupleftRupleft5}
    echo ${OUTSupleftRupleft5} >> ${TEXTSupleftRupleft0}
    runloop=5
fi
if [ -f ${TEXTSupleftRupleft6} ]; then
    rtmlist6=`cat ${TEXTSupleftRupleft6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSupleftRupleft6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupleftRupleft6}
    echo ${OUTSupleftRupleft6} >> ${TEXTSupleftRupleft0}
    runloop=6
fi

#######################
## generate DRTM images from SupleftRupleft
#######################
if [ -f ${TEXTSupleftRupleft0} ]; then
    rtmlist0=`cat ${TEXTSupleftRupleft0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2 > ${OUTSupleftRupleft}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupleftRupleft}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSupleftRupleft1} ${OUTSupleftRupleft2} ${OUTSupleftRupleft3} ${OUTSupleftRupleft4} ${OUTSupleftRupleft5} ${OUTSupleftRupleft6}
    rm -f ${TEXTSupleftRupleft0} ${TEXTSupleftRupleft1} ${TEXTSupleftRupleft2} ${TEXTSupleftRupleft3} ${TEXTSupleftRupleft4} ${TEXTSupleftRupleft5} ${TEXTSupleftRupleft6}
fi

###########################################
### Combine the RTM images from SdownrightRupleft
###########################################
echo "===================================="
echo "combining RTM images from SdownrightRupleft"
echo "===================================="
if [ -f ${TEXTSdownrightRupleft1} ]; then
    rtmlist1=`cat ${TEXTSdownrightRupleft1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSdownrightRupleft1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownrightRupleft1}
    echo ${OUTSdownrightRupleft1} >> ${TEXTSdownrightRupleft0}
    runloop=1
fi
if [ -f ${TEXTSdownrightRupleft2} ]; then
    rtmlist2=`cat ${TEXTSdownrightRupleft2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSdownrightRupleft2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownrightRupleft2}
    echo ${OUTSdownrightRupleft2} >> ${TEXTSdownrightRupleft0}
    runloop=2
fi
if [ -f ${TEXTSdownrightRupleft3} ]; then
    rtmlist3=`cat ${TEXTSdownrightRupleft3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSdownrightRupleft3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownrightRupleft3}
    echo ${OUTSdownrightRupleft3} >> ${TEXTSdownrightRupleft0}
    runloop=3
fi
if [ -f ${TEXTSdownrightRupleft4} ]; then
    rtmlist4=`cat ${TEXTSdownrightRupleft4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSdownrightRupleft4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownrightRupleft4}
    echo ${OUTSdownrightRupleft4} >> ${TEXTSdownrightRupleft0}
    runloop=4
fi
if [ -f ${TEXTSdownrightRupleft5} ]; then
    rtmlist5=`cat ${TEXTSdownrightRupleft5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSdownrightRupleft5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownrightRupleft5}
    echo ${OUTSdownrightRupleft5} >> ${TEXTSdownrightRupleft0}
    runloop=5
fi
if [ -f ${TEXTSdownrightRupleft6} ]; then
    rtmlist6=`cat ${TEXTSdownrightRupleft6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSdownrightRupleft6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownrightRupleft6}
    echo ${OUTSdownrightRupleft6} >> ${TEXTSdownrightRupleft0}
    runloop=6
fi
#######################
## generate DRTM images from SdownrightRupleft
#######################
if [ -f ${TEXTSdownrightRupleft0} ]; then
    rtmlist0=`cat ${TEXTSdownrightRupleft0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2 > ${OUTSdownrightRupleft}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownrightRupleft}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSdownrightRupleft1} ${OUTSdownrightRupleft2} ${OUTSdownrightRupleft3} ${OUTSdownrightRupleft4} ${OUTSdownrightRupleft5} ${OUTSdownrightRupleft6}
    rm -f ${TEXTSdownrightRupleft0} ${TEXTSdownrightRupleft1} ${TEXTSdownrightRupleft2} ${TEXTSdownrightRupleft3} ${TEXTSdownrightRupleft4} ${TEXTSdownrightRupleft5} ${TEXTSdownrightRupleft6}
fi

###########################################
### Combine the RTM images from SupleftRdownright
###########################################
echo "===================================="
echo "combining RTM images from SupleftRdownright"
echo "===================================="
if [ -f ${TEXTSupleftRdownright1} ]; then
    rtmlist1=`cat ${TEXTSupleftRdownright1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSupleftRdownright1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupleftRdownright1}
    echo ${OUTSupleftRdownright1} >> ${TEXTSupleftRdownright0}
    runloop=1
fi
if [ -f ${TEXTSupleftRdownright2} ]; then
    rtmlist2=`cat ${TEXTSupleftRdownright2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSupleftRdownright2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupleftRdownright2}
    echo ${OUTSupleftRdownright2} >> ${TEXTSupleftRdownright0}
    runloop=2
fi
if [ -f ${TEXTSupleftRdownright3} ]; then
    rtmlist3=`cat ${TEXTSupleftRdownright3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSupleftRdownright3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupleftRdownright3}
    echo ${OUTSupleftRdownright3} >> ${TEXTSupleftRdownright0}
    runloop=3
fi
if [ -f ${TEXTSupleftRdownright4} ]; then
    rtmlist4=`cat ${TEXTSupleftRdownright4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSupleftRdownright4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupleftRdownright4}
    echo ${OUTSupleftRdownright4} >> ${TEXTSupleftRdownright0}
    runloop=4
fi
if [ -f ${TEXTSupleftRdownright5} ]; then
    rtmlist5=`cat ${TEXTSupleftRdownright5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSupleftRdownright5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupleftRdownright5}
    echo ${OUTSupleftRdownright5} >> ${TEXTSupleftRdownright0}
    runloop=5
fi
if [ -f ${TEXTSupleftRdownright6} ]; then
    rtmlist6=`cat ${TEXTSupleftRdownright6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSupleftRdownright6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupleftRdownright6}
    echo ${OUTSupleftRdownright6} >> ${TEXTSupleftRdownright0}
    runloop=6
fi

#######################
## generate DRTM images from SupleftRdownright
#######################
if [ -f ${TEXTSupleftRdownright0} ]; then
    rtmlist0=`cat ${TEXTSupleftRdownright0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2 > ${OUTSupleftRdownright}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSupleftRdownright}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSupleftRdownright1} ${OUTSupleftRdownright2} ${OUTSupleftRdownright3} ${OUTSupleftRdownright4} ${OUTSupleftRdownright5} ${OUTSupleftRdownright6}
    rm -f ${TEXTSupleftRdownright0} ${TEXTSupleftRdownright1} ${TEXTSupleftRdownright2} ${TEXTSupleftRdownright3} ${TEXTSupleftRdownright4} ${TEXTSupleftRdownright5} ${TEXTSupleftRdownright6}
fi

## remove all RTM images from all shots
# Rm ${TEMPDIR}/drtm${m}imageshot*.H

exit
