#!/bin/sh
# File: drtm2d-tiltup.sh
# Script file to parallelly compute RTM image using FK decomposition techniques for upright- and downleft-going (tiltup) components (computing every ${jsx} shots along x-axis at the same time) 
# Input file: "rtm${m}in.H" (interpolated shot gathers)
# axes: 1=rec_z, 2=rec_x, 3=time, 4=sou_x, 5=sou_z
# Output file: "rtm${m}out.H"
# axes: 1=z, 2=x, 3=Undefined

###############################
### How to use this script in makefile???:
# drtmout$(m).H drtmout$(m).SuprightRupright.H drtmout$(m).SdownleftRdownleft.H drtmout$(m).SuprightRdownleft.H drtmout$(m).SdownleftRupright.H: inputdata$(m).H velfile.H soufile.H Par/drtm0.vel.P ./Bin/crtm2d_program.x ./Bin/Fkdecomp3d_program.x ./Bin/Padntape_program.x
#       sh drtm2d-tiltup.sh '$(m)' 'inputdata$(m).H' 'drtmout$(m).H' 'drtmout$(m).SuprightRupright.H' 'drtmout$(m).SdownleftRdownleft.H' 'drtmout$(m).SuprightRdownleft.H' 'drtmout$(m).SdownleftRupright.H' 'conventional_rtm2d_program.x' 'Fkdecomp3d_program.x' 'Padntape_beforeFFT.x' 'vel.mig.H' 'soufile' 'parfile' 'jsx=4' 'tmpdir=.' 'FKSuprightOUT.H' 'FKSdownleftOUT.H' 'FKRuprightOUT.H' 'FKRdownleftOUT.H' 'SuprightOUT.H' 'SdownleftOUT.H' 'RuprightOUT.H' 'RdownleftOUT.H' 'getonlyrealpart' 'decompicon'

###############################

###############################
## Note that parfile for DRTM definitely contains all parameters for calling the conventional RTM program which is able to produce all wavefields at every time steps.
## In addition, this parfile must contain parameters for Fk-decomposition and Padding as follows:
#
# For Padntape program we need:
#              beg1, beg2, extend(optional), taper(optional)
#              [ I might suggest to use "beg1=50 beg2=50 extend=2 taper=0.01" ]
#              n1out,n2out will already be computed in this script.
#
# For Fkdecomp3d program, we always use dim=2 axis1=-1 axis2=1 we need:
#              filter(optional), alpha(optional,available if using filter=2), phi(optional,available if using filter=2), get_filter, get_fkdataout
#              [ I might suggest to use "filter=2 alpha=0.2 phi=10" ]
#        NOTE that if get_*=y, only those auxiliary data from the 1st shot will be produced
#
# For selecting types of decomposed RTM output:
#              getonlyrealpart = 1, default    # the output is real-valued (only real part of complex-valued output)
#              getonlyrealpart = 0             # the output is complex-valued
#
# For selecting imaging condition for decomposed RTM (must be declared):
#              decompicon = 1          # Real part is the correlation of real parts, where Imag part is the correlation of imag parts
#              decompicon = 2          # taking complex conjugate for dsouwvfld in correlation, so the Imaginary part involves the image formation
###############################

## use double quatation marks "$NAME" to view the variable's contents
## single quatation marks '$NAME' does not show the contents of NAME 

echo "calling $0 script"
echo "reading parameters..."

m=$1         #suffix names for showong identity of processing
echo m=$m

IN=$2       #input file /gz,gx,t,sx,sz/ with interpreted gx
if [ -z $IN ];then echo 'IN=$2 does not exist';exit;fi #test for a zero-length string
echo IN=$IN

OUT=$3       #conventional rtm image file /z,x/
if [ -z $OUT ];then echo 'OUT=$3 does not exist';exit;fi
echo OUT=$OUT

OUTSuprightRupright=$4      # rtm image from SuprightRupright
if [ -z $OUTSuprightRupright ];then echo 'OUTSuprightRupright=$4 does not exist';exit;fi
echo OUTSuprightRupright=$OUTSuprightRupright

OUTSdownleftRdownleft=$5
if [ -z $OUTSdownleftRdownleft ];then echo 'OUTSdownleftRdownleft=$5 does not exist';exit;fi
echo OUTSdownleftRdownleft=$OUTSdownleftRdownleft

OUTSuprightRdownleft=$6
if [ -z $OUTSuprightRdownleft ];then echo 'OUTSuprightRdownleft=$6 does not exist';exit;fi
echo OUTSuprightRdownleft=$OUTSuprightRdownleft

OUTSdownleftRupright=$7
if [ -z $OUTSdownleftRupright ];then echo 'OUTSdownleftRupright=$7 does not exist';exit;fi
echo OUTSdownleftRupright=$OUTSdownleftRupright

RTMPROG=$8 # rtm program
if [ -z $RTMPROG ];then echo 'RTMPROG=$8 does not exist';exit;fi
echo RTMPROG=$RTMPROG

FKDECOMPROG=$9 # Fkdecomposition program
if [ -z $FKDECOMPROG ];then echo 'FKDECOMPROG=$9 does not exist';exit;fi
echo FKDECOMPROG=$FKDECOMPROG

PADNTAPEPROG=${10} # Pad and tape prgram before doing FFT
if [ -z $PADNTAPEPROG ];then echo 'PADNTAPEPROG=$8{10} does not exist';exit;fi
echo PADNTAPEPROG=$PADNTAPEPROG

VELFILE=${11}   #velocity file
if [ -z $VELFILE ];then echo 'VELFILE=${11} does not exist';exit;fi
echo VELFILE=$VELFILE

SOUFILE=${12}   #source wavelet file
if [ -z $SOUFILE ];then echo 'SOUFILE=${12} does not exist';exit;fi
echo SOUFILE=$SOUFILE

PARFILE=${13}   #parameter file for RTM program
if [ -z $PARFILE ];then echo 'PARFILE=${13} does not exist';exit;fi
echo PARFILE=$PARFILE

################################
## Read parameters from parfile to this shell script
echo "Reading parameters from the paramter file"
. ${PARFILE}
################################

jsx=${14}       #increment between shots along x-axis  (the number of shot record to be muted in the same time)
if [ -z $jsx ];then jsx=4;fi
echo jsx=$jsx

TEMPDIR=${15}   # temp file to store temporary data
if [ -z $TEMPDIR ];then TEMPDIR=.;fi
echo TEMPDIR=$TEMPDIR

##################
## Optional filtering results in FK domain
##################
# Supright in FK
FKSuprightOUT=${16}  # Supright file in FK domain
if [ -z $FKSuprightOUT ];then FKSuprightOUT=FKSupright${m}.H;fi
echo FKSuprightOUT=$FKSuprightOUT
# Sdownleft in FK
FKSdownleftOUT=${17}  # Sdownleft file in FK domain
if [ -z $FKSdownleftOUT ];then FKSdownleftOUT=FKSupright${m}.H;fi
echo FKSdownleftOUT=$FKSdownleftOUT
# Rupright in FK
FKRuprightOUT=${18}  # Rupright file in FK domain
if [ -z $FKRuprightOUT ];then FKRuprightOUT=FKRupright${m}.H;fi
echo FKRuprightOUT=$FKRuprightOUT
# Rdownleft in FK
FKRdownleftOUT=${19}  # Rdownleft file in FK domain
if [ -z $FKRdownleftOUT ];then FKRdownleftOUT=FKRdownleft${m}.H;fi
echo FKRdownleftOUT=$FKRdownleftOUT

##################
## Optional snapshot files of decomposed wavefields
## these file will keep only real parts of the wavefields after filtering
## output is /z,x,t/
##################
# Supright wavefield snapshots
SuprightOUT=${20}  # Supright file
if [ -z $SuprightOUT ];then SuprightOUT=Supright${m}.H;fi
echo SuprightOUT=$SuprightOUT
# Sdownleft wavefield snapshots
SdownleftOUT=${21}  # Sdownleft file
if [ -z $SdownleftOUT ];then SdownleftOUT=Supright${m}.H;fi
echo SdownleftOUT=$SdownleftOUT
# Rupright wavefield snapshots
RuprightOUT=${22}  # Rupright file
if [ -z $RuprightOUT ];then RuprightOUT=Rupright${m}.H;fi
echo RuprightOUT=$RuprightOUT
# Rdownleft wavefield snapshots
RdownleftOUT=${23}  # Rdownleft file
if [ -z $RdownleftOUT ];then RdownleftOUT=Rdownleft${m}.H;fi
echo RdownleftOUT=$RdownleftOUT

##################
## Optional parameters for selecting output types of DRTM and imaging condition
## if these parameters exist on the command line, the script will ignore the same parameters already defined in parfile
##################
if [ ! -z ${24} ]; then # if ${24} is a nonzero string (not blank or free-space string), new getonlyrealpart will be assigned
    getonlyrealpart=${24}   # [0] the output is complex-valued, [1] the output is real-valued (only real part of complex-valued output)
fi

if [ ! -z ${25} ]; then # if ${21} is a nonzero string (not blank or free-space string), new decompicon will be assigned
    decompicon=${25}   # [1] Real part is the correlation of real parts, where Imag part is the correlation of imag parts
                       # [2] taking complex conjugate for dsouwvfld in correlation, so the Imaginary part involves the image formation
fi


echo "reading parameters complete..."

##########################################
## IGNORE HERE
##########################################
#m=$1 # name of velocity model is the 1st argument when calling this script
###############################
#jsx=4  # increment between shots along x-axis  (the intershots will be RTMed at the same time)
#tdelay=8 # delayed time in seconds
###############################
#IN=rtm${m}in.H
## output history files
#OUT=rtm${m}out.H
#OUTSuprightRupright=rtm${m}out.SuprightRupright.H
#OUTSdownleftRdownleft=rtm${m}out.SdownleftRdownleft.H
#OUTSuprightRdownleft=rtm${m}out.SuprightRdownleft.H
#OUTSdownleftRupright=rtm${m}out.SdownleftRupright.H
#SOUSNAPOUT=sousnap${m}.H
#RECSNAPOUT=recsnap${m}.H
#SNAPOUT=reflsnap${m}.H
#PARFILE=./Par/rtm0.vel${m}.P
#RTMPROG=./Bin/crtm2d_program.x
#FKDECOMPROG=./Bin/Fkdecomp3d_program.x
#TEMPDIR=./temp
#VELFILE=vel${m}.mig.H
#SOUFILE=sou.HH
##########################################

echo "Start program to compute RTM image in vel1 model..."
echo "Reading shot info..."
nsx=`Get < $IN parform=n n4`
dsx=`Get < $IN parform=n d4`
osx=`Get < $IN parform=n o4`
echo "nsx,dsx,osx,jsx" $nsx $dsx $osx $jsx
nt=`Get < $IN parform=n n3`
dt=`Get < $IN parform=n d3`
ot=`Get < $IN parform=n o3`
echo "nt,dt,ot" $nt $dt $ot
ngx=`Get < $IN parform=n n1`
dgx=`Get < $IN parform=n d1`
ogx=`Get < $IN parform=n o1`
echo "ngx,dgx,ogx" $ngx $dgx $ogx
nz=`Get < $VELFILE parform=n n1`
dz=`Get < $VELFILE parform=n d1`
oz=`Get < $VELFILE parform=n o1`
echo "nz,dz,oz" $nz $dz $oz
nx=`Get < $VELFILE parform=n n2`
dx=`Get < $VELFILE parform=n d2`
ox=`Get < $VELFILE parform=n o2`
echo "nx,dx,ox" $nx $dx $ox

######################
## reorganizing some parameters
###### wavefield time step should equal to jt_image step
jtsnaprtm=${jt_image}
echo "jt_image=${jt_image}, and jtsnaprtm=${jtsnaprtm}"

echo "Input parameters for Padntape program:"
echo ".... beg1=${beg1}, beg2=${beg2}, extend=${extend} taper=${taper}"
PADTAPE="${PADNTAPEPROG} n1out=$(((${nt}-1)/${jtsnaprtm}+1+${beg1}+${beg1})) n2out=$((${nz}+${beg2}+${beg2})) n3out=$((${nx}+${beg3}+${beg3})) beg1=${beg1} beg2=${beg2} beg3=${beg3} extend=${extend} taper=${taper} esize=4"
UNPADTAPE="Window n1=$(((${nt}-1)/${jtsnaprtm}+1)) f1=${beg1} n2=${nz} f2=${beg2} n3=${nx} f3=${beg3}"
UNPADTAPEecho="o1=${ot} o2=${oz} o3={ox}"  # synchronize origin of unpadded tiltdownright wvfld

echo "Input parameters for Fkdecomp3d program:"
echo ".... dim=3 axis1=-1 axis2=1 axis3=1 filter=${filter} alpha=${alpha} phi=${phi} get_filter=${get_filter} get_fkdataout=${get_fkdataout}"
filter3d="filter=${filter} alpha=${alpha} phi=${phi} get_filter=${get_filter} get_fkdataout=${get_fkdataout} verb=${verb}"

###### Feature of decomposed RTM output is only REAL part (DEFAULT) after applying imaging condition or complex-valued file.
echo "getonlyrealpart=${getonlyrealpart}"
if [ ${getonlyrealpart} -eq 0 ]
then
    getonlyrealpart=" "
else
    getonlyrealpart="| Real"
fi

###### Imaging condition for decomposed RTM #####
echo "Declare decomposed RTM imaging condition where file1=dsouwvfld and file2=drecwvfld:"
if [ ${decompicon} -eq 1 ]
then 
    echo "Real part is the correlation of real parts, where Imag part is the correlation of imag parts"
    decompicon="exp_real='file1.re*file2.re' exp_imag='file1.im*file2.im'"
elif [ ${decompicon} -eq 2 ]
then
    #taking complex conjugate for dsouwvfld in correlation, so the Im part involves the image formation
    echo "taking complex conjugate for dsouwvfld in correlation, so the Im part involves the image formation"
    decompicon="exp_real='file1.re*file2.re+file1.im*file2.im' exp_imag='file1.re*file2.im-file1.im*file2.re'"
else
    echo "decompicon=${decompicon} is invalid"
    exit
fi
echo "decompicon=$decompicon"

#######
####################################
## Parameters for selecting decomposed RTM components [0 or 1] read from parfile
####################################
echo "get_DRTM_SuprightRupright=${get_DRTM_SuprightRupright}"
echo "get_DRTM_SdownleftRdownleft=${get_DRTM_SdownleftRdownleft}"
echo "get_DRTM_SuprightRdownleft=${get_DRTM_SuprightRdownleft}"
echo "get_DRTM_SdownleftRupright=${get_DRTM_SdownleftRupright}"


###############
## Old definition
##PADTAPE="${PADNTAPEPROG} n1out=$(((${nt}-1)/${jtsnaprtm}+1+100)) n2out=$((${nz}+100)) beg1=50 beg2=50 extend=2 taper=0.01"
##UNPADTAPE="Window n1=$(((${nt}-1)/${jtsnaprtm}+1)) f1=50 n2=${nz} f2=50"
##filter2d="filter=2 phi=10"
##	<$< ${PADTAPE} | Rtoc | $B/Fkdecomp3d_program.x dim=2 axis1=-1 axis2=1 flag=-1 ${filter2d} | Real | ${UNPADTAPE} |Grey ${GreyFFT}|Tube&
##
##  Decomposed RTM imaging condition where file1=decomposed.souwvfld and file2=decomposed.recwvfld .
##  exp_real='file1.re*file2.re+file1.im*file2.im' exp_imag='file1.re*file2.im-file1.im*file2.re'
######

# parameters for conventional RTM 
TEXT0=tmp0rtm${m}.txt
TEXT1=tmp1rtm${m}.txt
TEXT2=tmp2rtm${m}.txt
TEXT3=tmp3rtm${m}.txt
TEXT4=tmp4rtm${m}.txt
TEXT5=tmp5rtm${m}.txt
TEXT6=tmp6rtm${m}.txt
OUT1=rtm${m}out1.H
OUT2=rtm${m}out2.H
OUT3=rtm${m}out3.H
OUT4=rtm${m}out4.H
OUT5=rtm${m}out5.H
OUT6=rtm${m}out6.H

# parameters for RTM from SuprightRupright
TEXTSuprightRupright0=tmp0rtm${m}SuprightRupright.txt
TEXTSuprightRupright1=tmp1rtm${m}SuprightRupright.txt
TEXTSuprightRupright2=tmp2rtm${m}SuprightRupright.txt
TEXTSuprightRupright3=tmp3rtm${m}SuprightRupright.txt
TEXTSuprightRupright4=tmp4rtm${m}SuprightRupright.txt
TEXTSuprightRupright5=tmp5rtm${m}SuprightRupright.txt
TEXTSuprightRupright6=tmp6rtm${m}SuprightRupright.txt
OUTSuprightRupright1=rtm${m}out1SuprightRupright.H
OUTSuprightRupright2=rtm${m}out2SuprightRupright.H
OUTSuprightRupright3=rtm${m}out3SuprightRupright.H
OUTSuprightRupright4=rtm${m}out4SuprightRupright.H
OUTSuprightRupright5=rtm${m}out5SuprightRupright.H
OUTSuprightRupright6=rtm${m}out6SuprightRupright.H

# parameters for RTM from SdownleftRdownleft
TEXTSdownleftRdownleft0=tmp0rtm${m}SdownleftRdownleft.txt
TEXTSdownleftRdownleft1=tmp1rtm${m}SdownleftRdownleft.txt
TEXTSdownleftRdownleft2=tmp2rtm${m}SdownleftRdownleft.txt
TEXTSdownleftRdownleft3=tmp3rtm${m}SdownleftRdownleft.txt
TEXTSdownleftRdownleft4=tmp4rtm${m}SdownleftRdownleft.txt
TEXTSdownleftRdownleft5=tmp5rtm${m}SdownleftRdownleft.txt
TEXTSdownleftRdownleft6=tmp6rtm${m}SdownleftRdownleft.txt
OUTSdownleftRdownleft1=rtm${m}out1SdownleftRdownleft.H
OUTSdownleftRdownleft2=rtm${m}out2SdownleftRdownleft.H
OUTSdownleftRdownleft3=rtm${m}out3SdownleftRdownleft.H
OUTSdownleftRdownleft4=rtm${m}out4SdownleftRdownleft.H
OUTSdownleftRdownleft5=rtm${m}out5SdownleftRdownleft.H
OUTSdownleftRdownleft6=rtm${m}out6SdownleftRdownleft.H

# parameters for RTM from SuprightRdownleft
TEXTSuprightRdownleft0=tmp0rtm${m}SuprightRdownleft.txt
TEXTSuprightRdownleft1=tmp1rtm${m}SuprightRdownleft.txt
TEXTSuprightRdownleft2=tmp2rtm${m}SuprightRdownleft.txt
TEXTSuprightRdownleft3=tmp3rtm${m}SuprightRdownleft.txt
TEXTSuprightRdownleft4=tmp4rtm${m}SuprightRdownleft.txt
TEXTSuprightRdownleft5=tmp5rtm${m}SuprightRdownleft.txt
TEXTSuprightRdownleft6=tmp6rtm${m}SuprightRdownleft.txt
OUTSuprightRdownleft1=rtm${m}out1SuprightRdownleft.H
OUTSuprightRdownleft2=rtm${m}out2SuprightRdownleft.H
OUTSuprightRdownleft3=rtm${m}out3SuprightRdownleft.H
OUTSuprightRdownleft4=rtm${m}out4SuprightRdownleft.H
OUTSuprightRdownleft5=rtm${m}out5SuprightRdownleft.H
OUTSuprightRdownleft6=rtm${m}out6SuprightRdownleft.H

# parameters for RTM from SdownleftRupright
TEXTSdownleftRupright0=tmp0rtm${m}SdownleftRupright.txt
TEXTSdownleftRupright1=tmp1rtm${m}SdownleftRupright.txt
TEXTSdownleftRupright2=tmp2rtm${m}SdownleftRupright.txt
TEXTSdownleftRupright3=tmp3rtm${m}SdownleftRupright.txt
TEXTSdownleftRupright4=tmp4rtm${m}SdownleftRupright.txt
TEXTSdownleftRupright5=tmp5rtm${m}SdownleftRupright.txt
TEXTSdownleftRupright6=tmp6rtm${m}SdownleftRupright.txt
OUTSdownleftRupright1=rtm${m}out1SdownleftRupright.H
OUTSdownleftRupright2=rtm${m}out2SdownleftRupright.H
OUTSdownleftRupright3=rtm${m}out3SdownleftRupright.H
OUTSdownleftRupright4=rtm${m}out4SdownleftRupright.H
OUTSdownleftRupright5=rtm${m}out5SdownleftRupright.H
OUTSdownleftRupright6=rtm${m}out6SdownleftRupright.H

# remove files or text files before launching programs
Rm ${TEMPDIR}/rtm${m}imageshot*.H
Rm ${TEMPDIR}/rtm${m}shotrecord*.H
Rm ${OUT} ${OUTSuprightRupright} ${OUTSdownleftRdownleft} ${OUTSuprightRdownleft} ${OUTSdownleftRupright}
Rm ${OUT1} ${OUTSuprightRupright1} ${OUTSdownleftRdownleft1} ${OUTSuprightRdownleft1} ${OUTSdownleftRupright1}
Rm ${OUT2} ${OUTSuprightRupright2} ${OUTSdownleftRdownleft2} ${OUTSuprightRdownleft2} ${OUTSdownleftRupright2}
Rm ${OUT3} ${OUTSuprightRupright3} ${OUTSdownleftRdownleft3} ${OUTSuprightRdownleft3} ${OUTSdownleftRupright3}
Rm ${OUT4} ${OUTSuprightRupright4} ${OUTSdownleftRdownleft4} ${OUTSuprightRdownleft4} ${OUTSdownleftRupright4}
Rm ${OUT5} ${OUTSuprightRupright5} ${OUTSdownleftRdownleft5} ${OUTSuprightRdownleft5} ${OUTSdownleftRupright5}
Rm ${OUT6} ${OUTSuprightRupright6} ${OUTSdownleftRdownleft6} ${OUTSuprightRdownleft6} ${OUTSdownleftRupright6}
rm -f ${TEXT0} ${TEXTSuprightRupright0} ${TEXTSdownleftRdownleft0} ${TEXTSuprightRdownleft0} ${TEXTSdownleftRupright0}
rm -f ${TEXT1} ${TEXTSuprightRupright1} ${TEXTSdownleftRdownleft1} ${TEXTSuprightRdownleft1} ${TEXTSdownleftRupright1}
rm -f ${TEXT2} ${TEXTSuprightRupright2} ${TEXTSdownleftRdownleft2} ${TEXTSuprightRdownleft2} ${TEXTSdownleftRupright2}
rm -f ${TEXT3} ${TEXTSuprightRupright3} ${TEXTSdownleftRdownleft3} ${TEXTSuprightRdownleft3} ${TEXTSdownleftRupright3}
rm -f ${TEXT4} ${TEXTSuprightRupright4} ${TEXTSdownleftRdownleft4} ${TEXTSuprightRdownleft4} ${TEXTSdownleftRupright4}
rm -f ${TEXT5} ${TEXTSuprightRupright5} ${TEXTSdownleftRdownleft5} ${TEXTSuprightRdownleft5} ${TEXTSdownleftRupright5}
rm -f ${TEXT6} ${TEXTSuprightRupright6} ${TEXTSdownleftRdownleft6} ${TEXTSuprightRdownleft6} ${TEXTSdownleftRupright6}

#<${IN} Transp plane=13 | Transp plane=34 > ${OUT}  # for windowing purpose
#In ${OUT}


###########

echo "5555555555555555555555555555555555555555555555555555555555555555555555"
echo "The script is computing decomposed RTM images of vel${m} from ${jsx} shots at the same time."
echo "5555555555555555555555555555555555555555555555555555555555555555555555"
for((isx=1; $isx<=$nsx; isx=$isx+$jsx))
do # each iteration will do rtm for ${jsx} shots at the same time

    ##############
    ### do Windowing and Transposing input data to enter a conventional RTM program
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]
	then
	    echo "************* Windowing and Transposing SHOT$((${isx}+$k)) *****************"
	    echo "isx,osx,sx:" $(($isx+$k)) $osx `echo $osx+$(($isx+$k-1))*$dsx | bc`
	    # This set of command will consume 2 cpu's at the same time; one might guess tha the simutaneous loops should not be greater than half of #cpu
	    # BUT computing more than half of #cpu simutaneouly is significantly more effective than limiting ourselves to just 4 simutaneous loops.
	    # SO, what I did here is the best way. No need to window and transpose the whole shots first with 4-simutaneous process before entering RTM with ${jsx}-simutaneous process
	    # Note that calling only-Window commands simutaneously will share the whole processes in a single CPU. BUT calling only-Transp commands simutaneously will separate every single process to a different CPU. SO, if we make Window-Transp dependent to each other, it will behave like only-Transp. That's why I use .... Window | Transp .....
	    <${IN} Window n4=1 f4=$((${isx}+${k}-1)) | Transp plane=12 reshape=2,3,5 > ${TEMPDIR}/drtm${m}shotrecord$((${isx}+${k})).H &
	fi
    done
    # wait untill all background processes to finish
    wait
 
    #############
    #### do conventional RTM imaging and get source and receiver wavefields
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do # run rtm program within each combination of ${jsx} shots
	if [ $((${isx}+${k})) -le ${nsx} ]
	then
	    echo "************* Conventional RTMing SHOT$((${isx}+$k))VEL${m} *****************"
	    echo "**** and getting source & receiver wavefields at all sufficient time steps***"
	    # In the middle of each iteration we need to run the rtm program as background.
            # Every shot gives us snapshot file
	    <${VELFILE} ${RTMPROG} par=${PARFILE} jsnap=${jtsnaprtm} get_snap=0 get_sousnap=1 get_recsnap=1 source=${SOUFILE} receiver=${TEMPDIR}/drtm${m}shotrecord$((${isx}+${k})).H >${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H sousnap=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).H recsnap=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).H &

	    if [ $((${isx}+${k})) -le 20 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT1}
	    elif [ $((${isx}+${k})) -le 40 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT2}
	    elif [ $((${isx}+${k})) -le 60 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT3}
	    elif [ $((${isx}+${k})) -le 80 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT4}
	    elif [ $((${isx}+${k})) -le 100 ]
	    then
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT5}
	    else
		echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT6}
	    fi
	fi
    done
    # wait untill all background processes to finish
    wait

    #############
    #### remove shot records that have been used.
    echo "removing shot records that have been used..."
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/drtm${m}shotrecord$((${isx}+$k)).H
	fi
    done

#############################################################
### after this line it is all about decomposition and DRTM
#############################################################
if [ ${get_DRTM_SuprightRupright} -eq 1 -o ${get_DRTM_SdownleftRdownleft} -eq 1 -o ${get_DRTM_SuprightRdownleft} -eq 1 -o ${get_DRTM_SdownleftRupright} -eq 1 ]; then

    ##################################################
    #####***************************************######
    #### decompose source wavefield
    ##################################################
    echo "decomposing source wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
    #### Sorting the source wavefield ready for decomposition
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).H Transp plane=12 reshape=2,3 >${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H &
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed source wavefields before reordered. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).H
	fi
    done

    #####################################
    ## Preparing input for decomposing source wavefield Sdownleft&Supright
    #####################################
    # convert from real to complex input for Sdownleft and Supright
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H ${PADTAPE} | Rtoc >${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H &
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed source wavefields that have been used to separate wavefields. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H
	fi
    done

    #####################################
    #### select downleftgoing source wavefield Sdownleft if needed
    #####################################
    if [ ${get_DRTM_SdownleftRdownleft} -eq 1 -o ${get_DRTM_SdownleftRupright} -eq 1 ]; then
	echo "Getting downleftgoing source wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp3d program for Sdownleft
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=-2 ${filter3d} >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H fkdataout=${FKSdownleftOUT} &
		else
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=-2 ${filter3d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H ${UNPADTAPE} > ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H &
	    fi
	done
        # wait untill all background processes to finish
	wait
	
        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpSdownleft
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H
	    fi
	done
	
    fi
    #####################################

    #####################################
    #### select uprightgoing source wavefield Supright if needed
    #####################################
    if [ ${get_DRTM_SuprightRdownleft} -eq 1 -o ${get_DRTM_SuprightRupright} -eq 1 ]; then
	echo "Getting uprightgoing source wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp3d program for Supright
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=2 ${filter3d} >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H fkdataout=${FKSuprightOUT} &
		else
		    <${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=2 ${filter3d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H ${UNPADTAPE} > ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpSupright
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H
	    fi
	done

    fi

    ##### remove source wvfld that had been prepared for decomposition
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do #tmpSource
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/cmplx.sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H
	fi
    done

    #####################################


    ##################################################
    #####***************************************######
    #### decompose receiver wavefield
    ##################################################
    echo "decomposing receiver wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
    #### Sorting the receiver wavefield ready for decomposition
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).H Transp plane=12 reshape=2,3 >${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H &
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed receiver wavefields before reordered. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).H
	fi
    done

    #####################################
    ## Preparing input for decomposing receiver wavefield Rdownleft&Rupright
    #####################################
    # convert from real to complex input for Rdownleft and Rupright
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    <${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H ${PADTAPE} | Rtoc >${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H &
	fi
    done
    # wait untill all background processes to finish
    wait
    #############
    #### remove undecomposed receiver wavefields that have been used to separate wavefields. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H
	fi
    done

    #####################################
    #### select downleftgoing receiver wavefield Rdownleft if needed
    #####################################
    if [ ${get_DRTM_SuprightRdownleft} -eq 1 -o ${get_DRTM_SdownleftRdownleft} -eq 1 ]; then
	echo "Getting downleftgoing receiver wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp3d program for Rdownleft
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=-2 ${filter3d} >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H fkdataout=${FKRdownleftOUT} &
		else
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=-2 ${filter3d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H ${UNPADTAPE} > ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpRdownleft
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H
	    fi
	done

    fi
    #####################################

    #####################################
    #### select uprightgoing receiver wavefield Rupright
    #####################################
    if [ ${get_DRTM_SuprightRupright} -eq 1 -o ${get_DRTM_SdownleftRupright} -eq 1 ]; then
	echo "Getting uprightgoing receiver wavefield SHOT${isx} to SHOT$((${isx}+${jsx}-1))....."
        # apply Fkdecomp3d program for Rupright
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
        do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		if [ $((${isx}+${k})) -eq 1 ]
		then # The first shot might give us filtering output in FK domain
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=2 ${filter3d} >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H fkdataout=${FKRuprightOUT} &
		else
		    <${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H ${FKDECOMPROG} dim=3 axis1=-1 axis2=1 axis3=1 flag=2 ${filter3d} get_filter=n get_fkdataout=n >${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H &
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait
        #
        # unpadding the output filtering result
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H ${UNPADTAPE} > ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

        ## assign the origin of data arrays to be at zero (after we apply padding program)
        ## o1 is time, o2 is depth
        for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		echo ${UNPADTAPEecho} >> ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H
	    fi
	done

        #### remove temporary files that have been used. 
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpRupright
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/filtered.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H
	    fi
	done

    fi

    ##### remove receiver wvfld that had been prepared for decomposition

    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do #tmpReceiver
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/cmplx.recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.H
	fi
    done

    #####################################


######################
######################
    
     #########################################################################################
    #### apply RTM imaging condition to all combinations of source and receiver wavefields ####
     #########################################################################################
    
         # Math file1=souwvfld$*.tiltup.fkffto.upright.H file2=recwvfld$*.tiltup.fkffto.upright.H exp_real='file1.re*file2.re+file1.im*file2.im' exp_imag='file1.re*file2.im-file1.im*file2.re' >$@
         # <$< Transp plane=12 | Stack axes=2 | Real >$@ # sum over time
    ##########################################
    #### Getting RTM image from SuprightRupright components
    ##########################################
    if [ ${get_DRTM_SuprightRupright} -eq 1 ]; then
	echo "RTM from SuprightRupright SHOT${isx} to SHOT$((${isx}+${jsx}-1)).."
        # RTM from Supright*Rupright
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRupright.H &
	    fi
	done
        # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRupright.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRupright.H &
	        # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRupright.H" >> ${TEXTSuprightRupright1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRupright.H" >> ${TEXTSuprightRupright2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRupright.H" >> ${TEXTSuprightRupright3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRupright.H" >> ${TEXTSuprightRupright4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRupright.H" >> ${TEXTSuprightRupright5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRupright.H" >> ${TEXTSuprightRupright6}
		fi
	    fi
	done
        # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SuprightRupright
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRupright.H
	    fi
	done

    fi
    ##########################################

    ##########################################
    ## RTM from Sdownleft*Rdownleft
    ##########################################
    if [ ${get_DRTM_SdownleftRdownleft} -eq 1 ]; then
	echo "RTM from SdownleftRdownleft SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRdownleft.H &
	    fi
	done
    # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRdownleft.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRdownleft.H &
	    # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRdownleft.H" >> ${TEXTSdownleftRdownleft1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRdownleft.H" >> ${TEXTSdownleftRdownleft2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRdownleft.H" >> ${TEXTSdownleftRdownleft3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRdownleft.H" >> ${TEXTSdownleftRdownleft4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRdownleft.H" >> ${TEXTSdownleftRdownleft5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRdownleft.H" >> ${TEXTSdownleftRdownleft6}
		fi
	    fi
	done
    # wait untill all background processes to finish
	wait
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SdownleftRdownleft
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRdownleft.H
	fi
	done

    fi
    ##########################################

    ##########################################
    ### RTM from Supright*Rdownleft
    ##########################################
    if [ ${get_DRTM_SuprightRdownleft} -eq 1 ]; then
	echo "RTM from SuprightRdownleft SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRdownleft.H &
	    fi
	done
    # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRdownleft.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRdownleft.H &
	    # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRdownleft.H" >> ${TEXTSuprightRdownleft1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRdownleft.H" >> ${TEXTSuprightRdownleft2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRdownleft.H" >> ${TEXTSuprightRdownleft3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRdownleft.H" >> ${TEXTSuprightRdownleft4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRdownleft.H" >> ${TEXTSuprightRdownleft5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRdownleft.H" >> ${TEXTSuprightRdownleft6}
		fi
	    fi
	done
    # wait untill all background processes to finish
	wait
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SdownleftRdownleft
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltup.SuprightRdownleft.H
	    fi
	done

    fi
    ##########################################

    ##########################################
    #### RTM from Sdownleft*Rupright
    ##########################################
    if [ ${get_DRTM_SdownleftRupright} -eq 1 ]; then
	echo "RTM from SdownleftRupright SHOT${isx} to SHOT$((${isx}+${jsx}-1))..."
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Math file1=${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H file2=${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H ${decompicon} ${getonlyrealpart} >${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRupright.H &
	    fi
	done
    # wait untill all background processes to finish
	wait

	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		<${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRupright.H Transp plane=12 | Stack axes=2 > ${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRupright.H &
	    # save the list of RTM image to be stacked
		if [ $((${isx}+${k})) -le 20 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRupright.H" >> ${TEXTSdownleftRupright1}
		elif [ $((${isx}+${k})) -le 40 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRupright.H" >> ${TEXTSdownleftRupright2}
		elif [ $((${isx}+${k})) -le 60 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRupright.H" >> ${TEXTSdownleftRupright3}
		elif [ $((${isx}+${k})) -le 80 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRupright.H" >> ${TEXTSdownleftRupright4}
		elif [ $((${isx}+${k})) -le 100 ]
		then
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRupright.H" >> ${TEXTSdownleftRupright5}
		else
		    echo "${TEMPDIR}/drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRupright.H" >> ${TEXTSdownleftRupright6}
		fi
	    fi
	done
    # wait untill all background processes to finish
	wait
	for((k=0; $k<=$((${jsx}-1)); k=$k+1))
	do #tmpfile of RTM from SdownleftRdownleft
	    if [ $((${isx}+${k})) -le ${nsx} ]; then
		Rm ${TEMPDIR}/unstacked.drtm${m}imageshot$((${isx}+${k})).tiltup.SdownleftRupright.H
	    fi
	done

    fi
    ##########################################

    #############
    #### remove source wavefield and receiver wavefield that have been used to compute RTM images. 
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    if [ $((${isx}+${k})) -eq 1 ]; then
	        ## output the decomposed source- and receiver wavefield snapshots of the first shot
		<${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H Transp reshape=1,3 plane=12 | Real > ${SdownleftOUT} &
		<${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H Transp reshape=1,3 plane=12 | Real > ${SuprightOUT} &
		<${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H Transp reshape=1,3 plane=12 | Real > ${RdownleftOUT} &
		<${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H Transp reshape=1,3 plane=12 | Real > ${RuprightOUT} &
		wait
	    fi

	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H &
	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H &
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.downleft.H &
	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup.upright.H
	fi
    done

fi
#############################################################
### above this line it is all about decomposition and DRTM
#############################################################

    #############
    #### remove all source and receiver wavefields records that have been used. 
    #for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    #do
    #	if [ $((${isx}+${k})) -le ${nsx} ]; then
    #	    Rm ${TEMPDIR}/sousnapwvfld${m}shotrecord$((${isx}+${k})).tiltup*.H
    #	    Rm ${TEMPDIR}/recsnapwvfld${m}shotrecord$((${isx}+${k})).tiltup*.H
    #	fi
    #done

done

###########################################
### Combine the conventional RTM images
###########################################
echo "================================="
echo "combining conventional RTM images"
echo "================================="
if [ -f ${TEXT1} ]; then
    rtmlist1=`cat ${TEXT1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUT1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT1}
    echo ${OUT1} >> ${TEXT0}
    runloop=1
fi
if [ -f ${TEXT2} ]; then
    rtmlist2=`cat ${TEXT2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUT2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT2}
    echo ${OUT2} >> ${TEXT0}
    runloop=2
fi
if [ -f ${TEXT3} ]; then
    rtmlist3=`cat ${TEXT3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUT3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT3}
    echo ${OUT3} >> ${TEXT0}
    runloop=3
fi
if [ -f ${TEXT4} ]; then
    rtmlist4=`cat ${TEXT4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUT4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT4}
    echo ${OUT4} >> ${TEXT0}
    runloop=4
fi
if [ -f ${TEXT5} ]; then
    rtmlist5=`cat ${TEXT5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUT5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT5}
    echo ${OUT5} >> ${TEXT0}
    runloop=5
fi
if [ -f ${TEXT6} ]; then
    rtmlist6=`cat ${TEXT6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUT6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT6}
    echo ${OUT6} >> ${TEXT0}
    runloop=6
fi
#######################
## generate conventional RTM images
#######################
if [ -f ${TEXT0} ]; then
    rtmlist0=`cat ${TEXT0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23|Stack axes=2 > ${OUT}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUT1} ${OUT2} ${OUT3} ${OUT4} ${OUT5} ${OUT6}
    rm -f ${TEXT0} ${TEXT1} ${TEXT2} ${TEXT3} ${TEXT4} ${TEXT5} ${TEXT6}
fi

###########################################
### Combine the RTM images from SuprightRupright
###########################################
echo "===================================="
echo "combining RTM images from SuprightRupright"
echo "===================================="
if [ -f ${TEXTSuprightRupright1} ]; then
    rtmlist1=`cat ${TEXTSuprightRupright1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSuprightRupright1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSuprightRupright1}
    echo ${OUTSuprightRupright1} >> ${TEXTSuprightRupright0}
    runloop=1
fi
if [ -f ${TEXTSuprightRupright2} ]; then
    rtmlist2=`cat ${TEXTSuprightRupright2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSuprightRupright2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSuprightRupright2}
    echo ${OUTSuprightRupright2} >> ${TEXTSuprightRupright0}
    runloop=2
fi
if [ -f ${TEXTSuprightRupright3} ]; then
    rtmlist3=`cat ${TEXTSuprightRupright3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSuprightRupright3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSuprightRupright3}
    echo ${OUTSuprightRupright3} >> ${TEXTSuprightRupright0}
    runloop=3
fi
if [ -f ${TEXTSuprightRupright4} ]; then
    rtmlist4=`cat ${TEXTSuprightRupright4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSuprightRupright4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSuprightRupright4}
    echo ${OUTSuprightRupright4} >> ${TEXTSuprightRupright0}
    runloop=4
fi
if [ -f ${TEXTSuprightRupright5} ]; then
    rtmlist5=`cat ${TEXTSuprightRupright5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSuprightRupright5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSuprightRupright5}
    echo ${OUTSuprightRupright5} >> ${TEXTSuprightRupright0}
    runloop=5
fi
if [ -f ${TEXTSuprightRupright6} ]; then
    rtmlist6=`cat ${TEXTSuprightRupright6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSuprightRupright6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSuprightRupright6}
    echo ${OUTSuprightRupright6} >> ${TEXTSuprightRupright0}
    runloop=6
fi

#######################
## generate DRTM images from SuprightRupright
#######################
if [ -f ${TEXTSuprightRupright0} ]; then
    rtmlist0=`cat ${TEXTSuprightRupright0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2 > ${OUTSuprightRupright}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSuprightRupright}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSuprightRupright1} ${OUTSuprightRupright2} ${OUTSuprightRupright3} ${OUTSuprightRupright4} ${OUTSuprightRupright5} ${OUTSuprightRupright6}
    rm -f ${TEXTSuprightRupright0} ${TEXTSuprightRupright1} ${TEXTSuprightRupright2} ${TEXTSuprightRupright3} ${TEXTSuprightRupright4} ${TEXTSuprightRupright5} ${TEXTSuprightRupright6}
fi


###########################################
### Combine the RTM images from SdownleftRdownleft
###########################################
echo "===================================="
echo "combining RTM images from SdownleftRdownleft"
echo "===================================="
if [ -f ${TEXTSdownleftRdownleft1} ]; then
    rtmlist1=`cat ${TEXTSdownleftRdownleft1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSdownleftRdownleft1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownleftRdownleft1}
    echo ${OUTSdownleftRdownleft1} >> ${TEXTSdownleftRdownleft0}
    runloop=1
fi
if [ -f ${TEXTSdownleftRdownleft2} ]; then
    rtmlist2=`cat ${TEXTSdownleftRdownleft2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSdownleftRdownleft2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownleftRdownleft2}
    echo ${OUTSdownleftRdownleft2} >> ${TEXTSdownleftRdownleft0}
    runloop=2
fi
if [ -f ${TEXTSdownleftRdownleft3} ]; then
    rtmlist3=`cat ${TEXTSdownleftRdownleft3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSdownleftRdownleft3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownleftRdownleft3}
    echo ${OUTSdownleftRdownleft3} >> ${TEXTSdownleftRdownleft0}
    runloop=3
fi
if [ -f ${TEXTSdownleftRdownleft4} ]; then
    rtmlist4=`cat ${TEXTSdownleftRdownleft4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSdownleftRdownleft4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownleftRdownleft4}
    echo ${OUTSdownleftRdownleft4} >> ${TEXTSdownleftRdownleft0}
    runloop=4
fi
if [ -f ${TEXTSdownleftRdownleft5} ]; then
    rtmlist5=`cat ${TEXTSdownleftRdownleft5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSdownleftRdownleft5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownleftRdownleft5}
    echo ${OUTSdownleftRdownleft5} >> ${TEXTSdownleftRdownleft0}
    runloop=5
fi
if [ -f ${TEXTSdownleftRdownleft6} ]; then
    rtmlist6=`cat ${TEXTSdownleftRdownleft6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSdownleftRdownleft6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownleftRdownleft6}
    echo ${OUTSdownleftRdownleft6} >> ${TEXTSdownleftRdownleft0}
    runloop=6
fi

#######################
## generate DRTM images from SdownleftRdownleft
#######################
if [ -f ${TEXTSdownleftRdownleft0} ]; then
    rtmlist0=`cat ${TEXTSdownleftRdownleft0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2 > ${OUTSdownleftRdownleft}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownleftRdownleft}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSdownleftRdownleft1} ${OUTSdownleftRdownleft2} ${OUTSdownleftRdownleft3} ${OUTSdownleftRdownleft4} ${OUTSdownleftRdownleft5} ${OUTSdownleftRdownleft6}
    rm -f ${TEXTSdownleftRdownleft0} ${TEXTSdownleftRdownleft1} ${TEXTSdownleftRdownleft2} ${TEXTSdownleftRdownleft3} ${TEXTSdownleftRdownleft4} ${TEXTSdownleftRdownleft5} ${TEXTSdownleftRdownleft6}
fi

###########################################
### Combine the RTM images from SuprightRdownleft
###########################################
echo "===================================="
echo "combining RTM images from SuprightRdownleft"
echo "===================================="
if [ -f ${TEXTSuprightRdownleft1} ]; then
    rtmlist1=`cat ${TEXTSuprightRdownleft1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSuprightRdownleft1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSuprightRdownleft1}
    echo ${OUTSuprightRdownleft1} >> ${TEXTSuprightRdownleft0}
    runloop=1
fi
if [ -f ${TEXTSuprightRdownleft2} ]; then
    rtmlist2=`cat ${TEXTSuprightRdownleft2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSuprightRdownleft2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSuprightRdownleft2}
    echo ${OUTSuprightRdownleft2} >> ${TEXTSuprightRdownleft0}
    runloop=2
fi
if [ -f ${TEXTSuprightRdownleft3} ]; then
    rtmlist3=`cat ${TEXTSuprightRdownleft3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSuprightRdownleft3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSuprightRdownleft3}
    echo ${OUTSuprightRdownleft3} >> ${TEXTSuprightRdownleft0}
    runloop=3
fi
if [ -f ${TEXTSuprightRdownleft4} ]; then
    rtmlist4=`cat ${TEXTSuprightRdownleft4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSuprightRdownleft4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSuprightRdownleft4}
    echo ${OUTSuprightRdownleft4} >> ${TEXTSuprightRdownleft0}
    runloop=4
fi
if [ -f ${TEXTSuprightRdownleft5} ]; then
    rtmlist5=`cat ${TEXTSuprightRdownleft5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSuprightRdownleft5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSuprightRdownleft5}
    echo ${OUTSuprightRdownleft5} >> ${TEXTSuprightRdownleft0}
    runloop=5
fi
if [ -f ${TEXTSuprightRdownleft6} ]; then
    rtmlist6=`cat ${TEXTSuprightRdownleft6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSuprightRdownleft6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSuprightRdownleft6}
    echo ${OUTSuprightRdownleft6} >> ${TEXTSuprightRdownleft0}
    runloop=6
fi
#######################
## generate DRTM images from SuprightRdownleft
#######################
if [ -f ${TEXTSuprightRdownleft0} ]; then
    rtmlist0=`cat ${TEXTSuprightRdownleft0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2 > ${OUTSuprightRdownleft}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSuprightRdownleft}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSuprightRdownleft1} ${OUTSuprightRdownleft2} ${OUTSuprightRdownleft3} ${OUTSuprightRdownleft4} ${OUTSuprightRdownleft5} ${OUTSuprightRdownleft6}
    rm -f ${TEXTSuprightRdownleft0} ${TEXTSuprightRdownleft1} ${TEXTSuprightRdownleft2} ${TEXTSuprightRdownleft3} ${TEXTSuprightRdownleft4} ${TEXTSuprightRdownleft5} ${TEXTSuprightRdownleft6}
fi

###########################################
### Combine the RTM images from SdownleftRupright
###########################################
echo "===================================="
echo "combining RTM images from SdownleftRupright"
echo "===================================="
if [ -f ${TEXTSdownleftRupright1} ]; then
    rtmlist1=`cat ${TEXTSdownleftRupright1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUTSdownleftRupright1}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownleftRupright1}
    echo ${OUTSdownleftRupright1} >> ${TEXTSdownleftRupright0}
    runloop=1
fi
if [ -f ${TEXTSdownleftRupright2} ]; then
    rtmlist2=`cat ${TEXTSdownleftRupright2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUTSdownleftRupright2}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownleftRupright2}
    echo ${OUTSdownleftRupright2} >> ${TEXTSdownleftRupright0}
    runloop=2
fi
if [ -f ${TEXTSdownleftRupright3} ]; then
    rtmlist3=`cat ${TEXTSdownleftRupright3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUTSdownleftRupright3}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownleftRupright3}
    echo ${OUTSdownleftRupright3} >> ${TEXTSdownleftRupright0}
    runloop=3
fi
if [ -f ${TEXTSdownleftRupright4} ]; then
    rtmlist4=`cat ${TEXTSdownleftRupright4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUTSdownleftRupright4}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownleftRupright4}
    echo ${OUTSdownleftRupright4} >> ${TEXTSdownleftRupright0}
    runloop=4
fi
if [ -f ${TEXTSdownleftRupright5} ]; then
    rtmlist5=`cat ${TEXTSdownleftRupright5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUTSdownleftRupright5}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownleftRupright5}
    echo ${OUTSdownleftRupright5} >> ${TEXTSdownleftRupright0}
    runloop=5
fi
if [ -f ${TEXTSdownleftRupright6} ]; then
    rtmlist6=`cat ${TEXTSdownleftRupright6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUTSdownleftRupright6}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownleftRupright6}
    echo ${OUTSdownleftRupright6} >> ${TEXTSdownleftRupright0}
    runloop=6
fi

#######################
## generate DRTM images from SdownleftRupright
#######################
if [ -f ${TEXTSdownleftRupright0} ]; then
    rtmlist0=`cat ${TEXTSdownleftRupright0}`
    echo rtmlist0=$rtmlist0
    # Merge all RTM images together
    Merge axis=3 space=n $rtmlist0 |Transp plane=23 | Stack axes=2 > ${OUTSdownleftRupright}
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUTSdownleftRupright}
    Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
    Rm ${OUTSdownleftRupright1} ${OUTSdownleftRupright2} ${OUTSdownleftRupright3} ${OUTSdownleftRupright4} ${OUTSdownleftRupright5} ${OUTSdownleftRupright6}
    rm -f ${TEXTSdownleftRupright0} ${TEXTSdownleftRupright1} ${TEXTSdownleftRupright2} ${TEXTSdownleftRupright3} ${TEXTSdownleftRupright4} ${TEXTSdownleftRupright5} ${TEXTSdownleftRupright6}
fi

## remove all RTM images from all shots
# Rm ${TEMPDIR}/drtm${m}imageshot*.H

exit
