!!! This is a FFT program of a 2D-arrayed data
!
!   <in.H Fftw2D_program.x par= >out.H
!
!   in.H   # input wavefield file must be a complex array up to 3 axes.
!   out.H  # output wavefield file will be a complex array up to 3 axes.
!
!   parameters:
!      flag=1     # The direction of the transform: [+1,default] FORWARD FFT, [-1] BACKWARD FFT
!                 # If forward FFT, in.H is the data in normal domain, while out.H is the data in fourier domain.
!      dim=1      # The number of axes that will be FFTed: [1,default] only the 1st axis will be transformed, [2] both two axes will be transformed
!      axis1,axis2  # axis* define the types of the axes where FFT will involve. The default is "axis1=-1,axis2=1". [-1] time axis, [1] space axis.
!      shift=y   # Is the shifting process needed: [1,y,default] the fourier domain will be shifted before&after FFT,
!                     # [0,n] the fourier domain will NOT be shifted b&a FFT.
!
!      verb=n
!
!   Remark:
!      If there exists the 3rd axis, the program will only do FFT along the 1st axis or both the 1st and 2rd axes. The recursive process will be done for every constant point on the 3rd axis.
!!!

program Fftw2D_program

  use sep
  use fftshift_mod
  implicit none
  include "fftw3.f"

  logical           :: shift, verb
  integer           :: n1, n2, n3, i, j, flag, dim, stat
  real              :: o1,o2,o3, d1,d2,d3
  integer :: axis1,axis2
  character(len=128)  :: label1,label2,label3

  complex, dimension(:,:,:), allocatable :: in    !/n1,n2,n3/
  complex, dimension(:), allocatable   :: out   !/max(n1,n2)/

  integer*8 plan1f

  call sep_init()
  !! I/O stuff
  ! get parameters from the input history file
    call from_history("n1",n1)
    call from_history("d1",d1)
    call from_history("o1",o1,0.)
    call from_history('label1',label1,'t(s)')
    ! if there exists the 2nd axis
    call from_history("n2",n2,1)
    call from_history("d2",d2,10.)
    call from_history("o2",o2,0.)
    call from_history('label2',label2,'z(m)')
    ! if there exists the 3rd axis
    call from_history("n3",n3,1)
    call from_history("d3",d3,10.)
    call from_history("o3",o3,0.)
    call from_history('label3',label3,'x(m)')
    write(0,*) "Reading axes complete...."

    !! read parameters
    call from_param("flag",flag,1) ! forward or backward FFT: [+1,default] FORWARD FFT, [-1] BACKWARD FFT
    call from_param("dim",dim,1)   ! [1,default] only the 1st axis will be transformed, [2] both two axes will be transformed
    call from_param("axis1",axis1,-1) ! [-1] means time axis
    call from_param("axis2",axis2,1)  ! [1] means space axis
    call from_param("shift",shift,.true.)
    call from_param("verb",verb,.false.)
    ! check if the parameters are correct
    if(dim.ne.1 .and. dim.ne.2)then
       write(0,*) "dim=",dim
       call seperr("'dim' must be 1 or 2")
    end if
    if(flag.ne.1 .and. flag.ne.-1)then
       write(0,*) "flag=",flag
       call seperr("'flag' must be 1 or -1")
    end if
    
    if(axis1.ne.1 .and. axis1.ne.-1)then
       write(0,*) "axis1=", axis1
       call seperr("'axis1' must be 1 or -1")
    end if
    if(axis2.ne.1 .and. axis2.ne.-1)then
       write(0,*) "axis2=", axis2
       call seperr("'axis2' must be 1 or -1")
    end if

    !! Illustrate input parameters
    write(0,*) "calling Fftw2D_program.x...."
    if(verb)then
       write(0,*) '**************************************************'
       write(0,*) '*************** Input parameters.. ***************'
       write(0,*) '**************************************************'
       write(0,*) 'n1,d1,o1,label1', n1, d1, o1, label1
       write(0,*) 'n2,d2,o2,label2', n2, d2, o2, label2
       write(0,*) 'n3,d3,o3,label3', n3, d3, o3, label3
       write(0,*) 'flag,dim,shift', flag,dim,shift
       write(0,*) "axis1,axis2", axis1,axis2
       write(0,*) '**************************************************'
       write(0,*) 'Note: flag ~ [+1] FORWARD FFT or [-1] BACKWARD FFT'
       write(0,*) '       dim ~ [1] only the 1st axis or [2] both two axes will be transformed'
       write(0,*) '       axis1,axis2 ~ [-1] means time axis, [1] means space axis'
       write(0,*) '**************************************************'
       write(0,*) 'memmory(kB) ~ n1*n2*n3*8', (n1*n2*n3*8)/1000
       write(0,*) '**************************************************'
    end if
 
  !!******************* Converting axis parameters ********************
  !!.. flag = +1: forward
  !!..        -1: backward
  !!--------------------------------------------------------------------------------
    if(flag.eq.1)then 
       ! forward FFT
          ! DEFAULT: only the 1st axis will be transformed
          ! the transformed axis below works for both: time -> frequency, space -> wavenumber.
          if(shift)then
             o1 = -0.5/d1              ! If n is EVEN, of=-0.5/dt. 
             if(mod(n1,2).eq.1) o1=o1+0.5/(real(n1)*d1) ! BUT if n is ODD, of=-0.5/dt+0.5/(n*dt).
          else
             o1 = 0. ! set o1 = 0. if shift is not applied.
          end if
          d1 = 1.0/(real(n1)*d1)    ! df=1/(n*dt)
            if(axis1.eq.-1)then
               label1='frequency(Hz)'
            else
               label1='wavenumber(/m)'
            end if

       if(dim.eq.2)then
          ! dim==2: the 2nd axis will also be transformed
          ! the transformed axis below works for both: time -> frequency, space -> wavenumber.
          if(shift)then
             o2 = -0.5/d2              ! If n is EVEN, of=-0.5/dt. 
             if(mod(n2,2).eq.1) o2=o2+0.5/(real(n2)*d2) ! BUT if n is ODD, of=-0.5/dt+0.5/(n*dt).
          else
             o2 = 0. ! set o2 = 0. if shift is not applied.
          end if
          d2 = 1.0/(real(n2)*d2)    ! df=1/(n*dt)
            if(axis2.eq.-1)then
               label2='frequency(Hz)'
            else
               label2='wavenumber(/m)'
            end if
       end if
    else
       ! backward FFT
          ! DEFAULT: only the 1st axis will be transformed
          ! the transformed axis below works for both: frequency -> time, wavenumber -> space.
          o1 = 0.              ! normal axis begins at zero by default
          d1 = 1.0/(real(n1)*d1)    ! dt=1/(n*df)
            if(axis1.eq.-1)then
               label1='time(s)'
            else
               label1='space(m)'
            end if

       if(dim.eq.2)then
          ! dim==2: the 2nd axis will also be transformed
          ! the transformed axis below works for both: frequency -> time, wavenumber -> space.
          o2 = 0.              ! normal axis begins at zero by default
          d2 = 1.0/(real(n2)*d2)    ! dt=1/(n*df)
            if(axis2.eq.-1)then
               label2='time(s)'
            else
               label2='space(m)'
            end if
       end if
    end if
  !!******************* XXXXXXXXXXXXXXXXXXXXXXX ********************

    !! write axes parameters to the output history file
    call to_history("n1",n1)
    call to_history("d1",d1)
    call to_history("o1",o1)
    call to_history("label1",label1)
    call to_history("n2",n2)
    call to_history("d2",d2)
    call to_history("o2",o2)
    call to_history("label2",label2)
    call to_history("n3",n3)
    call to_history("d3",d3)
    call to_history("o3",o3)
    call to_history("label3",label3)
    if(verb) write(0,*) "Writing axes complete...."

    !! Illustrate output parameters
    if(verb)then
       write(0,*) '**************************************************'
       write(0,*) '*************** Output parameters.. ***************'
       write(0,*) '**************************************************'
       write(0,*) 'n1,d1,o1,label1', n1, d1, o1, label1
       write(0,*) 'n2,d2,o2,label2', n2, d2, o2, label2
       write(0,*) 'n3,d3,o3,label3', n3, d3, o3, label3
       write(0,*) '**************************************************'
    end if

  call sep_close()

  ! Allocate parameters
  allocate(in(n1,n2,n3),out(max(n1,n2)))
  out=(0.,0.)
  if(verb) write(0,*) "allocation completed"

  !! our work begins here
  !! read data
  call sep_read(in)
  if(verb) write(0,*) "data have been read"
  !!******************************************************
  !!***** Shift the axes ready for computation
  !! This part is applicable only if the input file is in fourier domain and need shifting process
  !! That is, shifting the input data from [-fnyq,fnyq) to [0,2*fnyq)
  !!******************************************************

  if(shift) then
     if(verb) write(0,*) "Doing shifting from [-fnyq,fnyq) to [0,2*fnyq)..."
     if(flag.eq.-1)then
        !! only for doing backward FFT
        ! DEFAULT: only the 1st axis will be shifted
        ! Shifting works for both: frequency and wavenumber domains.
        do j=1,n3 ! 3rd axis is not in our computation, so shifting is not needed
           do i=1,n2
              stat=fftshift1Dc_lop(.true.,.false.,out(1:n1),in(:,i,j)) !fftshift1Dc_lop(adj,add,x,y)
              in(:,i,j)=out(1:n1)
           end do
        end do

        if(dim.eq.2)then
           ! dim==2: the 2nd axis will also be shifted
           do j=1,n3
              do i=1,n1
                 stat=fftshift1Dc_lop(.true.,.false.,out(1:n2),in(i,:,j)) !fftshift1Dc_lop(adj,add,x,y)
                 in(i,:,j)=out(1:n2)
              end do
           end do
        end if

     end if
  end if

  !!******************************************************

  !!===========

  !!******************************************************
  !!****************** Begin doing FFT *******************
  !!** flag = +1: forward FFT
  !!**        -1: backward FFT
  !!******************************************************

  !! DEFAULT: only the 1st axis will be transformed
  if(verb) write(0,*) "Doing forward FFT along the 1st axis..."
  if(axis1.eq.-1)then
     ! time <-> frequency
     if(flag.eq.1)then
        ! flag=1: forward, exp(+iwt)
        call sfftw_plan_dft_1d(plan1f, n1, in(:,1,1), out(1:n1), FFTW_BACKWARD, FFTW_ESTIMATE)
        do j=1,n3
           do i=1,n2
              call sfftw_execute_dft(plan1f, in(:,i,j), out(1:n1))
              in(:,i,j)=out(1:n1)/sqrt(real(n1)) ! divided by sqrt(n1) instead of n1. this helps when computing correlation in FFT domain
           end do
        end do
        call sfftw_destroy_plan(plan1f)
     else
        ! flag=-1: backward, exp(-iwt)
        call sfftw_plan_dft_1d(plan1f, n1, in(:,1,1), out(1:n1), FFTW_FORWARD, FFTW_ESTIMATE)
        do j=1,n3
           do i=1,n2
              call sfftw_execute_dft(plan1f, in(:,i,j), out(1:n1))
              in(:,i,j)=out(1:n1)/sqrt(real(n1)) ! divided by sqrt(n1) instead of 1
           end do
        end do
        call sfftw_destroy_plan(plan1f)
     end if
  else
     ! space <-> wavenumber
     if(flag.eq.1)then
        ! flag=1: forward, exp(-ikx)
        call sfftw_plan_dft_1d(plan1f, n1, in(:,1,1), out(1:n1), FFTW_FORWARD, FFTW_ESTIMATE)
        do j=1,n3
           do i=1,n2
              call sfftw_execute_dft(plan1f, in(:,i,j), out(1:n1))
              in(:,i,j)=out(1:n1)/sqrt(real(n1)) ! divided by sqrt(n1) instead of 1
           end do
        end do
        call sfftw_destroy_plan(plan1f)
     else
        ! flag=-1: backward, exp(+ikx)
        call sfftw_plan_dft_1d(plan1f, n1, in(:,1,1), out(1:n1), FFTW_BACKWARD, FFTW_ESTIMATE)
        do j=1,n3
           do i=1,n2
              call sfftw_execute_dft(plan1f, in(:,i,j), out(1:n1))
              in(:,i,j)=out(1:n1)/sqrt(real(n1)) ! divided by sqrt(n1) instead of n1
           end do
        end do
        call sfftw_destroy_plan(plan1f)
     end if
  end if


    !write(0,*) "dim1 FFT done"
  !! If dim=2: the 2nd axis will also be transformed
  if(dim.eq.2)then
     if(verb) write(0,*) "Doing forward FFT along the 2nd axis..."
     if(axis2.eq.-1)then
        ! time <-> frequency
        if(flag.eq.1)then
           ! flag=1: forward, exp(+iwt)
           call sfftw_plan_dft_1d(plan1f, n2, in(1,:,1), out(1:n2), FFTW_BACKWARD, FFTW_ESTIMATE)
           do j=1,n3
              do i=1,n1
                 call sfftw_execute_dft(plan1f, in(i,:,j), out(1:n2))
                 in(i,:,j)=out(1:n2)/sqrt(real(n2)) !divided by sqrt(n2) instead of n2
              end do
           end do
           call sfftw_destroy_plan(plan1f)
        else
           ! flag=-1: backward, exp(-iwt)
           call sfftw_plan_dft_1d(plan1f, n2, in(1,:,1), out(1:n2), FFTW_FORWARD, FFTW_ESTIMATE)
           do j=1,n3
              do i=1,n1
                 call sfftw_execute_dft(plan1f, in(i,:,j), out(1:n2))
                 in(i,:,j)=out(1:n2)/sqrt(real(n2)) !divided by sqrt(n2) instead of 1
              end do
           end do
           call sfftw_destroy_plan(plan1f)
        end if
     else
        ! space <-> wavenumber
        if(flag.eq.1)then
           ! flag=1: forward, exp(-ikx)
           call sfftw_plan_dft_1d(plan1f, n2, in(1,:,1), out(1:n2), FFTW_FORWARD, FFTW_ESTIMATE)
           do j=1,n3
              do i=1,n1
                 call sfftw_execute_dft(plan1f, in(i,:,j), out(1:n2))
                 in(i,:,j)=out(1:n2)/sqrt(real(n2)) !divided by sqrt(n2) instead of 1
              end do
           end do
           call sfftw_destroy_plan(plan1f)
        else
           ! flag=-1: backward, exp(+ikx)
           call sfftw_plan_dft_1d(plan1f, n2, in(1,:,1), out(1:n2), FFTW_BACKWARD, FFTW_ESTIMATE)
           do j=1,n3
              do i=1,n1
                 call sfftw_execute_dft(plan1f, in(i,:,j), out(1:n2))
                 in(i,:,j)=out(1:n2)/sqrt(real(n2)) !divided by sqrt(n2) instead of n2
              end do
           end do
           call sfftw_destroy_plan(plan1f)
        end if
     end if
    !write(0,*) "dim2 FFT done"

 end if

  !!******************* End doing FFT ********************
  !!******************************************************

  !!===========

  !!******************************************************
  !!***** Shift the axes ready for writing the output binary file
  !! This part is applicable only if the output file is in fourier domain and need shifting process
  !! That is, shifting the output data from [0,2*fnyq) to [-fnyq,fnyq)
  !!******************************************************

  if(shift) then
     if(verb) write(0,*) "Doing shifting from [0,2*fnyq) to [-fnyq,fnyq)..."
     if(flag.eq.1)then
        !! only for doing forward FFT
        ! DEFAULT: only the 1st axis will be shifted
        ! Shifting works for both: frequency and wavenumber domains.
        do j=1,n3
           do i=1,n2
              stat=fftshift1Dc_lop(.false.,.false.,in(:,i,j),out(1:n1)) !fftshift1Dc_lop(adj,add,x,y)
              in(:,i,j)=out(1:n1)
           end do
        end do
        !write(0,*) "dim1 shift done"

        if(dim.eq.2)then
           ! dim==2: the 2nd axis will also be shifted
           do j=1,n3
              do i=1,n1
                 stat=fftshift1Dc_lop(.false.,.false.,in(i,:,j),out(1:n2)) !fftshift1Dc_lop(adj,add,x,y)
                 in(i,:,j)=out(1:n2)
              end do
           end do
           !write(0,*) "dim2 shift done"
        end if
     end if
  end if

  !!******************************************************

  !! write data
  call sep_write(in)
  if(verb) write(0,*) "data have been written"

  !! work ends here

  !deallocate parameters
  deallocate(in,out)
  if(verb) write(0,*) "Deallocation completed"

end program Fftw2D_program
