!!! This is a wavefield decomposition program in 3D 
!! It will do: FFT -> Fkfilter -> iFFT
!!
!! Author: Kittinat Taweesintananon, April 2011
!
! Example:
!               <in.H Fkdecom3d_program.x par= >out.H filter=filter.H fkdataout=fkdataout.H
!
! Related history files:
!         in.H   # input wavefield file must be a complex array up to 3 axes.
!         out.H  # output wavefield file will be a complex array up to 3 axes.
!
!         filter=filter.H  # (optional, available by setting get_filter=y) the image of the filter function we use
!         fkdataout=fkdataout.H  # (optional, available by setting get_fkdata=y) the image of output filtered data in fourier domain 
!
! Parameters:
!
!!   ** FFT stuff:
!      dim=1      # The number of axes that will be FFTed: [1,default] only the 1st axis will be transformed and filtered, [2] first two axes will be transformed and filtered, [3] all three axes will be transformed and filtered
!      axis1,axis2,axis3  # axis* define the types of the axes where FFT will involve. The default is "axis1=-1,axis2=1,axis3=1". [-1] time axis, [1] space axis.
!
!!   ** Fkfilter stuff:
!      flag=1    # Choose the area that the information will be stored. The data outside this area else will be removed or attenuated.
!                # [+1,default] If dim=1, only the data in the positive range of the 1st axis will be stored.
!                #              If dim=2, only the data in the region where the components of both axes have the same sign will be stored, namely only positive-sloped data: e.g. +f and +k area or -f and -k area.
!                # [-1] If dim=1, only the data in the negative range of the 1st axis will be stored.
!                #      If dim=2, only the data in the region where the components of both axes have the opposite sign to each other will be stored, namely only negative-sloped data: e.g. +f and -k area or -f and +k area.
!
!     ##### If dim=3 and input is /t,z,x/:
!         # flag=1 keep the data in (+f,+kz,+kx) and (-f,-kz,-kx): downright-going wavefields
!         # flag=-1 keep the data in (+f,-kz,-kx) and (-f,+kz,+kx): upleft-going wavefields
!         # flag=2 keep the data in (+f,-kz,+kx) and (-f,+kz,-kx): upright-going wavefields
!         # flag=-2 keep the data in (+f,+kz,-kx) and (-f,-kz,+kx): downleft-going wavefields
!
!      filter=0     # Choose the filter function used in the program: [0,default] do nothing, [1] step cut, [2] fan with Tukey tapering function
!      alpha=0.2  # [0.2] If dim=1 and mode=2, alpha in Tukey function is required in the range [0,1], but 0.2 is recommended.
!      phi=5.  # [5.] If dim=2 and mode=2, angles of fans (phi) in Tukey function are required in the range [0,90] degree, but 15 degree is recommended.
!
!      get_filter=n # [0,n,default] whether to get the image of the filter function we use.
!      get_fkdataout=n # [0,n,default] whether to get the filtered data in fourier domain
!
!      verb      # [n] verbal expression
!
!
!   Remark:
!
!!!

program Fkdecomp3d_program

  use sep
  use fftshift_mod
  use fkfilter_mod
  use fkfilter3D_mod
  implicit none
  include "fftw3.f"

  logical           :: verb, get_filter, get_fkdataout
  integer           :: n1, n2, n3, i, j, flag, dim, filter, stat
  real              :: o1,o2,o3, d1,d2,d3, alpha,phi, o1fk,o2fk,o3fk, d1fk,d2fk,d3fk
  integer :: axis1,axis2,axis3
  character(len=128)  :: label1,label2,label3, label1fk,label2fk,label3fk, filter_, fkdataout_

  complex, dimension(:,:,:), allocatable :: in    !/n1,n2,n3/
  complex, dimension(:), allocatable   :: out   !/max(n1,n2)/
  real, dimension(:,:),allocatable      :: temp

  integer*8 plan1f

  filter_="filter"
  fkdataout_="fkdataout"

  call sep_init()
  !! I/O stuff
  ! get parameters from the input history file
    call from_history("n1",n1)
    call from_history("d1",d1)
    call from_history("o1",o1,0.)
    call from_history('label1',label1,'t(s)')
    ! if there exists the 2nd axis
    call from_history("n2",n2,1)
    call from_history("d2",d2,10.)
    call from_history("o2",o2,0.)
    call from_history('label2',label2,'z(m)')
    ! if there exists the 3rd axis
    call from_history("n3",n3,1)
    call from_history("d3",d3,10.)
    call from_history("o3",o3,0.)
    call from_history('label3',label3,'x(m)')
    write(0,*) "Reading axes complete...."

    !! read parameters
    call from_param("verb",verb,.false.) ! [n] verbal

    !! FFT stuff
    call from_param("dim",dim,1)   ! [1,default] only the 1st axis will be transformed, [2] both two axes will be transformed
    call from_param("axis1",axis1,-1) ! [-1] means time axis
    call from_param("axis2",axis2,1)  ! [1] means space axis
    call from_param("axis3",axis3,1)  ! [1] means space axis

    !! filter stuff
    call from_param("flag",flag,1) ! Choose the area that the information will be stored.
    call from_param("filter",filter,0)
    call from_param("alpha",alpha,0.2)
    call from_param("phi",phi,5.)
    call from_param("get_filter",get_filter,.false.)

    !! image of output (filtered data) in fourier domain
    call from_param("get_fkdataout",get_fkdataout,.false.)

    ! check if the parameters are correct
    if(dim.ne.1 .and. dim.ne.2 .and. dim.ne.3)then
       write(0,*) "dim=",dim
       call seperr("'dim' must be 1 or 2")
    end if
    if(dim.eq.1 .or. dim.eq.2)then
       if(flag.ne.1 .and. flag.ne.-1)then
          write(0,*) "flag=",flag
          call seperr("'flag' must be 1 or -1")
       end if
    else
       if(flag.ne.1 .and. flag.ne.-1 .and. flag.ne.2 .and. flag.ne.-2)then
          write(0,*) "flag=",flag
          call seperr("'flag' must be 1, -1, 2, or -2")
       end if
    end if
    
    if(axis1.ne.1 .and. axis1.ne.-1)then
       write(0,*) "axis1=", axis1
       call seperr("'axis1' must be 1 or -1")
    end if
    if(axis2.ne.1 .and. axis2.ne.-1)then
       write(0,*) "axis2=", axis2
       call seperr("'axis2' must be 1 or -1")
    end if
    if(axis3.ne.1 .and. axis3.ne.-1)then
       write(0,*) "axis3=", axis3
       call seperr("'axis3' must be 1 or -1")
    end if

    !! Illustrate input parameters
  if(verb)then
    write(0,*) '**************************************************'
    write(0,*) '*************** Input parameters.. ***************'
    write(0,*) '**************************************************'
    write(0,*) 'n1,d1,o1,label1', n1, d1, o1, label1
    write(0,*) 'n2,d2,o2,label2', n2, d2, o2, label2
    write(0,*) 'n3,d3,o3,label3', n3, d3, o3, label3
    write(0,*) 'dim', dim
    write(0,*) "axis1,axis2,axis3", axis1,axis2,axis3
    write(0,*) 'get_filter,get_fkdataout', get_filter,get_fkdataout
    write(0,*) '**************************************************'
    write(0,*) 'Note:  dim ~ [1] only the 1st axis or [2] first two axes will be transformed or [3] three axes'
    write(0,*) '       axis1,axis2 ~ [-1] means time axis, [1] means space axis'
    write(0,*) '       flag ~ [+1] Storing positive axis (dim=1) or slope (dim=2), [-1] Storing negative axis (dim=1) or slope (dim=2)'
    write(0,*) '       filter ~ [0] no filter, [1] sharp-cut filter, [2] smooth filter'
    write(0,*) '**************************************************'
    write(0,*) 'memmory(kB) ~ n1*n2*n3*8', (n1*n2*n3*8)/1000
    write(0,*) '**************************************************'
  end if

  !!******* Converting axis parameters to output filtering result in fourier domain (fkdataout) ********
    ! forward FFT output is in fourier domain
    ! DEFAULT: only the 1st axis will be transformed
    ! the transformed axis below works for both: time -> frequency, space -> wavenumber.
  if(get_filter.or.get_fkdataout)then
    !! Axes in FK domain are needed only if we want some outputs in FK domain
    o1fk = -0.5/d1              ! If n is EVEN, of=-0.5/dt. 
    if(mod(n1,2).eq.1) o1fk=o1fk+0.5/(real(n1)*d1) ! BUT if n is ODD, of=-0.5/dt+0.5/(n*dt).
    d1fk = 1.0/(real(n1)*d1)    ! df=1/(n*dt)
    if(axis1.eq.-1)then
       label1fk='frequency(Hz)'
    else
       label1fk='wavenumber(/m)'
    end if
    
    if(dim.eq.2 .or. dim.eq.3)then
       ! dim==2 or dim==3: the 2nd axis will also be transformed
       ! the transformed axis below works for both: time -> frequency, space -> wavenumber.
       o2fk = -0.5/d2              ! If n is EVEN, of=-0.5/dt. 
       if(mod(n2,2).eq.1) o2fk=o2fk+0.5/(real(n2)*d2) ! BUT if n is ODD, of=-0.5/dt+0.5/(n*dt).
       d2fk = 1.0/(real(n2)*d2)    ! df=1/(n*dt)
       if(axis2.eq.-1)then
          label2fk='Frequency(Hz)'
       else
          label2fk='Kz(/m)'
       end if
    end if

    if(dim.eq.3)then
       ! dim==3: the 3nd axis will also be transformed
       ! the transformed axis below works for both: time -> frequency, space -> wavenumber.
       o3fk = -0.5/d3              ! If n is EVEN, of=-0.5/dt. 
       if(mod(n3,2).eq.1) o3fk=o3fk+0.5/(real(n3)*d3) ! BUT if n is ODD, of=-0.5/dt+0.5/(n*dt).
       d3fk = 1.0/(real(n3)*d3)    ! df=1/(n*dt)
       if(axis3.eq.-1)then
          label3fk='Frequency(Hz)'
       else
          label3fk='Kx(/m)'
       end if
    end if

  end if
  !!******************* XXXXXXXXXXXXXXXXXXXXXXX ********************

    !! write axes parameters to the output history file
    call to_history("n1",n1)
    call to_history("d1",d1)
    call to_history("o1",o1)
    call to_history("label1",label1)
    call to_history("n2",n2)
    call to_history("d2",d2)
    call to_history("o2",o2)
    call to_history("label2",label2)
    call to_history("n3",n3)
    call to_history("d3",d3)
    call to_history("o3",o3)
    call to_history("label3",label3)
    if(verb) write(0,*) "Writing axes complete...."

    !! write axes parameters to the auxiliary history file for outputting the filter function
    if(get_filter .and. dim.ne.3) then
       !if(dim.eq.1) then
          ! filter as a 1-D function
          call auxputch("n1",'i', n1, filter_); call auxputch("d1",'f', d1fk,filter_);  
          call auxputch("o1",'f', o1fk, filter_); call auxputch("label1", 's', label1fk, filter_);
       !end if
        if(dim.eq.2)then
          call auxputch("n2",'i', n2, filter_); call auxputch("d2",'f', d2fk,filter_);
          call auxputch("o2",'f', o2fk, filter_); call auxputch("label2", 's', label2fk, filter_);           
        end if
    end if

    !! write axes parameters to the auxiliary history file for outputting the filtered result in fk domain
    if(get_fkdataout) then
       ! filtered result in fk domain
       call auxputch("n1",'i', n1, fkdataout_); call auxputch("d1",'f', d1fk, fkdataout_);  
       call auxputch("o1",'f', o1fk, fkdataout_); call auxputch("label1", 's', label1fk, fkdataout_);
       if(dim.eq.2 .or. dim.eq.3)then
          call auxputch("n2",'i', n2, fkdataout_); call auxputch("d2",'f', d2fk, fkdataout_);
          call auxputch("o2",'f', o2fk, fkdataout_); call auxputch("label2", 's', label2fk, fkdataout_);
       else
          call auxputch("n2",'i', n2, fkdataout_); call auxputch("d2",'f', d2, fkdataout_);
          call auxputch("o2",'f', o2, fkdataout_); call auxputch("label2", 's', label2, fkdataout_);
       end if
       if(dim.eq.3)then
          call auxputch("n3",'i', n3, fkdataout_); call auxputch("d3",'f', d3fk, fkdataout_);
          call auxputch("o3",'f', o3fk, fkdataout_); call auxputch("label3", 's', label3fk, fkdataout_);
       else
          call auxputch("n3",'i', n3, fkdataout_); call auxputch("d3",'f', d3, fkdataout_);
          call auxputch("o3",'f', o3, fkdataout_); call auxputch("label3", 's', label3, fkdataout_);
       end if

    end if

    !! Illustrate output parameters
    if(verb)then
!       write(0,*) '**************************************************'
!       write(0,*) '************** Output parameters.. ***************'
!       write(0,*) '**************************************************'
!       write(0,*) 'n1,d1,o1,label1', n1, d1, o1, label1
!       write(0,*) 'n2,d2,o2,label2', n2, d2, o2, label2
!       write(0,*) 'n3,d3,o3,label3', n3, d3, o3, label3
!       write(0,*) '**************************************************'
    end if

  call sep_close()

  ! Allocate parameters
  allocate(in(n1,n2,n3),out(max(n1,n2,n3)))
  if(dim.eq.1) allocate(temp(n1,1))
  if(dim.eq.2) allocate(temp(n1,n2))
  out=(0.,0.)
  temp=0.
  if(verb) write(0,*) "size(temp)",size(temp)
  if(verb) write(0,*) "allocation completed"

  !! our work begins here
  !! read data
  call sep_read(in)
  if(verb) write(0,*) "data have been read"

  !!******************************************************
  !! Do forward FFT here
  !! DEFAULT: only the 1st axis will be transformed
  if(verb) write(0,*) "Doing forward FFT along the 1st axis..."
  if(axis1.eq.-1)then
     ! time => frequency: forward, exp(+iwt)
     call sfftw_plan_dft_1d(plan1f, n1, in(:,1,1), out(1:n1), FFTW_BACKWARD, FFTW_ESTIMATE)
     do j=1,n3
        do i=1,n2
           call sfftw_execute_dft(plan1f, in(:,i,j), out(1:n1))
           in(:,i,j)=out(1:n1)/sqrt(real(n1)) ! divided by sqrt(n1) instead of n1. this helps when computing correlation in FFT domain
        end do
     end do
  else
     ! space => wavenumber: forward, exp(-ikx)
     call sfftw_plan_dft_1d(plan1f, n1, in(:,1,1), out(1:n1), FFTW_FORWARD, FFTW_ESTIMATE)
     do j=1,n3
        do i=1,n2
           call sfftw_execute_dft(plan1f, in(:,i,j), out(1:n1))
           in(:,i,j)=out(1:n1)/sqrt(real(n1)) ! divided by sqrt(n1) instead of 1
        end do
     end do
  end if
  call sfftw_destroy_plan(plan1f)

  if(dim.eq.2 .or. dim.eq.3)then
     if(verb) write(0,*) "Doing forward FFT along the 2nd axis..."
     if(axis2.eq.-1)then
        ! time => frequency: forward, exp(+iwt)
        call sfftw_plan_dft_1d(plan1f, n2, in(1,:,1), out(1:n2), FFTW_BACKWARD, FFTW_ESTIMATE)
        do j=1,n3
           do i=1,n1
              call sfftw_execute_dft(plan1f, in(i,:,j), out(1:n2))
              in(i,:,j)=out(1:n2)/sqrt(real(n2)) !divided by sqrt(n2) instead of n2
           end do
        end do
     else
        ! space => wavenumber: forward, exp(-ikx)
        call sfftw_plan_dft_1d(plan1f, n2, in(1,:,1), out(1:n2), FFTW_FORWARD, FFTW_ESTIMATE)
        do j=1,n3
           do i=1,n1
              call sfftw_execute_dft(plan1f, in(i,:,j), out(1:n2))
              in(i,:,j)=out(1:n2)/sqrt(real(n2)) !divided by sqrt(n2) instead of 1
           end do
        end do
     end if
     call sfftw_destroy_plan(plan1f)
  end if

  if(dim.eq.3)then
     if(verb) write(0,*) "Doing forward FFT along the 3nd axis..."
     if(axis3.eq.-1)then
        ! time => frequency: forward, exp(+iwt)
        call sfftw_plan_dft_1d(plan1f, n3, in(1,1,:), out(1:n3), FFTW_BACKWARD, FFTW_ESTIMATE)
        do j=1,n2
           do i=1,n1
              call sfftw_execute_dft(plan1f, in(i,j,:), out(1:n3))
              in(i,j,:)=out(1:n3)/sqrt(real(n3)) !divided by sqrt(n3) instead of n3
           end do
        end do
     else
        ! space => wavenumber: forward, exp(-ikx)
        call sfftw_plan_dft_1d(plan1f, n3, in(1,1,:), out(1:n3), FFTW_FORWARD, FFTW_ESTIMATE)
        do j=1,n2
           do i=1,n1
              call sfftw_execute_dft(plan1f, in(i,j,:), out(1:n3))
              in(i,j,:)=out(1:n3)/sqrt(real(n3)) !divided by sqrt(n3) instead of 1
           end do
        end do
     end if
     call sfftw_destroy_plan(plan1f)
  end if


  !!******************************************************
  !! Do shifting from [0,2*fnyq) to [-fnyq,fnyq) so that we can separate wavefield components
  ! DEFAULT: only the 1st axis will be shifted
  ! Shifting works for both: frequency and wavenumber domains.
  do j=1,n3
     do i=1,n2
        stat=fftshift1Dc_lop(.false.,.false.,in(:,i,j),out(1:n1)) !fftshift1Dc_lop(adj,add,x,y)
        in(:,i,j)=out(1:n1)
     end do
  end do
  
  if(dim.eq.2 .or. dim.eq.3)then
     ! dim==2 or dim==3: the 2nd axis will also be shifted
     do j=1,n3
        do i=1,n1
           stat=fftshift1Dc_lop(.false.,.false.,in(i,:,j),out(1:n2)) !fftshift1Dc_lop(adj,add,x,y)
           in(i,:,j)=out(1:n2)
        end do
     end do
     !write(0,*) "dim2 shift done"
  end if

  if(dim.eq.3)then
     ! dim==3: the 3nd axis will also be shifted
     do j=1,n2
        do i=1,n1
           stat=fftshift1Dc_lop(.false.,.false.,in(i,j,:),out(1:n3)) !fftshift1Dc_lop(adj,add,x,y)
           in(i,j,:)=out(1:n3)
        end do
     end do
     !write(0,*) "dim3 shift done"
  end if
        

  !!******************************************************
  !! Choose wavefield components we want. Now our data are in the range [-fnyq,fnyq).
  !!************************************************************
  !!****************** Begin doing filtering *******************
  if(verb) write(0,*) "Applying FK filtering..."
  !!******************* Data-storing parameters ********************
  !!.. flag = +1: Storing positive axis (dim=1) or slope (dim=2)
  !!..        -1: Storing negative axis (dim=1) or slope (dim=2)
  !!--------------------------------------------------------------------------------

  !! If dim=1, we will store only +/- values of the 1st axis
  if(dim.eq.1)then
     !! dim=1: working only on the 1st axis
     call filter1Dfn(temp(:,1),filter,flag,alpha=alpha) ! crete the temp function corresponding to the selected filter and flag.
     do i=1,n1
        in(i,:,:) = in(i,:,:) * temp(i,1)
     end do
  elseif(dim.eq.2)then
     !! dim=2: simutaneously working on first two axes
     call filter2Dfn(temp(:,:),filter,flag,phi=phi)
     do j=1,n3
        in(:,:,j) = in(:,:,j) * temp(:,:)
     end do
  elseif(dim.eq.3)then
     !! dim=3: simutaneously working on all three axes
     call filter3D(in(:,:,:),filter,flag,alpha=alpha,phi=phi)
  end if ! end dim condition

  !!******************************************************
  if(get_filter .and. dim.ne.3)then
     call sep_write(temp,filter_)
     if(verb) write(0,*) "filtering function has been written"
  end if
  !deallocate temp(:,:)
  if(dim.eq.1 .or. dim.eq.2) deallocate(temp)
  if(verb) write(0,*) "filtering function has been deallocated"
  !! write out the filtering result in FK domain [-fnyq,fnyq]
  if(get_fkdataout)then
     ! call sep_write(in,fkdataout_)
     call srite(fkdataout_,real(in(:,:,:)),4*n1*n2*n3)
     if(verb) write(0,*) "fkdataoutdata has been written"
  end if
  !!******************************************************


  !!******************************************************
  !! Do reshifting from [-fnyq,fnyq) to [0,2*fnyq) so that they can enter iFFT program
!! only for doing backward FFT
  ! DEFAULT: only the 1st axis will be shifted
  ! Shifting works for both: frequency and wavenumber domains.
  do j=1,n3 ! 3rd axis is not in our computation, so shifting is not needed
     do i=1,n2
        stat=fftshift1Dc_lop(.true.,.false.,out(1:n1),in(:,i,j)) !fftshift1Dc_lop(adj,add,x,y)
        in(:,i,j)=out(1:n1)
     end do
  end do
  
  if(dim.eq.2 .or. dim.eq.3)then
     ! dim==2: the 2nd axis will also be shifted
     do j=1,n3
        do i=1,n1
           stat=fftshift1Dc_lop(.true.,.false.,out(1:n2),in(i,:,j)) !fftshift1Dc_lop(adj,add,x,y)
           in(i,:,j)=out(1:n2)
        end do
     end do
  end if

  if(dim.eq.3)then
     ! dim==3: the 3nd axis will also be shifted
     do j=1,n2
        do i=1,n1
           stat=fftshift1Dc_lop(.true.,.false.,out(1:n3),in(i,j,:)) !fftshift1Dc_lop(adj,add,x,y)
           in(i,j,:)=out(1:n3)
        end do
     end do
     !write(0,*) "dim3 shift done"
  end if

  !!******************************************************
  !! Do backward FFT (iFFT) here
  !! DEFAULT: only the 1st axis will be transformed
  if(verb) write(0,*) "Doing backward FFT along the 1st axis..."
  if(axis1.eq.-1)then
     ! time => frequency: backward, exp(-iwt)
     call sfftw_plan_dft_1d(plan1f, n1, in(:,1,1), out(1:n1), FFTW_FORWARD, FFTW_ESTIMATE)
     do j=1,n3
        do i=1,n2
           call sfftw_execute_dft(plan1f, in(:,i,j), out(1:n1))
           in(:,i,j)=out(1:n1)/sqrt(real(n1)) ! divided by sqrt(n1) instead of 1
        end do
     end do
  else
     ! space => wavenumber: backward, exp(+ikx)
     call sfftw_plan_dft_1d(plan1f, n1, in(:,1,1), out(1:n1), FFTW_BACKWARD, FFTW_ESTIMATE)
     do j=1,n3
        do i=1,n2
           call sfftw_execute_dft(plan1f, in(:,i,j), out(1:n1))
           in(:,i,j)=out(1:n1)/sqrt(real(n1)) ! divided by sqrt(n1) instead of n1
        end do
     end do
     call sfftw_destroy_plan(plan1f)
  end if

  if(dim.eq.2 .or. dim.eq.3)then
     if(verb) write(0,*) "Doing backward FFT along the 2nd axis..."
     if(axis2.eq.-1)then
        ! time => frequency: backward, exp(-iwt)
        call sfftw_plan_dft_1d(plan1f, n2, in(1,:,1), out(1:n2), FFTW_FORWARD, FFTW_ESTIMATE)
        do j=1,n3
           do i=1,n1
              call sfftw_execute_dft(plan1f, in(i,:,j), out(1:n2))
              in(i,:,j)=out(1:n2)/sqrt(real(n2)) !divided by sqrt(n2) instead of 1
           end do
        end do
     else
        ! space => wavenumber: forward, exp(-ikx)
        call sfftw_plan_dft_1d(plan1f, n2, in(1,:,1), out(1:n2), FFTW_BACKWARD, FFTW_ESTIMATE)
        do j=1,n3
           do i=1,n1
              call sfftw_execute_dft(plan1f, in(i,:,j), out(1:n2))
              in(i,:,j)=out(1:n2)/sqrt(real(n2)) !divided by sqrt(n2) instead of n2
           end do
        end do
     end if
     call sfftw_destroy_plan(plan1f)
  end if

  if(dim.eq.3)then
     if(verb) write(0,*) "Doing backward FFT along the 3rd axis..."
     if(axis3.eq.-1)then
        ! time => frequency: backward, exp(-iwt)
        call sfftw_plan_dft_1d(plan1f, n3, in(1,1,:), out(1:n3), FFTW_FORWARD, FFTW_ESTIMATE)
        do j=1,n2
           do i=1,n1
              call sfftw_execute_dft(plan1f, in(i,j,:), out(1:n3))
              in(i,j,:)=out(1:n3)/sqrt(real(n3)) !divided by sqrt(n3) instead of 1
           end do
        end do
     else
        ! space => wavenumber: forward, exp(-ikx)
        call sfftw_plan_dft_1d(plan1f, n3, in(1,1,:), out(1:n3), FFTW_BACKWARD, FFTW_ESTIMATE)
        do j=1,n2
           do i=1,n1
              call sfftw_execute_dft(plan1f, in(i,j,:), out(1:n3))
              in(i,j,:)=out(1:n3)/sqrt(real(n3)) !divided by sqrt(n3) instead of n3
           end do
        end do
     end if
     call sfftw_destroy_plan(plan1f)
  end if


  !!******************************************************

  !! write data
  call sep_write(in)
  if(verb) write(0,*) "data have been written"

  !! work ends here

  !deallocate parameters
  deallocate(in,out)
  if(verb) write(0,*) "Deallocation completed"

end program Fkdecomp3d_program
