! Module contains a subrourine that computes conventional RTM images: inputs include source wavelet and shot record.
! 2nd order approx in time and 'selected' order approx in space

!! Conventional RTM module

module crtm2d_mod

  use sep
  use type_mod
  use wave2d_mod
  implicit none

  ! approximation: 2nd order in time and ??th order in space (nfd_time=1)
  integer   :: order_time=2
  integer   :: nfd_time=1
  real      :: pi=3.1415926536

contains

 !!! Subroutine to do rtm calculation of source wavefield from ONE shot (forward in time)
  subroutine crtm2d(  vp,source,rec,refl,icon,recadd,    &    ! icon implies the image condition we use
                     isz,isx,soutaper,jt_image,tpow,  &     ! shot location and options for tapering source and receiver initial wavefields
                     sou_grid,rec_grid,fd_grid, &
                     npad,abstop,  &   ! absorbing boundaries
                     get_sousnap,get_recsnap,get_snap, &
                     sousnap,recsnap,reflsnap,  &
                     jsnap,nt_snap, &
                     smoothvel,laplac_order,nfd_space,verb )

    ! icon: [1] cross-correlation, [2] normalized cross-correlation
    ! laplac_order: 4 or 10

    ! define input/output parameters
    logical             :: abstop,verb,get_sousnap,get_recsnap,get_snap,smoothvel, soutaper, recadd
    integer             :: icon,isz,isx,laplac_order ! source and receiver locations on FD grid (x,z)
    integer             :: jsnap,nt_snap,nfd_space, jt_image
    type(grid)          :: fd_grid,sou_grid,rec_grid  ! FD stencils already padded & source & receiver grids
    type(npad_values)   :: npad       ! # of padding stencils already applied in model grid

    real, intent(in)    :: vp(fd_grid%z%n-npad%top-npad%bot, fd_grid%x%n-npad%left-npad%right)  ! acoustic velocity without padding layers
    real, intent(in)    :: source(sou_grid%t%n)         ! source wavelet
    real, intent(in)    :: rec(rec_grid%z%n,rec_grid%x%n,rec_grid%t%n)  ! recorded data
    real, intent(inout) :: refl(fd_grid%z%n-npad%top-npad%bot, fd_grid%x%n-npad%left-npad%right)  !! RTM image

    real, intent(inout) :: sousnap(fd_grid%z%n-npad%top-npad%bot, fd_grid%x%n-npad%left-npad%right, nt_snap)
    real, intent(inout) :: recsnap(fd_grid%z%n-npad%top-npad%bot, fd_grid%x%n-npad%left-npad%right, nt_snap)
    real, intent(inout) :: reflsnap(fd_grid%z%n-npad%top-npad%bot, fd_grid%x%n-npad%left-npad%right, nt_snap)

    ! local parameters    
    real     :: p(1-nfd_space:fd_grid%z%n+nfd_space,1-nfd_space:fd_grid%x%n+nfd_space,3) ! source wavefield stencils
    real     :: rr, dt,dx,dz,ot,ox,oz, dgx,dgz,ogx,ogz, dsx,dsz,osx,osz, t,tpow
    integer  :: iz,ix,it,igx,igz,i,j,k, nz,nx,nt, ngz,ngx, nsz,nsx, it_snap, sou_iz,sou_ix
    integer  :: rec_iz,rec_ix, rec_ix_begin,rec_ix_end, rec_iz_begin,rec_iz_end, nt_image,it_image

    real     :: vel(fd_grid%z%n,fd_grid%x%n)
    real     :: temp(fd_grid%z%n-npad%top-npad%bot, fd_grid%x%n-npad%left-npad%right)
    real,dimension(:,:,:),allocatable     :: fullwvfld

    ! define wavefield time axis
    nt_image = (fd_grid%t%n-1)/jt_image + 1
    allocate(fullwvfld(fd_grid%z%n-npad%top-npad%bot,fd_grid%x%n-npad%left-npad%right,nt_image)) ! saved wvfld

    ! checking choices of image conditions
    if(icon.ne.1 .and. icon.ne.2) call seperr("WARNING: image condition icon must be 1 or 2")

    !----------------------------------------------
    ! checking grid info
    if((sou_grid%t%d.ne.fd_grid%t%d).and.(rec_grid%t%d.ne.fd_grid%t%d)) then
       call seperr("Source, receiver and FD grid intervals (d's) along time axis need to be compatible")
    end if
    if((sou_grid%t%o.ne.fd_grid%t%o).and.(rec_grid%t%o.ne.fd_grid%t%o)) then
       call seperr("Source, receiver and FD grid origins (o's) along time axis need to be compatible (=zero)")
    end if
    if(rec_grid%t%n.eq.fd_grid%t%d) call seperr("For RTM, # of receiver grids must equal that of FD grids")
    
    ! Check if the receiver grids in x- and z-axes have been interpolated to be compatible with FD grids
    if((rec_grid%x%n.ne.1) .and. (rec_grid%x%d.ne.fd_grid%x%d)) call seperr("The receiver grid in x-axis need to be interpolated")
    if((rec_grid%z%n.ne.1) .and. (rec_grid%z%d.ne.fd_grid%z%d)) call seperr("The receiver grid in z-axis need to be interpolated")
 
    if(.not.abstop .and. npad%top.ne.0)then
       call seperr("When top boundary is reflective type (abstop=.false.), no padding at the top must be applied (npad%top=0).")
    end if
    !----------------------------------------------
    if(verb) write(0,*) "grid checking passed"
    !----------------------------------------------
    !! set up initial values and the boundary conditions
    ! symbols for FD grids with padding regions
    dz=fd_grid%z%d  !
    dx=fd_grid%x%d  ! just for easier to write the program 
    dt=fd_grid%t%d  !
    nz=fd_grid%z%n  !
    nx=fd_grid%x%n  ! just for easier to write the program 
    nt=fd_grid%t%n  !
    oz=fd_grid%z%o  !
    ox=fd_grid%x%o  ! just for easier to write the program 
    ot=fd_grid%t%o  !
    ! symbols for receiver grids
    dgz=rec_grid%z%d  !
    dgx=rec_grid%x%d  ! just for easier to write the program 
    ngz=rec_grid%z%n  !
    ngx=rec_grid%x%n  ! just for easier to write the program 
    ogz=rec_grid%z%o  !
    ogx=rec_grid%x%o  ! just for easier to write the program 
    ! symbols for source grids
    dsz=sou_grid%z%d  !
    dsx=sou_grid%x%d  ! just for easier to write the program 
    nsz=sou_grid%z%n  !
    nsx=sou_grid%x%n  ! just for easier to write the program 
    osz=sou_grid%z%o  !
    osx=sou_grid%x%o  ! just for easier to write the program 

    ! compute the source location on the FD grids (z,x)
    sou_iz = npad%top + 1.5 + ( (osz+(isz-1)*dsz) - oz )/dz
    sou_ix = npad%left + 1.5 + ( (osx+(isx-1)*dsx) - ox )/dx
    ! compute the depth(z) of the receiver plane on the FD grids (z) and the edge points on the horizontal plane
    rec_iz_begin = npad%top + 1.5 + ( ogz - oz )/dz
    rec_iz_end = npad%top + 1.5 + ( (ogz+(ngz-1)*dgz) - oz )/dz
    if(rec_iz_begin.gt.rec_iz_end) call seperr("dz must not be negative") 

    rec_ix_begin = npad%left + 1.5 + ( ogx - ox )/dx
    rec_ix_end = npad%left + 1.5 + ( (ogx+(ngx-1)*dgx) - ox )/dx
    if(rec_ix_begin.gt.rec_ix_end) call seperr("dx must not be negative") 


    ! initialize wavefields 
    ! But, we are not initializing refl(:,:), since it is an accumulated result from many shots.
    if(verb) write(0,*) "Initializing data....."
    p(:,:,:)=0.
    sousnap(:,:,:)=0.
    recsnap(:,:,:)=0.
    reflsnap(:,:,:)=0.
    fullwvfld=0.
    if(verb) write(0,*) "passed"

    !!!===============================
    !! Adjust the velocity model
    !!!===============================
    ! calculate velocity to be compatible with the FD scheme
    vel(:,:)=0.
    do iz=1,nz-npad%top-npad%bot
       do ix=1,nx-npad%left-npad%right
          vel(iz+npad%top,ix+npad%left) = vp(iz,ix)
       end do
    end do
    ! extrapolate the velocity in the padding regions and the FD boundaries
    do iz=1,npad%top;         vel(iz,:) = vel(1+npad%top,:);    end do  ! top bound
    do iz=nz-npad%bot+1,nz;   vel(iz,:) = vel(nz-npad%bot,:);   end do  ! bottom bound
    do ix=1,npad%left;        vel(:,ix) = vel(:,1+npad%left);   end do  ! left bound 
    do ix=nx-npad%right+1,nx; vel(:,ix) = vel(:,nx-npad%right); end do  ! right bound

    if(smoothvel) then  ! smoothen velocity for laplacian operator if needed
       call smooth2d(vel,nz,nx,nfd_space)
    end if

    !!================================================
    !! begin extrapolation scheme for source wavefield
    !!================================================
    write(0,*) "Extrapolating source wavefields (forward in time)...."

    do it=1,fd_grid%t%n   ! time forward loop
       t = ot + float(it-1)*dt
       !!========================================================================
       !! Add boundary conditions
          ! Add the source wavelet at the source location: The source is 
          ! introduced over a small region (not at just one point to prevent 
          ! energy at nyquist wavelength from entering the calculation)
          if(it.le.sou_grid%t%n) then
             if(soutaper) then
                do iz=sou_iz-2,sou_iz+2
                   do ix=sou_ix-2,sou_ix+2
                      rr=sqrt(real((iz-sou_iz)**2+(ix-sou_ix)**2))    ! sou_ix is source location on FD grids
                      p(iz,ix,2) = p(iz,ix,2) - source(it)*.5*(1.+cos(rr/3*pi)) ! add source BC using Tukey window tapering
                   end do
                end do
             else
                p(sou_iz,sou_ix,2) = p(sou_iz,sou_ix,2) - source(it)
             end if
          end if

       !!========================================================================

       ! Compute the laplacian of the wavefield at the current time p(:,:,2)
       call laplac2d(p(:,:,2),p(:,:,3),dz,dx,nz,nx,nfd_space,laplac_order)
!       write(0,*)"pass1"

       do iz=1,nz        ! vertical loop
          do ix=1,nx     ! horizontal loop             
             ! extrapolation step second-order in time
             p(iz,ix,3) = dt*dt*vel(iz,ix)*vel(iz,ix)*p(iz,ix,3)
             p(iz,ix,3) = p(iz,ix,3) + 2*p(iz,ix,2) - p(iz,ix,1) ! extrapolate wave in time
          end do
       end do
       do iz=1,nz        ! vertical loop
          do ix=1,nx     ! horizontal loop            
             !!========================================================================
             !! update wavefield within the data grids ready for next time step
             p(iz,ix,1) = p(iz,ix,2)
             p(iz,ix,2) = p(iz,ix,3)
             !!========================================================================
          end do   ! horizontal x loop end
       end do   ! vertical z loop end
!       write(0,*) "pass2"

       !! apply absorbing tapering and/or absorbing FD boundary
       call taper(vel,p,dt,dz,dx,nz,nx,npad,abstop,nfd_space)
!       write(0,*) "pass3"

       !!===================================================
       !! get snapshot of source wavefield
       !!===================================================
       
       if(get_sousnap) then
          if(mod(it,jsnap).eq.1) then
             it_snap = 1 + (it-1)/jsnap
             if(it_snap.le.nt_snap) then
                do iz=1,fd_grid%z%n-npad%top-npad%bot
                   do ix=1,fd_grid%x%n-npad%left-npad%right
                      sousnap(iz,ix,it_snap) = p(iz+npad%top,ix+npad%left,1)*(t**tpow)    ! choose snapshots (notice we apply tpow here as well)
                      ! sousnap(:,:,it_snap) = p(1+npad%top:nz-npad%bot,1+npad%left:nx-npad%right,1)     ! choose snapshots
                   end do
                end do
             end if
          end if
       end if
!       write(0,*) "pass4"

       !! store the source wavefield at selected time in memmory (such a ridiculous opinion! I am revising it, though.)
       if(mod(it,jt_image).eq.1) then
          it_image = 1 + (it-1)/jt_image
          !fullwvfld(:,:,it_image) = p(:,:,1)
          do iz=1,nz-npad%top-npad%bot  
             do ix=1,nx-npad%left-npad%right
                fullwvfld(iz,ix,it_image) = p(iz+npad%top,ix+npad%left,1)*(t**tpow) 
             end do
          end do
       end if
!       write(0,*) "pass5"
       
       if(verb .and. mod(it,jsnap).eq.1) write(0,*) it, sum(p(:,:,1)), p(sou_iz,sou_ix,1)
    
    end do   !!! time loop end
    !!================================================
    !! end extrapolation scheme for source wavefield
    !!================================================

!!===========================================================================================================
!!****** Let's use temp as the determinator for normalized RTM IC ******!!
    if(icon.eq.2) then ! for normalized cross-correlation IC
       temp(:,:) = 0.
       do it=1,nt_image; temp(:,:) = temp(:,:)+fullwvfld(:,:,it)*fullwvfld(:,:,it); end do
    end if
!!****** Hereafter, vp(:,:) will not be used. Let's use it as the determinator for normalized RTM IC ******!!
!!===========================================================================================================

    !!==================================================
    !! begin extrapolation scheme for RECEIVER wavefield
    !!==================================================
    write(0,*) "Extrapolating receiver wavefields (backward in time)...."
    write(0,*) "Meanwhile, computing rtm images"
    p=0.  ! clear values of the tensor p(:,:,:)

    do it=fd_grid%t%n,1,-1   ! time backward loop
       t = ot + float(it-1)*dt

       !!========================================================================
       !! Place the receiver wavefield at z=0 ====================
       !! no tapering the boundary condition (recorded data or initial receiver wavefield)
       do igz=1,ngz
          rec_iz = npad%top + 1.5 + ( (ogz+(igz-1)*dgz) - oz )/dz
          do igx=1,ngx
             rec_ix = npad%left + 1.5 + ( (ogx+(igx-1)*dgx) - ox )/dx
             if(recadd) then
                p(rec_iz,rec_ix,2) = p(rec_iz,rec_ix,2) + rec(igz,igx,it)
             else
                p(rec_iz,rec_ix,2) = rec(igz,igx,it)
             end if
          end do
       end do
       !! end boundary condition stuff =======================
       !!========================================================================

       
       ! Compute the laplacian of the wavefield at the current time p(:,:,2)
       call laplac2d(p(:,:,2),p(:,:,3),dz,dx,nz,nx,nfd_space,laplac_order)
       
       do iz=1,nz        ! vertical loop
          do ix=1,nx     ! horizontal loop             
             ! extrapolation step second-order in time
             p(iz,ix,3) = dt*dt*vel(iz,ix)*vel(iz,ix)*p(iz,ix,3)
             p(iz,ix,3) = p(iz,ix,3) + 2*p(iz,ix,2) - p(iz,ix,1) ! extrapolate wave in time
          end do
       end do
       do iz=1,nz        ! vertical loop
          do ix=1,nx     ! horizontal loop            
             !!========================================================================
             !! update wavefield within the data grids ready for next time step
             p(iz,ix,1) = p(iz,ix,2)
             p(iz,ix,2) = p(iz,ix,3)
             !!========================================================================
          end do   ! horizontal x loop end
       end do   ! vertical z loop end
!       write(0,*) "pass2"

       !! apply absorbing tapering and/or absorbing FD boundary
       call taper(vel,p,dt,dz,dx,nz,nx,npad,abstop,nfd_space)
!       write(0,*) "pass3"

       !!===================================================
       !! get snapshots of receiver wavefield
       !!===================================================
       if(get_recsnap) then
          if(mod(it,jsnap).eq.1) then
             it_snap = 1 + (it-1)/jsnap
             if(it_snap.le.nt_snap) then
                do iz=1,fd_grid%z%n-npad%top-npad%bot
                   do ix=1,fd_grid%x%n-npad%left-npad%right
                      recsnap(iz,ix,it_snap) = p(iz+npad%top,ix+npad%left,1)*(t**tpow)     ! choose snapshots
                      ! recsnap(:,:,it_snap) = p(1+npad%top:nz-npad%bot,1+npad%left:nx-npad%right,1)     ! choose snapshots
                   end do
                end do
             end if
          end if
       end if
!       write(0,*) "pass4"
       !!

!       if(mod(it,jt_image).eq.1) then
!          it_image = 1 + (it-1)/jt_image
!          fullwvfld(:,:,it_image) = p(:,:,1)
!       end if
       !!===================================================
       !!******** apply image condition for RTM ************
       !!===================================================
       if(mod(it,jt_image).eq.1) then
          it_image = 1 + (it-1)/jt_image
          if(icon.eq.1)then
             ! cross-correlation
             do iz=1,nz-npad%top-npad%bot  
                do ix=1,nx-npad%left-npad%right
                   refl(iz,ix) = refl(iz,ix) + fullwvfld(iz,ix,it_image)*p(iz+npad%top,ix+npad%left,1)*(t**tpow)
                end do
             end do
          elseif(icon.eq.2)then
             ! normalized cross-correlation
             do iz=1,nz-npad%top-npad%bot
                do ix=1,nx-npad%left-npad%right
                   refl(iz,ix) = refl(iz,ix) + fullwvfld(iz,ix,it_image)*(p(iz+npad%top,ix+npad%left,1)*(t**tpow))/(temp(iz,ix)+0.01)
                end do
             end do
          end if
       end if
       !!===================================================
       !!===================================================

       !!===================================================
       !! get snapshots of image condition
       !!===================================================
       if(get_snap) then
          if(mod(it,jsnap).eq.1) then
             it_snap = 1 + (it-1)/jsnap
             if(it_snap.le.nt_snap) then
                do iz=1,fd_grid%z%n-npad%top-npad%bot
                   do ix=1,fd_grid%x%n-npad%left-npad%right
                      reflsnap(iz,ix,it_snap) = refl(iz,ix)     ! choose snapshots
                   end do
                end do
             end if
          end if
       end if

       if(verb .and. mod(it,jsnap).eq.1) write(0,*) it, sum(p(:,:,1)), sum(refl(:,:))
    
    end do   !!! time loop end
    !!================================================
    !! end extrapolation scheme for RECEIVER wavefield
    !!================================================

    deallocate(fullwvfld)

  end subroutine crtm2d


end module crtm2d_mod
