!	   		       crtm2d_program
!
!            Conventional RTM program
!            by time stepping using an explicit finite-difference 
!	     solution to the acoustic wave equation accurate to 2nd order 
!            in time and 4th order in x and z.
!
!		Author: Kittinat Taweesintananon
!
! Example:
!
! <vel.H rtm2d_program.x source=wvlet.H receiver=recwvfld.H par=parfile  > refl.H snap=snap.H sousnap=sousnap.H recsnap=recsnap.H  
!
!
!	Necessary input files:
!
!		vel.H is the Header for the velocity model, stdin: dimension /z x/
!
!               wvlet.H is an auxilary file containing source wavelet file: dimension /t/
!
!		recwvfld.H is an auxilary file containing receiver wavefield file: dimension /gz gx t sx sz/
!
!		parfile  contains the geometry parameters
!			 for the synthetic seismic survey
!
!	Output files:
!
!		refl.H    Header file for the output reflectivity: dimension /z x/
!
!		snap.H	  Header for the (optional) output snapshots of the cross-correlation related to the first shot: dimension /z x t/
!
!		sousnap.H	  Header for the (optional) output snapshots from the source wavelet of the first shot: dimension /z x t/
!
!		recsnap.H	  Header for the (optional) output snapshots from the receiver wavefield related to the first shot: dimension /z x t/
!
! **Necessary parameters****************************************************
! ****** most quantitites like distances refer to grid points **************
!
!	From wvlet.H:	n1 	# of gridpoints in t 
!			d1	t gridpoint spacing (becomes dt)
!
!	From vel.H:	n1 	# of gridpoints in z (becomes nz)
!			n2	# of gridpoints in x (becomes nx)
!			d1	z gridpoint spacing (becomes dz)
!			d2	x gridpoint spacing (becomes dx)
!
!From par:	
!
! rollalong=.false.	        #roll the receiver spread along with shots
!				# [1, .true.] marine acquisition(offshore): one vessel and towed streamers behind 
!                       	# the hydrophone locations (all location parameters) are with respect to the vessel (source locations)
!                       	# [0, .false.=default] land or OBC acquisition(onshore): 
!				# the geophone locations are with respect to the earth frame (the velocity model);
! 
! abstop=y        # [1,y = default] apply absorbing top boundary, so artifacts from multiples would disappear
!                 # [0,n] apply full reflective top boundary, so weak images from multiples would disappear
! npadtop,npadbot,npadleft,npadright  # the number of padding expansion (gaussian tapering) for top,bottome,left,right boundaries
! percent_taper          # [98.] percent of absorbed amplitude at the end of pad (using gaussian tapering)
!
! soutaper            # [0,.false.] apply taper to the initial source wavefield BC
! recadd              # [1,.true.] accumulate the receiver boundary wavefields to the grids
! tpow                # [0] gain trace data (extrapolated wavefield) by t**tpow
!
! nt,dt	              #   Time length/interval of the shot records 
! jt_image            #   [8] The time increment for computing the cross-correlation of wavefields
!
! get_snap=.false. # [0,.false.=default] for printing out the cross-correlation snapshot 
! get_sousnap=.false.
! get_recsnap=.false.      
! jsnap            # snapshot time grid increment [100]
!
! icon=1        # image condition we use in RTM: [1,default] cross-correlation, [2] normalized cross-correlation
! smoothvel=0   # use smoothened velocity when using FD scheme: [1,.true] or [0,.false.=default]
! laplac_order=4  # the order of laplacian computing: selected from 2 or 4(default) or 10 
!
!! Source, receiver and FD grids are already in source and recorded wavefield files.
!
!================================================================

program crtm2d_program

  use sep
  use type_mod
  use crtm2d_mod

  implicit none

  !!====================================================================
  !! define parameters
  logical             :: verb, smoothvel, soutaper, recadd
  logical             :: abstop, rollalong, get_snap, get_sousnap, get_recsnap

  type(grid)          :: fd_grid,sou_grid,rec_grid  ! FD stencils already padded & source & receiver grids
  type(npad_values)   :: npad       ! # of padding stencils already applied in model grid

  real                :: dt_snap, dt,dx,dz,ot,ox,oz, dgx,dgz,ogx,ogz, dsx,dsz,osx,osz, tpow
  integer             :: nt_snap, nz,nx,nt, ngz,ngx, nsz,nsx, icon,laplac_order,nfd_space

  integer             :: it_snap, i,j,k, iz,ix,it,igz,igx,isz,isx, sou_iz,sou_ix, rec_iz,rec_ix, jsnap, jt_image      ! indices

  real,dimension(:),allocatable      :: source
  real,dimension(:,:),allocatable    :: vp, refl
  real,dimension(:,:,:),allocatable  :: snap, sousnap, recsnap !, rec
  real,dimension(:,:,:,:,:),allocatable  :: receiver

  !!--------------------
  character(len=128)  :: source_,receiver_,snap_,sousnap_,recsnap_    ! Define the tagnames for the auxilary files 
  source_ ="source"; receiver_="receiver"; snap_ ="snap"; sousnap_="sousnap"; recsnap_="recsnap"      ! [source wavelet and snapshot wavefield]

  !!====================================================================
  !!======================input/output stuff============================
  call sep_init()

  !!====================================================================
  !! read the parameters from the command lines

  
  !!---------grid setting------------
  !! FD grids info
  ! get parameters from the input history file [velocity parameters, v(z,x)]
    call from_history("n1",fd_grid%z%n)   ! from velocity input v(z,x)
    call from_history("n2",fd_grid%x%n)   ! from velocity input v(z,x)
    call from_history("d1",fd_grid%z%d)   ! from velocity input v(z,x)
    call from_history("d2",fd_grid%x%d)   ! from velocity input v(z,x)    
    call from_history("o1",fd_grid%z%o,0.)   ! from velocity input v(z,x)
    call from_history("o2",fd_grid%x%o,0.)   ! from velocity input v(z,x)

    call from_aux(receiver_,"n3",fd_grid%t%n)
    call from_aux(receiver_,"d3",fd_grid%t%d)
    call from_aux(receiver_,"o3",fd_grid%t%o,0.)

  !! Source grids info
    call from_aux(receiver_,"n4",sou_grid%x%n)
    call from_aux(receiver_,"d4",sou_grid%x%d)
    call from_aux(receiver_,"o4",sou_grid%x%o)
    call from_aux(receiver_,"n5",sou_grid%z%n)
    call from_aux(receiver_,"d5",sou_grid%z%d)
    call from_aux(receiver_,"o5",sou_grid%z%o)

    ! get time parameters from the source wavelet file [parameters of source signal]
    call from_aux(source_,"n1",sou_grid%t%n)   ! # of t gridpoints for the source signal
    call from_aux(source_,"d1",sou_grid%t%d)
    call from_aux(source_,"o1",sou_grid%t%o,0.)

  !! Receiver grids info
    call from_aux(receiver_,"n2",rec_grid%x%n)
    call from_aux(receiver_,"d2",rec_grid%x%d)
    call from_aux(receiver_,"o2",rec_grid%x%o)
    call from_aux(receiver_,"n1",rec_grid%z%n)
    call from_aux(receiver_,"d1",rec_grid%z%d)
    call from_aux(receiver_,"o1",rec_grid%z%o)

    call from_aux(receiver_,"n3",rec_grid%t%n)
    call from_aux(receiver_,"d3",rec_grid%t%d)
    call from_aux(receiver_,"o3",rec_grid%t%o,0.)

  ! Check time grids
    if(rec_grid%t%d.ne.sou_grid%t%d)then
       write(0,*) "WARNING: source wavelet dt is not the same as recording dt"
       write(0,*) "source wavelet dt, and recording dt are", sou_grid%t%d, rec_grid%t%d 
       call seperr("time inverval of source and receiver signals must be equal")  ! time grid spacing of receiver
    end if

  ! Check if the receiver grids in x-axis have been interpolated to be compatible with FD grids
    if(rec_grid%x%d.ne.fd_grid%x%d) call seperr("The receiver grid in x-axis need to be interpolated")

    write(0,*) "read axes completed"
  !!---------grid setting completed------------

  !!====================================================================
  !!====================================================================
  !!---------parameter reading------------
    write(0,*) "Reading parameters....."

    call from_param("verb",verb,.false.)   ! verbal description
    call from_param("rollalong",rollalong,.false.)  ! roll the receiver spread along with shots: 
    call from_param("jt_image",jt_image,8) ! The time increment for computing the cross-correlation of wavefields

  !! read parameters for absorbing/reflective boundaries 
    call from_param("abstop",abstop,.true.)  ! type of top boundary: [.true.=default] for absorbing bound; [.false.] for reflective bound.
    if(abstop)then
       call from_param("npadtop",npad%top,0)
    else
       npad%top=0
    end if
    call from_param("npadbot",npad%bot,0)    ! padding expansion
    call from_param("npadleft",npad%left,0); call from_param("npadright",npad%right,0);    ! padding expansion
    call from_param("percent_taper",npad%taper,98.)
    
  !! parameters for tapering source and receiver wavefields
    call from_param("soutaper",soutaper,.false.)
    call from_param("recadd",recadd,.true.) ! accumulate the boundary values
    call from_param("tpow",tpow,0.) ! gain with t**tpow

  !! ================================
  !! read parameters for wavefield snapshot
    call from_param("get_snap",get_snap,.false.)   ! snapshot of cross-correlation
    call from_param("get_sousnap",get_sousnap,.false.)  ! snapshot of source wavefield
    call from_param("get_recsnap",get_recsnap,.false.)  ! snapshot of receiver wavefield

    call from_param("jsnap",jsnap,100)
    nt_snap = (fd_grid%t%n-1)/jsnap + 1
    dt_snap = fd_grid%t%d*jsnap

  !! parameters for RTM operation
    ! choices of image conditions for RTM [1] cross-correlation, [2] normalized cross-correlation
    call from_param("icon",icon,1) 
    ! use smoothened velocity during extrapolation process or not
    call from_param("smoothvel",smoothvel,.true.)
    ! order of approx in space for FD computing (4 or 10)
    call from_param("laplac_order",laplac_order,4)
    ! set up the laplacian order
    if(laplac_order.eq.4) then 
       nfd_space=2
    else if(laplac_order.eq.10) then
       nfd_space=5
    else if(laplac_order.eq.2)then
       nfd_space=1
    else
       write(0,*) "laplac_order:",laplac_order
       call seperr("WARNING: the laplacian order must be 2, 4 or 10")
    end if


  !!====================================================================
  !!====================================================================
  !! write axes parameters to the history files
    !! output file (refl.H => reflectivity file /z x/)
    call to_history("n1",fd_grid%z%n); call to_history("d1",fd_grid%z%d); call to_history("o1",fd_grid%z%o); call to_history("label1", 'z(m)');  ! z
    call to_history("n2",fd_grid%x%n); call to_history("d2",fd_grid%x%d); call to_history("o2",fd_grid%x%o); call to_history("label2", 'x(m)');  ! x


    !! write axes for cross-correlation snapshot auxilary file (snap.H => /z x t/)
    if(get_snap) then
       ! receiver z
       call auxputch("n1",'i', fd_grid%z%n, snap_); call auxputch("d1",'f', fd_grid%z%d,snap_);  
       call auxputch("o1",'f', fd_grid%z%o, snap_); call auxputch("label1", 's', 'z', snap_);
       ! receiver x
       call auxputch("n2",'i', fd_grid%x%n,snap_); call auxputch("d2",'f', fd_grid%x%d, snap_);  
       call auxputch("o2",'f', fd_grid%x%o,snap_); call auxputch("label2", 's', 'x', snap_);
       ! time
       call auxputch("n3",'i', nt_snap,snap_); call auxputch("d3",'f',dt_snap,snap_);  
       call auxputch("o3",'f', fd_grid%t%o,snap_); call auxputch("label3", 's', 'time', snap_);
    end if

    !! write axes for source wavefield snapshot auxilary file (sousnap.H => /z x t/)
    if(get_sousnap) then
       ! receiver z
       call auxputch("n1",'i', fd_grid%z%n, sousnap_); call auxputch("d1",'f', fd_grid%z%d,sousnap_);  
       call auxputch("o1",'f', fd_grid%z%o, sousnap_); call auxputch("label1", 's', 'z', sousnap_);
       ! receiver x
       call auxputch("n2",'i', fd_grid%x%n,sousnap_); call auxputch("d2",'f', fd_grid%x%d, sousnap_);  
       call auxputch("o2",'f', fd_grid%x%o,sousnap_); call auxputch("label2", 's', 'x', sousnap_);
       ! time
       call auxputch("n3",'i', nt_snap,sousnap_); call auxputch("d3",'f',dt_snap,sousnap_);  
       call auxputch("o3",'f', fd_grid%t%o,sousnap_); call auxputch("label3", 's', 'time', sousnap_);
    end if

    !! write axes for receiver wavefield snapshot auxilary file (recsnap.H => /z x t/)
    if(get_recsnap) then
       ! receiver z
       call auxputch("n1",'i', fd_grid%z%n, recsnap_); call auxputch("d1",'f', fd_grid%z%d,recsnap_);  
       call auxputch("o1",'f', fd_grid%z%o, recsnap_); call auxputch("label1", 's', 'z', recsnap_);
       ! receiver x
       call auxputch("n2",'i', fd_grid%x%n,recsnap_); call auxputch("d2",'f', fd_grid%x%d, recsnap_);  
       call auxputch("o2",'f', fd_grid%x%o,recsnap_); call auxputch("label2", 's', 'x', recsnap_);
       ! time
       call auxputch("n3",'i', nt_snap,recsnap_); call auxputch("d3",'f',dt_snap,recsnap_);  
       call auxputch("o3",'f', fd_grid%t%o,recsnap_); call auxputch("label3", 's', 'time', recsnap_);
    end if
    write(0,*) "write axes completed"
    

  !! write out the input parameters
    write(0,*) '***********************************'
    write(0,*) '***********************************'
    write(0,*) 'rollalong',rollalong
    write(0,*) '***********************************'
    write(0,*) 'ngz,dgz,ogz', rec_grid%z%n, rec_grid%z%d, rec_grid%z%o
    write(0,*) 'ngx,dgx,ogx', rec_grid%x%n, rec_grid%x%d, rec_grid%x%o
    write(0,*) 'nt, dt, ot', rec_grid%t%n, rec_grid%t%d, rec_grid%t%o
    write(0,*) 'nsx,dsx,osx', sou_grid%x%n, sou_grid%x%d, sou_grid%x%o
    write(0,*) 'nsz,dsz,osz', sou_grid%z%n, sou_grid%z%d, sou_grid%z%o
    write(0,*) '***********************************'
    write(0,*) '************* FD grid *************'
    write(0,*) 'nz,dz,oz', fd_grid%z%n, fd_grid%z%d, fd_grid%z%o
    write(0,*) 'nx,dx,ox', fd_grid%x%n, fd_grid%x%d, fd_grid%x%o
    write(0,*) 'jt_image', jt_image
    write(0,*) '***********************************'
    write(0,*) 'boundaries...'
    write(0,*) 'abstop', abstop
    write(0,*) 'npadtop,npadbot,npadleft,npadright', npad%top,npad%bot,npad%left,npad%right
    write(0,*) 'percent_taper', npad%taper
    write(0,*) 'soutaper,tpow', soutaper,tpow
    write(0,*) 'get_snap,get_sousnap,get_recsnap', get_snap,get_sousnap,get_recsnap
    write(0,*) 'jsnap, nt_snap, dt_snap', jsnap,nt_snap,dt_snap
    write(0,*) 'icon,smoothvel,laplac_order,nfd_space',icon,smoothvel,laplac_order,nfd_space
    write(0,*) '***********************************'
  call sep_close()

  !!====================================================================
  !!=========================end input/output===========================

  !!================================
  !! Expand FD domain by padding layers (only the number of gridpoints)
  fd_grid%z%n = fd_grid%z%n + npad%top + npad%bot
  fd_grid%x%n = fd_grid%x%n + npad%left + npad%right
  !!================================

    ! symbols for FD grids
    dz=fd_grid%z%d  !
    dx=fd_grid%x%d  ! just for easier to write the program 
    dt=fd_grid%t%d  !
    nz=fd_grid%z%n  !
    nx=fd_grid%x%n  ! just for easier to write the program 
    nt=fd_grid%t%n  !
    oz=fd_grid%z%o  !
    ox=fd_grid%x%o  ! just for easier to write the program 
    ot=fd_grid%t%o  !
    ! symbols for receiver grids
    dgz=rec_grid%z%d  !
    dgx=rec_grid%x%d  ! just for easier to write the program 
    ngz=rec_grid%z%n  !
    ngx=rec_grid%x%n  ! just for easier to write the program 
    ogz=rec_grid%z%o  !
    ogx=rec_grid%x%o  ! just for easier to write the program 

    ! symbols for source grids
    dsz=sou_grid%z%d  !
    dsx=sou_grid%x%d  ! just for easier to write the program 
    nsz=sou_grid%z%n  !
    nsx=sou_grid%x%n  ! just for easier to write the program 
    osz=sou_grid%z%o  !
    osx=sou_grid%x%o  ! just for easier to write the program 

    !------------------------------------

    ! allocate space for the velocity model, source wavelet, data, wavefields
  allocate(source(sou_grid%t%n))
  allocate(vp(fd_grid%z%n-npad%top-npad%bot, fd_grid%x%n-npad%left-npad%right))    ! /nz, nx/ no pad
  allocate(refl(fd_grid%z%n-npad%top-npad%bot, fd_grid%x%n-npad%left-npad%right))   ! reflectivity /nz, nx/ no pad
  allocate(snap(fd_grid%z%n-npad%top-npad%bot, fd_grid%x%n-npad%left-npad%right, nt_snap))  ! snap /nz, nx, nt_snap/
  allocate(sousnap(fd_grid%z%n-npad%top-npad%bot, fd_grid%x%n-npad%left-npad%right, nt_snap))
  allocate(recsnap(fd_grid%z%n-npad%top-npad%bot, fd_grid%x%n-npad%left-npad%right, nt_snap))
  allocate(receiver(rec_grid%z%n, rec_grid%x%n, rec_grid%t%n,sou_grid%x%n,sou_grid%z%n))   ! shot record /ngz, ngx, nt, nsx, nsz/
!  allocate(rec(fd_grid%z%n,fd_grid%x%n,fd_grid%t%n))

  write(0,*) "allocation completed"

  ! Read source wavelet
  call sep_read(source(1:sou_grid%t%n),source_)
  write(0,*) "source wavelet is read"

  ! Read recorded receiver wavefield
  call sep_read(receiver,receiver_)
  write(0,*) "recorded receiver wavefield is read"

  ! Read velocity model
  call sep_read(vp) ! original FD grid size
  write(0,*) "the velocity model is read"

  ! initialize to zero
  refl=0.;snap=0.;recsnap=0.;sousnap=0.

    ! generate the shot records for all shots
  do isz=1,nsz     ! loop over shots
     do isx=1,nsx

          write(0,*) "********************************************************"
          write(0,*) "**Computing RTM for the shot number isz, isx ", isz, isx
          write(0,*) "**Computing RTM for the shot sz, sx ", osz+(isz-1)*dsz, osx+(isx-1)*dsx
          write(0,*) "********************************************************"

          ! compute source locations on the FD grids (z,x)
          ! sou_iz = npad%top + 1.5 + ( (osz+(isz-1)*dsz) - oz )/dz
          ! sou_ix = npad%left + 1.5 + ( (osx+(isx-1)*dsx) - ox )/dx

          if(rollalong) then  ! For marine acquisition, the origin of streamer on x-axis is with respect to each source location.
             rec_grid%x%o = ogx + osx + (isx-1)*dsx    ! Thus, change the origin of streamer for every shot.
          end if

          ! compute the depth(z) of the receiver plane on the FD grids (z)
          ! rec_iz = npad%top + 1.5 + ( (ogz+(igz-1)*dgz) - oz )/dz
          ! do igx=1,ngx
          !    rec_ix = npad%left + 1.5 + ( (rec_grid%x%o+(igx-1)*dgx) - ox )/dx
          !    rec(rec_iz,rec_ix,:)=receiver(igz,igx,:,isx,isz) ! rec() is the recorded data (receiver wf) in FD grids 
          ! end do

          ! compute rtm image for each shot
          write(0,*) "calling crtm2d for shot isz isx:",isz,isx

          call crtm2d(  vp,source,receiver(:,:,:,isx,isz),refl,icon,recadd,    &    ! icon implies the image condition we use
                     isz,isx,soutaper,jt_image,tpow, &
                     sou_grid,rec_grid,fd_grid, &
                     npad,abstop,  &   ! absorbing boundaries
                     get_sousnap.and.(isz*isx.eq.1), &
                     get_recsnap.and.(isz*isx.eq.1), &
                     get_snap.and.(isz*isx.eq.1), &
                     sousnap,recsnap,snap,  &
                     jsnap,nt_snap, &
                     smoothvel,laplac_order,nfd_space,verb )

          ! write out the source wavefield snapshot for the 1st shot
          if(get_sousnap .and. (isz*isx.eq.1)) then  
             write(0,*) "writing out the first source snapshot"
             call sep_write(sousnap,sousnap_)  
          end if

          ! write out the receiver wavefield snapshot for the 1st shot
          if(get_recsnap .and. (isz*isx.eq.1)) then  
             write(0,*) "writing out the first receiver snapshot"
             !! write out the snapshot file
             call sep_write(recsnap,recsnap_)  
          end if

          !! write out the snapshot file of crosscorrelation for the 1st shot
          if(get_snap .and. (isz*isx.eq.1)) then
             write(0,*) "writing out the first reflectivity snapshot"
             call sep_write(snap,snap_)  
          end if
          !!---

     end do
   end do     ! end loop over shots
   

  !!================================
  !! Reset FD domain by deleting the padding layers (only the number of gridpoints)
  fd_grid%z%n = fd_grid%z%n - npad%top - npad%bot
  fd_grid%x%n = fd_grid%x%n - npad%left - npad%right
  !!================================
  write(0,*) "pass1"

   
  !! write out the RTM image resulting from all shot imfomation
  write(0,*) "writing out the RTM image"
  call srite("out",refl(:,:),4*fd_grid%z%n*fd_grid%x%n)
  ! call sep_write(refl(:,:))
  write(0,*) "passed"



  ! deallocate all
  deallocate(source, vp, refl,snap,sousnap,recsnap,receiver)
  write(0,*) "pass2"

!  deallocate(receiver)
  write(0,*) "deallocation completed"

  call exit(0)

end program crtm2d_program

!!!! end program
