! Module contains a subrourine that computes wavefield propagation: inputs include source wavelet.
! 2nd order approx in time and 'selected' order approx in space

module model2d_mod

  use sep
  use type_mod
  use wave2d_mod
  implicit none

  ! approximation: 2nd order in time and ??th order in space (nfd_time=1)
  integer   :: order_time=2
  integer   :: nfd_time=1
  real      :: pi=3.1415926536

contains


 !!! Subroutine to do rtm calculation of source wavefield from ONE shot (forward in time)
  subroutine model2d(  vp,data,   &  !bcinput: [1] source wavelet BC, [2] receiver wavefield BC
                     source,isz,isx,  &
                     sou_grid,rec_grid,fd_grid, &
                     npad,abstop,  &   ! absorbing boundaries
                     get_snap, &
                     snap,  &
                     jsnap,nt_snap, &
                     smoothvel,laplac_order,nfd_space,verb )

    ! laplac_order: 2, 4 or 10
    ! fd_grid must have already been padded

    ! define input/output parameters
    logical             :: abstop,verb,get_snap,smoothvel
    integer             :: isz,isx,laplac_order ! source and receiver locations on FD grid (x,z)
    integer             :: jsnap,nt_snap,nfd_space
    type(grid)          :: fd_grid,sou_grid,rec_grid  ! FD stencils already padded & source & receiver grids
    type(npad_values)   :: npad       ! # of padding stencils already applied in model grid

    real, intent(in)    :: vp(fd_grid%z%n-npad%top-npad%bot, fd_grid%x%n-npad%left-npad%right)  ! acoustic velocity without padding layers
    real, intent(in)    :: source(sou_grid%t%n)         ! source wavelet (boundary)
    real, intent(inout) :: data(rec_grid%z%n, rec_grid%x%n, rec_grid%t%n)  !! recorded data

    real, intent(inout) :: snap(fd_grid%z%n-npad%top-npad%bot, fd_grid%x%n-npad%left-npad%right, nt_snap)

    ! local parameters    
    real     :: p(1-nfd_space:fd_grid%z%n+nfd_space,1-nfd_space:fd_grid%x%n+nfd_space,3) ! source wavefield stencils
    real     :: rr, dt,dx,dz,ot,ox,oz, dgx,dgz,ogx,ogz, dsx,dsz,osx,osz, delp
    integer  :: iz,ix,it,igz,igx,i,j,k, nz,nx,nt, ngz,ngx, nsz,nsx, it_snap, sou_iz,sou_ix
    real     :: vel(fd_grid%z%n,fd_grid%x%n)

    !----------------------------------------------
    ! checking grid info
    if((sou_grid%t%d.ne.fd_grid%t%d).and.(rec_grid%t%d.ne.fd_grid%t%d)) then
       call seperr("Source, receiver and FD grid intervals (d's) along time axis need to be compatible")
    end if
    if((sou_grid%t%o.ne.fd_grid%t%o).and.(rec_grid%t%o.ne.fd_grid%t%o)) then
       call seperr("Source, receiver and FD grid origins (o's) along time axis need to be compatible (=zero)")
    end if

    if(.not.abstop .and. npad%top.ne.0)then
       call seperr("When top boundary is reflective type (abstop=.false.), no padding at the top must be applied (npad%top=0).")
    end if
    !----------------------------------------------
    if(verb) write(0,*) "grid checking passed"
    !----------------------------------------------
    !! set up initial values and the boundary conditions
    ! symbols for FD grids with padding regions
    dz=fd_grid%z%d  !
    dx=fd_grid%x%d  ! just for easier to write the program 
    dt=fd_grid%t%d  !
    nz=fd_grid%z%n  !
    nx=fd_grid%x%n  ! just for easier to write the program 
    nt=fd_grid%t%n  !
    oz=fd_grid%z%o  !
    ox=fd_grid%x%o  ! just for easier to write the program 
    ot=fd_grid%t%o  !
    ! symbols for receiver grids
    dgz=rec_grid%z%d  !
    dgx=rec_grid%x%d  ! just for easier to write the program 
    ngz=rec_grid%z%n  !
    ngx=rec_grid%x%n  ! just for easier to write the program 
    ogz=rec_grid%z%o  !
    ogx=rec_grid%x%o  ! just for easier to write the program 
    ! symbols for source grids
    dsz=sou_grid%z%d  !
    dsx=sou_grid%x%d  ! just for easier to write the program 
    nsz=sou_grid%z%n  !
    nsx=sou_grid%x%n  ! just for easier to write the program 
    osz=sou_grid%z%o  !
    osx=sou_grid%x%o  ! just for easier to write the program 

    ! compute the source location on the FD grids (z,x)
    sou_iz = npad%top + 1.5 + ( (osz+(isz-1)*dsz) - oz )/dz
    sou_ix = npad%left + 1.5 + ( (osx+(isx-1)*dsx) - ox )/dx


    ! initialize wavefields 
    ! But, we are not initializing refl(:,:), since it is an accumulated result from many shots.
    if(verb) write(0,*) "Initializing data....."
    p(:,:,:)=0. 
    snap(:,:,:)=0.
    data(:,:,:)=0.
    if(verb) write(0,*) "passed"

    !!!===============================
    !! Adjust the velocity model
    !!!===============================
    ! calculate velocity to be compatible with the FD scheme
    vel(:,:)=0.
    do iz=1,nz-npad%top-npad%bot
       do ix=1,nx-npad%left-npad%right
          vel(iz+npad%top,ix+npad%left) = vp(iz,ix)
       end do
    end do
    ! extrapolate the velocity in the padding regions and the FD boundaries
    do iz=1,npad%top;         vel(iz,:) = vel(1+npad%top,:);    end do  ! top bound
    do iz=nz-npad%bot+1,nz;   vel(iz,:) = vel(nz-npad%bot,:);   end do  ! bottom bound
    do ix=1,npad%left;        vel(:,ix) = vel(:,1+npad%left);   end do  ! left bound 
    do ix=nx-npad%right+1,nx; vel(:,ix) = vel(:,nx-npad%right); end do  ! right bound

    if(smoothvel) then  ! smoothen velocity for laplacian operator if needed
       call smooth2d(vel,nz,nx,nfd_space)
    end if

    !!================================================
    !! begin extrapolation scheme for source wavefield
    !!================================================
    write(0,*) "Extrapolating source wavefields (forward in time)...."

    do it=1,fd_grid%t%n   ! time forward loop

       !!========================================================================
       !! Add boundary conditions
          ! Add the source wavelet at the source location: The source is 
          ! introduced over a small region (not at just one point to prevent 
          ! energy at nyquist wavelength from entering the calculation)
          if(it.le.sou_grid%t%n) then
             if(.true.) then
                do iz=sou_iz-2,sou_iz+2
                   do ix=sou_ix-2,sou_ix+2
                      rr=sqrt(float((iz-sou_iz)**2+(ix-sou_ix)**2))    ! sou_ix is source location on FD grids
                      p(iz,ix,2) = p(iz,ix,2) - source(it)*.5*(1.+cos(rr/3*pi)) ! update and apply the BC here
                   end do
                end do
             else
                p(sou_iz,sou_ix,2) = p(sou_iz,sou_ix,2) - source(it)
             end if
          end if

       !!========================================================================
         
       !----------------------------------------------
       ! If the reflective top bound is assigned, apply anti-symetric reflection at the top free surface
       !if(.not.abstop) then             
       !   do i=1,nx
       !      p(0,i,2) = 0.
       !      p(-1,i,2) = -p(1,i,2)
       !      ! This p(j,i,2) is exactly what will be used in the coming laplacian operation
       !   end do
       !end if
       !----------------------------------------------
       
       ! Compute the laplacian of the wavefield at the current time p(:,:,2) and save to p(:,:,3)
       call laplac2d(p(:,:,2),p(:,:,3),dz,dx,nz,nx,nfd_space,laplac_order)
!       write(0,*)"pass1"

       do iz=1,nz        ! vertical loop
          do ix=1,nx     ! horizontal loop             
             ! extrapolation step second-order in time
             p(iz,ix,3) = dt*dt*vel(iz,ix)*vel(iz,ix)*p(iz,ix,3)
             p(iz,ix,3) = p(iz,ix,3) + 2*p(iz,ix,2) - p(iz,ix,1) ! extrapolate wave in time
          end do
       end do
       do iz=1,nz        ! vertical loop
          do ix=1,nx     ! horizontal loop            
             !!========================================================================
             !! update wavefield within the data grids ready for next time step
             p(iz,ix,1) = p(iz,ix,2)
             p(iz,ix,2) = p(iz,ix,3)
             !!========================================================================
          end do   ! horizontal x loop end
       end do   ! vertical z loop end
!       write(0,*) "pass2"

       !! apply absorbing tapering and/or absorbing FD boundary
       call taper(vel,p,dt,dz,dx,nz,nx,npad,abstop,nfd_space)
!       write(0,*) "pass3"

       !! get snapshot of source wavefield
       if(get_snap) then
          if(mod(it,jsnap).eq.1) then
             it_snap = 1 + (it-1)/jsnap
             if(it_snap.le.nt_snap) then
                do iz=1,fd_grid%z%n-npad%top-npad%bot
                   do ix=1,fd_grid%x%n-npad%left-npad%right
                      snap(iz,ix,it_snap) = p(iz+npad%top,ix+npad%left,1)     ! choose snapshots
                      ! snap(:,:,it_snap) = p(1+npad%top:nz-npad%bot,1+npad%left:nx-npad%right,1)     ! choose snapshots
                   end do
                end do
             end if
          end if
       end if
!       write(0,*) "pass4"

       !! get recorded data 
       do igz=1,rec_grid%z%n
          do igx=1,rec_grid%x%n
             ! compute the location of receivers on the FD grids
             iz = npad%top + 1.5 + ( (ogz+(igz-1)*dgz) - oz )/dz  
             ix = npad%left + 1.5 + ( (ogx+(igx-1)*dgx) - ox )/dx
             data(igz,igx,it) = p(iz,ix,1)
          end do
       end do
       
       if(verb .and. mod(it,jsnap).eq.1) write(0,*) it, sum(p(:,:,1)), p(sou_iz,sou_ix,1)
    
    end do   !!! time loop end
    !!================================================
    !! end extrapolation scheme for source wavefield
    !!================================================


  end subroutine model2d


end module model2d_mod
