!	   		        model2d_program
!
!            by time stepping using an explicit finite-difference 
!	     solution to the acoustic wave equation accurate to 2nd order 
!            in time and selected order in x and z.
!
!		Author: Kittinat Taweesintananon
!
! Example:
!
! <vel.H model2d_program.x source=wvlet.H par=parfile  >data.H  snap=snap.H
!
!
!	Necessary input files:
!
!		vel.H is the Header for the velocity model, stdin: dimension /z x/
!
!               wvlet.H is an auxilary file containing source wavelet file as the boundary: dimension /t/
!
!		parfile  contains the geometry parameters
!			 for the synthetic seismic survey
!
!	Output files:
!
!		data.H    Header file for the output seismograms (shot records): dimension /gz gx t sx sz/
!
!		snap.H	  Header for the (optional) output snapshots of the cross-correlation related to the first shot: dimension /z x t/
!
!
! **Necessary parameters****************************************************
! ****** most quantitites like distances refer to grid points **************
!
!	From wvlet.H:	n1 	# of gridpoints in t 
!			d1	t gridpoint spacing (becomes dt)
!
!	From vel.H:	n1 	# of gridpoints in z (becomes nz)
!			n2	# of gridpoints in x (becomes nx)
!			d1	z gridpoint spacing (becomes dz)
!			d2	x gridpoint spacing (becomes dx)
!
!From par:	
!
! rollalong=.false.	        #roll the receiver spread along with shots
!				# [1, .true.] marine acquisition(offshore): one vessel and towed streamers behind 
!                       	# the hydrophone locations (all location parameters) are with respect to the vessel (source locations)
!                       	# [0, .false.=default] land or OBC acquisition(onshore): 
!				# the geophone locations are with respect to the earth frame (the velocity model);
!
! abstop=.false.        # [1, .true.] apply absorbing top boundary
!                       # [0, .false.=default] apply full reflective top boundary
! npadtop,npadbot,npadleft,npadright  # the number of padding expansion (gaussian tapering) for top,bottome,left,right boundaries
! percent_taper          # [98.] percent of absorbed amplitude at the end of pad (using gaussian tapering)
!
! nt,dt	                Time length/interval of the shot records 
!
! get_snap         # [0,.false.=default] for printing out the snapshot of propagating wavefield
! jsnap            # snapshot time grid interval[50]
!
! smoothvel=0   # use smoothened velocity when using FD scheme: [1,.true] or [0,.false.=default]
! laplac_order=4  # the order of laplacian computing: selected from 2 or 4(default) or 10 
!
!! source locations			
! nsz,dsz,osz  	# source locations along z-axis
! nsx,dsx,osx  	# source locations along x-axis
!
!! receiver locations			
! ngz,dgz,ogz  	# receiver locations along z-axis
! ngx,dgx,ogx  	# receiver locations along x-axis

program model2d_program

  use sep
  use type_mod
  use model2d_mod

  implicit none

  !!====================================================================
  !! define parameters
  logical             :: verb, smoothvel
  logical             :: abstop, rollalong, get_snap
  type(grid)          :: fd_grid,sou_grid,rec_grid  ! FD stencils already padded & source & receiver grids
  type(npad_values)   :: npad       ! # of padding stencils already applied in model grid

  real                :: dt_snap, dt,dx,dz,ot,ox,oz, dgx,dgz,ogx,ogz, dsx,dsz,osx,osz
  integer             :: nt_snap, nz,nx,nt, ngz,ngx, nsz,nsx,laplac_order,nfd_space

  integer             :: it_snap, i,j,k, iz,ix,it,igz,igx,isz,isx, sou_iz,sou_ix, rec_iz,rec_ix, jsnap      ! indices

  real,dimension(:),allocatable      :: source
  real,dimension(:,:),allocatable    :: vp
  real,dimension(:,:,:),allocatable  :: snap
  real,dimension(:,:,:,:,:),allocatable  :: data


  !!--------------------
  character(len=128)  :: source_,snap_   ! Define the tagnames for the auxilary files 
  source_ ="source"; snap_ ="snap";

  !!====================================================================
  !!======================input/output stuff============================
  call sep_init()

  !!====================================================================
  !! read the parameters from the command lines
  !!====================================================================
  !!====================================================================  
  !!---------grid setting------------
  write(0,*) "Reading axes....."
  !! Source grids info
    call from_param("nsz",sou_grid%z%n,1)
    call from_param("dsz",sou_grid%z%d,1.)
    call from_param("osz",sou_grid%z%o,0.)

    call from_param("nsx",sou_grid%x%n,1)
    call from_param("dsx",sou_grid%x%d,1.)
    call from_param("osx",sou_grid%x%o,0.) 
    
    call from_aux(source_,"n1",sou_grid%t%n)   ! # of t gridpoints for the source signal
    call from_aux(source_,"d1",sou_grid%t%d)
    call from_aux(source_,"o1",sou_grid%t%o,0.)

  !! Receiver grids info
    call from_param("ngz",rec_grid%z%n,1)
    call from_param("dgz",rec_grid%z%d,1.)
    call from_param("ogz",rec_grid%z%o,0.)

    call from_param("ngx",rec_grid%x%n,1)
    call from_param("dgx",rec_grid%x%d,1.)
    call from_param("ogx",rec_grid%x%o,0.)

    call from_param("nt",rec_grid%t%n,sou_grid%t%n)
    call from_param("dt",rec_grid%t%d,sou_grid%t%d)
    rec_grid%t%o = sou_grid%t%o

    if(rec_grid%t%d .ne. sou_grid%t%d) then
       write(0,*) "source wavelet dt, and recording dt are", sou_grid%t%d, rec_grid%t%d
       call seperr("receiver dt must equal source dt")
    end if

  !! FD grids info
  ! get parameters from the input history file [velocity parameters, v(z,x)]
    call from_history("n1",fd_grid%z%n)   
    call from_history("d1",fd_grid%z%d)   ! from velocity input v(z,x)
    call from_history("o1",fd_grid%z%o,0.)   ! from velocity input v(z,x)

    call from_history("n2",fd_grid%x%n)   ! from velocity input v(z,x)
    call from_history("d2",fd_grid%x%d)   ! from velocity input v(z,x)    
    call from_history("o2",fd_grid%x%o,0.)   ! from velocity input v(z,x)

    fd_grid%t%n = rec_grid%t%n
    fd_grid%t%d = rec_grid%t%d
    fd_grid%t%o = rec_grid%t%o

  !!---------end grid reading------------

  !!====================================================================
  !!====================================================================
  !!---------parameter reading------------
    write(0,*) "Reading parameters....."

    call from_param("verb",verb,.false.)   ! verbal description
    call from_param("rollalong",rollalong,.false.)  ! roll the receiver spread along with shots: 

  !! read parameters for absorbing/reflective boundaries 
    call from_param("abstop",abstop,.false.)  ! type of top boundary: [.true.] for absorbing bound; [.false.] for reflective bound.
    
    call from_param("npadtop",npad%top,0)
    if(.not.abstop)then
       npad%top=0.
    end if
    call from_param("npadbot",npad%bot,0)    ! padding expansion
    call from_param("npadleft",npad%left,0); call from_param("npadright",npad%right,0);    ! padding expansion
    call from_param("percent_taper",npad%taper,98.)
  
  !! ================================
  !! read parameters for wavefield snapshot
    call from_param("get_snap",get_snap,.false.)   ! snapshot of cross-correlation
    call from_param("jsnap",jsnap,50)
    nt_snap = (fd_grid%t%n-1)/jsnap + 1
    dt_snap = fd_grid%t%d*jsnap

  !! parameters for FD computation
    ! use smoothened velocity during extrapolation process or not
    call from_param("smoothvel",smoothvel,.false.)
    ! order of approx in space for FD computing (4 or 10)
    call from_param("laplac_order",laplac_order,4)
    ! set up the laplacian order
    if(laplac_order.eq.4) then 
       nfd_space=2
    else if(laplac_order.eq.10) then
       nfd_space=5
    else if(laplac_order.eq.2)then
       nfd_space=1
    else
       write(0,*) "laplac_order:",laplac_order
       call seperr("WARNING: the laplacian order must be 2, 4 or 10")
    end if


  !!====================================================================
  !!====================================================================
  !! write axes parameters to the history files
    write(0,*) "Writing axes....."
    !! output file (data.H => shot record file /gz gx t sx sz/)
    call to_history("n1",rec_grid%z%n); call to_history("d1",rec_grid%z%d); call to_history("o1",rec_grid%z%o);  ! receiver z
    call to_history("n2",rec_grid%x%n); call to_history("d2",rec_grid%x%d); call to_history("o2",rec_grid%x%o);  ! receiver x
    call to_history("n3",rec_grid%t%n); call to_history("d3",rec_grid%t%d); call to_history("o3",rec_grid%t%o);  ! recording time t
    call to_history("n4",sou_grid%x%n); call to_history("d4",sou_grid%x%d); call to_history("o4",sou_grid%x%o);  ! source x
    call to_history("n5",sou_grid%z%n); call to_history("d5",sou_grid%z%d); call to_history("o5",sou_grid%z%o);  ! source z

    if(rollalong) then ! roll receiver spread along with shots parallel to xy-plane (marine acquisition)
       call to_history("label1", 'rec_z(m)'); call to_history("label2", 'offset_x(m)');
       call to_history("label3", 'time(s)'); call to_history("label4", 'sou_x(m)');
       call to_history("label5", 'sou_z(m)');
    else
       call to_history("label1", 'rec_z(m)'); call to_history("label2", 'rec_x(m)');
       call to_history("label3", 'time(s)'); call to_history("label4", 'sou_x(m)');
       call to_history("label5", 'sou_z(m)');      
    end if



    !! create information of wavefield snapshot auxilary file (snap.H => /z x t/)
    if(get_snap) then
       ! receiver z
       call auxputch("n1",'i', fd_grid%z%n, snap_); call auxputch("d1",'f', fd_grid%z%d,snap_);  
       call auxputch("o1",'f', fd_grid%z%o, snap_); call auxputch("label1", 's', 'z', snap_);
       ! receiver x
       call auxputch("n2",'i', fd_grid%x%n,snap_); call auxputch("d2",'f', fd_grid%x%d, snap_);  
       call auxputch("o2",'f', fd_grid%x%o,snap_); call auxputch("label2", 's', 'x', snap_);
       ! time
       call auxputch("n3",'i', nt_snap,snap_); call auxputch("d3",'f',dt_snap,snap_);  
       call auxputch("o3",'f', fd_grid%t%o,snap_); call auxputch("label3", 's', 'time', snap_);
    end if

  !!====================================================================
  !!====================================================================
  !! write out the input parameters
    write(0,*) '***********************************'
    write(0,*) '***********************************'
    write(0,*) 'rollalong',rollalong
    write(0,*) '***********************************'
    write(0,*) 'ngz,dgz,ogz', rec_grid%z%n, rec_grid%z%d, rec_grid%z%o
    write(0,*) 'ngx,dgx,ogx', rec_grid%x%n, rec_grid%x%d, rec_grid%x%o
    write(0,*) 'nt, dt, ot', rec_grid%t%n, rec_grid%t%d, rec_grid%t%o
    write(0,*) 'nsx,dsx,osx', sou_grid%x%n, sou_grid%x%d, sou_grid%x%o
    write(0,*) 'nsz,dsz,osz', sou_grid%z%n, sou_grid%z%d, sou_grid%z%o
    write(0,*) '***********************************'
    write(0,*) '************* FD grid *************'
    write(0,*) 'nz,dz,oz', fd_grid%z%n, fd_grid%z%d, fd_grid%z%o
    write(0,*) 'nx,dx,ox', fd_grid%x%n, fd_grid%x%d, fd_grid%x%o
    write(0,*) '***********************************'
    write(0,*) 'abstop', abstop
    write(0,*) 'npadtop,npadbot,npadleft,npadright', npad%top,npad%bot,npad%left,npad%right
    write(0,*) 'percent_taper', npad%taper
    write(0,*) 'get_snap', get_snap
    write(0,*) 'jsnap, nt_snap, dt_snap', jsnap,nt_snap,dt_snap
    write(0,*) 'smoothvel,laplac_order,nfd_space',smoothvel,laplac_order,nfd_space
    write(0,*) '***********************************'
  call sep_close()

  !!====================================================================
  !!=========================end input/output===========================

  !!================================
  ! allocate space for the velocity model, source wavelet, data, wavefields
  allocate(source(sou_grid%t%n))
  allocate(vp(fd_grid%z%n, fd_grid%x%n))    ! /nz, nx/ no pad
  allocate(snap(fd_grid%z%n, fd_grid%x%n, nt_snap))  ! snap /nz, nx, nt_snap/
  allocate(data(rec_grid%z%n, rec_grid%x%n, rec_grid%t%n, sou_grid%x%n, sou_grid%z%n))   ! shot record /ngz, ngx, nt, nsx, nsz/
  write(0,*) "allocation completed"
  !!================================

  ! Read velocity model
  call sep_read(vp(1:fd_grid%z%n,1:fd_grid%x%n)) ! original FD grid size
  write(0,*) "the velocity model is read"

  ! Read source wavelet
  call sep_read(source(1:sou_grid%t%n),source_)
  write(0,*) "source wavelet is read"

  !!================================
  !! Expand FD domain by padding layers (only the number of gridpoints)
   fd_grid%z%n = fd_grid%z%n + npad%top + npad%bot
   fd_grid%x%n = fd_grid%x%n + npad%left + npad%right
  !!================================

    ! symbols for FD grids
    dz=fd_grid%z%d  !
    dx=fd_grid%x%d  ! just for easier to write the program 
    dt=fd_grid%t%d  !
    nz=fd_grid%z%n  !
    nx=fd_grid%x%n  ! just for easier to write the program 
    nt=fd_grid%t%n  !
    oz=fd_grid%z%o  !
    ox=fd_grid%x%o  ! just for easier to write the program 
    ot=fd_grid%t%o  !
    ! symbols for receiver grids
    dgz=rec_grid%z%d  !
    dgx=rec_grid%x%d  ! just for easier to write the program 
    ngz=rec_grid%z%n  !
    ngx=rec_grid%x%n  ! just for easier to write the program 
    ogz=rec_grid%z%o  !
    ogx=rec_grid%x%o  ! just for easier to write the program 

    ! symbols for source grids
    dsz=sou_grid%z%d  !
    dsx=sou_grid%x%d  ! just for easier to write the program 
    nsz=sou_grid%z%n  !
    nsx=sou_grid%x%n  ! just for easier to write the program 
    osz=sou_grid%z%o  !
    osx=sou_grid%x%o  ! just for easier to write the program 

    !------------------------------------

  ! initialize to zero
  data=0.;snap=0.;

  ! generate the shot records for all shots
  do isz=1,nsz     ! loop over shots
     do isx=1,nsx

        if(rollalong) rec_grid%x%o = ogx + osx + (isx-1)*dsx 

          write(0,*) "***********************************************"
          write(0,*) "**Modeling the shot number isz, isx ", isz, isx
          write(0,*) "**Modeling the shot sz, sx ", osz+(isz-1)*dsz, osx+(isx-1)*dsx
          write(0,*) "***********************************************"

          write(0,*) "calling model2d_mod for shot isz isx:",isz,isx
          call model2d(  vp,data(:,:,:,isx,isz),   &
                     source,isz,isx,  &
                     sou_grid,rec_grid,fd_grid, &
                     npad,abstop,  &   ! absorbing boundaries
                     get_snap.and.(isz*isx.eq.1), &
                     snap,  &
                     jsnap,nt_snap, &
                     smoothvel,laplac_order,nfd_space,verb )

          !! write out the snapshot file of the wavefield from the 1st shot
          if(get_snap .and. (isz*isx.eq.1)) then
             write(0,*) "writing out the wavefield snapshot from the 1st shot"
             !call srite(snap_,snap,4*fd_grid%z%n*fd_grid%x%n*nt_snap)
             call sep_write(snap,snap_)  
          end if
          !!---

          ! writing the recorded data
          write(0,*) "Writing shot record ...."
          !call data_io(data(:,:,:,isx,isz),rec_grid,sou_grid,isx)
          call srite("out",data(:,:,:,isx,isz),4*rec_grid%z%n*rec_grid%x%n*rec_grid%t%n)

     end do
   end do     ! end loop over shots

   write(0,*) "data for all shots are written"
   !!================================

   ! deallocate all
   deallocate(vp,source,snap,data)
   write(0,*) "deallocation completed"

   call exit(0)

end program model2d_program

!!!! end program

!!====
subroutine data_io(data,rec_grid,sou_grid,isx)
  use type_mod
  use sep
  implicit none

  type(grid)         :: rec_grid,sou_grid
  real,intent(inout) :: data(rec_grid%z%n,rec_grid%x%n,rec_grid%t%n)
  integer            :: isx

  integer,dimension(4)  :: n_cube,n_wind,f_wind,j_wind
  integer  :: n_dim_cube, err

  n_dim_cube = 4
  n_cube = (/rec_grid%z%n,rec_grid%x%n,rec_grid%t%n,sou_grid%x%n/)
  n_wind = (/rec_grid%z%n,rec_grid%x%n,rec_grid%t%n,1/)
  f_wind = (/0,0,0,isx-1/)
  j_wind = (/1,1,1,1/)

  write(0,*) "writing shot:",isx,"of",sou_grid%x%n,"...."
  ! srite_window(tag_history,n_dim_cube,n_cube,n_wind,f_wind,j_wind,esize,values)
  call srite_window("out",n_dim_cube,n_cube,n_wind,f_wind,j_wind,4,data)
  !if(err.ne.0) call seperr("problem with writing data")

end subroutine data_io
