! 
!	   		       rtmformation2d_program.x
!
!            This program will do cross-correlation of two input source- and receiver wavefield files (snapshots). 
!            The result is the snapshots of the accumulation process along time axis from tmax to t0
!
!		           Author: Kittinat Taweesintananon
!
! Example:
!
! rtmformation2d_program.x soufile=fullsouwvfldsnap.H recfile=fullrecwvfldsnap.H > imagesnap.H
!
!
!	Necessary input files:
!
!		soufile=,soufile=    # input full wavefield snapshots /z,x,t/. Both must have the same size.
!
!	Output files:
!
!		imagesnap.H    # Header file for the output snapshots of the cross-correlation accumulated from tmax to t0: dimension /z,x,t/
!
!       From par:
!               verb      # [0,n] verbal expression
!
!================================================================

program rtmformation2d_program

  use type_mod
  use sep
  implicit none

  !!====================================================================
  !! define parameters
  logical             :: verb
  type(grid)          :: grid1,grid2  ! source- & receiver wavefield grids
  integer             :: iz,ix,it

  real,dimension(:,:,:),allocatable  :: sousnap, recsnap

  !!--------------------
  character(len=128)  :: soufile_,recfile_,label1,label2,label3   ! Define the tagnames for input auxilary files 
  soufile_="soufile"; recfile_="recfile"

  !!====================================================================
  !!======================input/output stuff============================
  call sep_init()

  !!====================================================================
  !! read the parameters from the command lines
  call from_param("verb",verb,.false.)
  
  !!--------------
  !! read axes from soufile /z,x,t/
  ! z axis
  call from_aux(soufile_,"n1",grid1%z%n)
  call from_aux(soufile_,"d1",grid1%z%d)
  call from_aux(soufile_,"o1",grid1%z%o)
  call from_aux(soufile_,"label1",label1)
  ! x axis
  call from_aux(soufile_,"n2",grid1%x%n)
  call from_aux(soufile_,"d2",grid1%x%d)
  call from_aux(soufile_,"o2",grid1%x%o)
  call from_aux(soufile_,"label2",label2)
  ! t axis
  call from_aux(soufile_,"n3",grid1%t%n)
  call from_aux(soufile_,"d3",grid1%t%d)
  call from_aux(soufile_,"o3",grid1%t%o)
  call from_aux(soufile_,"label3",label3)

  !!--------------
  !! read axes from recfile /z,x,t/
  ! z axis
  call from_aux(recfile_,"n1",grid2%z%n)
  call from_aux(recfile_,"d1",grid2%z%d)
  call from_aux(recfile_,"o1",grid2%z%o)
  ! x axis
  call from_aux(recfile_,"n2",grid2%x%n)
  call from_aux(recfile_,"d2",grid2%x%d)
  call from_aux(recfile_,"o2",grid2%x%o)
  ! t axis
  call from_aux(recfile_,"n3",grid2%t%n)
  call from_aux(recfile_,"d3",grid2%t%d)
  call from_aux(recfile_,"o3",grid2%t%o)

  !! check if soufile and recfile are the same
  if((grid1%z%n.ne.grid2%z%n).or.(grid1%z%d.ne.grid2%z%d).or.(grid1%z%o.ne.grid2%z%o))then
     call seperr("soufile and recfile must have the same size.")
  else if((grid1%x%n.ne.grid2%x%n).or.(grid1%x%d.ne.grid2%x%d).or.(grid1%x%o.ne.grid2%x%o))then
     call seperr("soufile and recfile must have the same size.")
  else if((grid1%t%n.ne.grid2%t%n).or.(grid1%t%d.ne.grid2%t%d).or.(grid1%t%o.ne.grid2%t%o))then
     call seperr("soufile and recfile must have the same size.")
  end if


  !! write the axes of output file which should be the same as soufile /z,x,t/
  ! z axis
  call to_history("n1",grid1%z%n)
  call to_history("d1",grid1%z%d)
  call to_history("o1",grid1%z%o)
  call to_history("label1",label1)
  ! x axis
  call to_history("n2",grid1%x%n)
  call to_history("d2",grid1%x%d)
  call to_history("o2",grid1%x%o)
  call to_history("label2",label2)
  ! t axis
  call to_history("n3",grid1%t%n)
  call to_history("d3",grid1%t%d)
  call to_history("o3",grid1%t%o)
  call to_history("label3",label3)

  call sep_close()

  !!=========================end input/output===========================
  !!====================================================================
  ! allocate variables
  allocate(sousnap(grid1%z%n, grid1%x%n, grid1%t%n)) !/nz, nx, nt_snap/
  allocate(recsnap(grid2%z%n, grid2%x%n, grid2%t%n))
  if(verb) write(0,*) "allocation complete"

  !! read data
  call sep_read(sousnap,soufile_)
  call sep_read(recsnap,recfile_)
  if(verb) write(0,*) "data was read"

  ! compute imagesnap at t=tmax saving it into the sousnap variable
  sousnap(iz,ix,grid1%t%n) = sousnap(iz,ix,grid1%t%n)*recsnap(iz,ix,grid1%t%n)

  ! compute accumulated RTM snapshot images
  do it=grid1%t%n-1,1,-1
     do iz=1,grid1%z%n
        do ix=1,grid1%x%n
           sousnap(iz,ix,it) = sousnap(iz,ix,it+1) + sousnap(iz,ix,it)*recsnap(iz,ix,it)
        end do
     end do
  end do
  !! now, sousnap is the imagesnap

  !! write the snapshots of image formation to disk
  call sep_write(sousnap)
  if(verb) write(0,*) "data has been written"

  ! deallocate variables
  deallocate(sousnap,recsnap)
  if(verb) write(0,*) "allocation complete"

  call exit(0)

end program rtmformation2d_program
