! Module contains subrourines that are helpful for extrapolating wavefield
! - laplacian2d
! - taper (gaussian tapering)

module wave2d_mod

  use sep
  use type_mod
  implicit none

contains

!!################################################
  subroutine laplac2d(pin,pout,dz,dx,nz,nx,nfd_space,order)
    integer           :: nz,nx,nfd_space,order  ! order: 2,4,10
    real              :: dz,dx
    real,intent(in):: pin(1-nfd_space:nz+nfd_space,1-nfd_space:nx+nfd_space)
    real,intent(out):: pout(1-nfd_space:nz+nfd_space,1-nfd_space:nx+nfd_space)

    ! local parameters
    integer  :: iz,ix
    real     :: a0,a1x,a2x,a3x,a4x,a5x
    real     :: a1z,a2z,a3z,a4z,a5z

    if(order.ne.2 .and. order.ne.4 .and. order.ne.10) call seperr("*** I cannot compute Laplacian: wrong order input")

    ! 2nd order
    if(order.eq.2) then
       if(nfd_space.ne.1) call seperr("nfd_space is invalid")
       ! set up coefficients for 2th-order laplacian approx.
       a1x = 1./(dx*dx)
       a1z = 1./(dz*dz)
       a0 = -2*(a1x+a1z)  

       do iz=1,nz
          do ix=1,nx
             pout(iz,ix) = a0*pin(iz,ix) +                       &
                          a1x*(pin(iz,ix+1) + pin(iz,ix-1)) +   &
                          a1z*(pin(iz+1,ix) + pin(iz-1,ix))
          end do
       end do
    end if

    ! 4th order laplacian calculation
    if(order.eq.4) then
       if(nfd_space.ne.2) call seperr("nfd_space is invalid")
       ! set up coefficients for 4th-order laplacian approx.
       a1x = (4./3)/(dx*dx)
       a1z = (4./3)/(dz*dz)
       a2x = (-1./12)/(dx*dx)
       a2z = (-1./12)/(dz*dz)
       a0 = -2*(a1x+a2x+a1z+a2z)  ! that is, a0 = -5*(1/dx**2 + 1/dz**2).

       do iz=1,nz
          do ix=1,nx
             pout(iz,ix) = a0*pin(iz,ix) +                       &
                          a1x*(pin(iz,ix+1) + pin(iz,ix-1)) +   &
                          a2x*(pin(iz,ix+2) + pin(iz,ix-2)) +   &
                          a1z*(pin(iz+1,ix) + pin(iz-1,ix)) +   &
                          a2z*(pin(iz+2,ix) + pin(iz-2,ix))
          end do
       end do
    end if

    ! 10th order laplacian calculation
    if(order.eq.10) then
       if(nfd_space.ne.5) call seperr("nfd_space is invalid")
       ! set up coefficients for 10th-order laplacian approx.
       ! They are from [Dablain(1986) Geophysics. vol.51]
       a1x = (3.3333333333)/(2*dx*dx)
       a1z = (3.3333333333)/(2*dz*dz)
       
       a2x = (-0.4761904762)/(2*dx*dx)
       a2z = (-0.4761904762)/(2*dz*dz)
       
       a3x = (0.0793650794)/(2*dx*dx)
       a3z = (0.0793650794)/(2*dz*dz)

       a4x = (-0.0099206349)/(2*dx*dx)
       a4z = (-0.0099206349)/(2*dz*dz)
       
       a5x = (0.0006349206)/(2*dx*dx)
       a5z = (0.0006349206)/(2*dx*dx)

       a0 = -5.8544444444/(2*dx*dx) - 5.8544444444/(2*dz*dz)

       do iz=1,nz
          do ix=1,nx
             pout(iz,ix) = a0*pin(iz,ix) +                       &
                          a1x*(pin(iz,ix+1) + pin(iz,ix-1)) +   &
                          a2x*(pin(iz,ix+2) + pin(iz,ix-2)) +   &
                          a3x*(pin(iz,ix+3) + pin(iz,ix-3)) +   &
                          a4x*(pin(iz,ix+4) + pin(iz,ix-4)) +   &
                          a5x*(pin(iz,ix+5) + pin(iz,ix-5)) +   &
                          a1z*(pin(iz+1,ix) + pin(iz-1,ix)) +   &
                          a2z*(pin(iz+2,ix) + pin(iz-2,ix)) +   &
                          a3z*(pin(iz+3,ix) + pin(iz-3,ix)) +   &
                          a4z*(pin(iz+4,ix) + pin(iz-4,ix)) +   &
                          a5z*(pin(iz+5,ix) + pin(iz-5,ix))
          end do
       end do
    end if

  end subroutine laplac2d


  !! subroutine to apply tapering and absorbing FD boundaries
  subroutine taper(vp,p,dt,dz,dx,nz,nx,npad,abstop,nfd_space)
    !! This subroutine should be applied after updating wavefield in each time loop
    !! That is, here p(:,:,2) is the current wavefield in the next time loop,
    !! and p(:,:,1) is the current wavefield in the current time loop.

    ! input/output parameters
    integer           :: nz,nx,nfd_space
    real              :: dt,dz,dx
    type(npad_values) :: npad
    logical           :: abstop
    real,intent(in):: vp(1:nz,1:nx)
    real,intent(inout):: p(1-nfd_space:nz+nfd_space,1-nfd_space:nx+nfd_space,3)

    ! local parameters
    real     :: delp
    integer  :: i,j

             ! Apply gaussian taper to the wavefield in the padding areas (boundary layer) 
             ! the amplitude goes down to 95% at the end with respect to the begin of the padding areas

             ! 
             ! update the right-side layer
          if(npad%right.ne.0) then
             delp = -log(npad%taper*0.01)/real(npad%right)**2  ! -ln(%)=-0.0513/xmax**2
             do j=1,nz
                do i=nx-npad%right+1,nx
                   p(j,i,1) = p(j,i,1)*exp(-(i - (nx-npad%right))**2*delp)
                   p(j,i,2) = p(j,i,2)*exp(-(i - (nx-npad%right))**2*delp)
                end do
             end do
          end if
             ! 
             ! update the left-side layer
          if(npad%left.ne.0) then
             delp = -log(npad%taper*0.01)/real(npad%left)**2  ! -ln(%)=-0.0513/xmax**2
             do j=1,nz
                do i=1,npad%left
                   p(j,i,1) = p(j,i,1)*exp(-(i - (1+npad%left))**2*delp)
                   p(j,i,2) = p(j,i,2)*exp(-(i - (1+npad%left))**2*delp)
                end do
             end do
          end if

             ! 
             ! update the bottom layer
          if(npad%bot.ne.0) then
             delp = -log(npad%taper*0.01)/real(npad%bot)**2  ! -ln(%)=-0.0513/xmax**2
             do j=nz-npad%bot+1,nz
                do i=1,nx
                   p(j,i,1) = p(j,i,1)*exp(-(j - (nz-npad%bot))**2*delp)
                   p(j,i,2) = p(j,i,2)*exp(-(j - (nz-npad%bot))**2*delp)
                end do
             end do
          end if
             ! 
             ! update the top layer
          if(npad%top.ne.0) then
             delp = -log(npad%taper*0.01)/real(npad%top)**2  ! -ln(%)=-0.0513/xmax**2
             do j=1,npad%top
                do i=1,nx
                   p(j,i,1) = p(j,i,1)*exp(-(j - (1+npad%top))**2*delp)
                   p(j,i,2) = p(j,i,2)*exp(-(j - (1+npad%top))**2*delp)
                end do
             end do
          end if

             
             !
             ! Apply FD equation for absorbing FD boundary condition, because 
             !   the Laplacian operator includes the stencils outside the data area.
             ! This is related to a paper writted by Robert Clayton and Bjorn Engquist on
             !   absorbing boundary conditions for acoustic and elastic wave equations.
!             if(verb) write(0,*) "apply FD boundary"
             !   
             !update the top FD boundary if the absorbing top boundary is assigned; otherwise,
             !the wavefield in the boundary always remains zero full energy reflections.
             if(abstop) then             
                do j=0,1-nfd_space,-1
                   do i=1,nx
                      delp = vp(1,i)*dt/dz
                      p(j,i,2) = p(j+1,i,1) + (1-delp)/(1+delp)*(p(j,i,1)-p(j+1,i,2)) 
                      p(j,i,1) = p(j,i,2)
                      ! This p(j,i,2) is exactly what will be used in the laplacian operation in next time step.
                   end do
                end do
             end if
             !   
             !update the bottom boundary
             do j=nz+1,nz+nfd_space
                do i=1,nx
                      delp = vp(nz,i)*dt/dz
                      p(j,i,2) = p(j-1,i,1) + (1-delp)/(1+delp)*(p(j,i,1)-p(j-1,i,2)) 
                      p(j,i,1) = p(j,i,2)
                      ! This p(j,i,2) is exactly what will be used in the laplacian operation in next time step.
                end do
             end do

             !   
             !update the left boundary
             do j=1,nz
                do i=0,1-nfd_space,-1
                      delp = vp(j,1)*dt/dx
                      p(j,i,2) = p(j,i+1,1) + (1-delp)/(1+delp)*(p(j,i,1)-p(j,i+1,2)) 
                      p(j,i,1) = p(j,i,2)
                      ! This p(j,i,2) is exactly what will be used in the laplacian operation in next time step.
                end do
             end do
             !   
             !update the right boundary
             do j=1,nz
                do i=nx+1,nx+nfd_space
                      delp = vp(j,nx)*dt/dx
                      p(j,i,2) = p(j,i-1,1) + (1-delp)/(1+delp)*(p(j,i,1)-p(j,i-1,2))
                      p(j,i,1) = p(j,i,2)
                      ! This p(j,i,2) is exactly what will be used in the laplacian operation in next time step.
                end do
             end do

  end subroutine taper


  subroutine smooth2d(vel,nz,nx,nfd_space)
    integer             :: nz,nx,nfd_space, iz,ix,i,j,k
    real, intent(inout) :: vel(nz,nx)
    real     :: vtemp(1-nfd_space:nz+nfd_space,1-nfd_space:nx+nfd_space), rr

    if(nfd_space.le.0) call seperr("nfd_space must be a positive integer")
    do iz=1,nz
       do ix=1,nx
          vtemp(iz,ix) = vel(iz,ix)
       end do
    end do
    do iz=1-nfd_space,0;         vtemp(iz,:) = vtemp(1,:);    end do  ! top bound
    do iz=nz+1,nz+nfd_space;   vtemp(iz,:) = vtemp(nz,:);   end do  ! bottom bound
    do ix=1-nfd_space,0;        vtemp(:,ix) = vtemp(:,1);   end do  ! left bound 
    do ix=nx+1,nx+nfd_space; vtemp(:,ix) = vtemp(:,nx); end do  ! right bound

       do iz=1,nz
          do ix=1,nx
             k=0 ! loop counting index
             rr=0.
             do j=-nfd_space,nfd_space,1
                do i=-nfd_space,nfd_space,1
                   rr = rr + vtemp(iz+j,ix+i)
                   k=k+1  ! loop counting
                end do
             end do
             vel(iz,ix) = rr/real(k)  ! avarage the sum
          end do
       end do
  end subroutine smooth2d


end module wave2d_mod
